@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.c,v 1.14 2005/05/04 17:37:22 graaff Exp $
+ * $Id: geo_ip.c,v 1.25 2005/12/17 10:32:06 cbiere Exp $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -28,11 +28,14 @@
  * @file
  *
  * Support for geographic (country-level) IP mapping.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: geo_ip.c,v 1.14 2005/05/04 17:37:22 graaff Exp $");
+RCSID("$Id: geo_ip.c,v 1.25 2005/12/17 10:32:06 cbiere Exp $");
 
 #include "geo_ip.h"
 #include "settings.h"
@@ -53,18 +56,18 @@ RCSID("$Id: geo_ip.c,v 1.14 2005/05/04 17:37:22 graaff Exp $");
 static const gchar gip_file[] = "geo-ip.txt";
 static const gchar gip_what[] = "Geographic IP mappings";
 
-static gpointer geo_db;		/* The database of bogus CIDR ranges */
+static gpointer geo_db;			/**< The database of bogus CIDR ranges */
 
-/*
+/**
  * Context used during ip_range_split() calls.
  */
 struct range_context {
-	guint32 ip1;				/* Original lower IP in global range */
-	guint32 ip2;				/* Original upper IP in global range */
-	gint country;				/* Country code (numerical encoded) */
-	gint count;					/* Amount of ranges we added, for stats */
-	gchar *line;				/* The line from the input file */
-	gint linenum;				/* Line number in input file, for errors */
+	guint32 ip1;				/**< Original lower IP in global range */
+	guint32 ip2;				/**< Original upper IP in global range */
+	gint country;				/**< Country code (numerical encoded) */
+	gint count;					/**< Amount of ranges we added, for stats */
+	gchar *line;				/**< The line from the input file */
+	gint linenum;				/**< Line number in input file, for errors */
 };
 
 /**
@@ -81,7 +84,7 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 
 	if (dbg > 4)
 		printf("GEO adding %s/%d for \"%s\"\n",
-			ip_to_gchar(ip), bits, ctx->line);
+			ip_to_string(ip), bits, ctx->line);
 
 	ccode = GUINT_TO_POINTER(ctx->country);
 	error = iprange_add_cidr(geo_db, ip, bits, ccode);
@@ -94,13 +97,13 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 		if (error == IPR_ERR_OK) {
 			g_warning("%s, line %d: "
 				"entry \"%s\" (%s/%d) superseded earlier smaller range",
-				gip_file, ctx->linenum, ctx->line, ip_to_gchar(ip), bits);
+				gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits);
 			break;
 		}
 		/* FALL THROUGH */
 	default:
 		g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
-			gip_file, ctx->linenum, ctx->line, ip_to_gchar(ip), bits,
+			gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits,
 			iprange_strerror(error));
 		return;
 	}
@@ -110,7 +113,8 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 
 /**
  * Load geographic IP data from the supplied FILE.
- * Returns the amount of entries loaded.
+ *
+ * @return The amount of entries loaded.
  */
 static gint
 gip_load(FILE *f)
@@ -159,7 +163,7 @@ gip_load(FILE *f)
 			continue;
 		}
 
-		if (!gchar_to_ip_strict(line, &ctx.ip1, NULL)) {
+		if (!string_to_ip_strict(line, &ctx.ip1, NULL)) {
 			g_warning("%s, line %d: invalid first IP in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -177,7 +181,7 @@ gip_load(FILE *f)
 			end++;
 		}
 
-		if (!gchar_to_ip_strict(end, &ctx.ip2, &end)) {
+		if (!string_to_ip_strict(end, &ctx.ip2, &end)) {
 			g_warning("%s, line %d: invalid second IP in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -270,12 +274,14 @@ gip_changed(const gchar *filename, gpointer unused)
 }
 
 /**
- * Loads the geo-ip.txt into memory, choosing the first file we find
- * among the several places we look at, typically:
+ * Loads the geo-ip.txt into memory.
  *
- *    ~/.gtk-gnutella/geo-ip.txt
- *    /usr/share/gtk-gnutella/geo-ip.txt
- *    /home/src/gtk-gnutella/geo-ip.txt
+ * Choosing the first file we find among the several places we look at,
+ * typically:
+ *
+ *		-# ~/.gtk-gnutella/geo-ip.txt
+ *		-# /usr/share/gtk-gnutella/geo-ip.txt
+ *		-# /home/src/gtk-gnutella/geo-ip.txt
  *
  * The selected file will then be monitored and a reloading will occur
  * shortly after a modification.
@@ -295,7 +301,7 @@ gip_retrieve(void)
 	file_path_set(&fp[0], settings_config_dir(), gip_file);
 	file_path_set(&fp[1], PRIVLIB_EXP, gip_file);
 #ifndef OFFICIAL_BUILD
-	file_path_set(&fp[2], PACKAGE_SOURCE_DIR, gip_file);
+	file_path_set(&fp[2], PACKAGE_EXTRA_SOURCE_DIR, gip_file);
 #endif
 
 	f = file_config_open_read_norename_chosen(
@@ -334,18 +340,29 @@ gip_close(void)
 }
 
 /**
- * Retrieves the country an IP address is assigned to.
+ * Retrieves the country an address is assigned to.
  *
+ * @param ha the host address to look up.
  * @return the country mapped to this IP address as an numerical encoded
  * country code, * or -1 when unknown.
  */
 gint
-gip_country(guint32 ip)
+gip_country(const host_addr_t ha)
 {
-	gpointer code;
+	host_addr_t to;
 
-	code = geo_db != NULL ? iprange_get(geo_db, ip) : NULL;
-	return NULL != code ? (GPOINTER_TO_INT(code) >> 1) - 1 : -1;
+	if (
+		host_addr_convert(ha, &to, NET_TYPE_IPV4) ||
+		host_addr_6to4_to_ipv4(ha, &to)
+	) {
+		gpointer code;
+		guint32 ip;
+
+		ip = host_addr_ipv4(to);
+		if (geo_db && NULL != (code = iprange_get(geo_db, ip)))
+			return (GPOINTER_TO_INT(code) >> 1) - 1;
+	}
+	return -1;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
