@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.c,v 1.14 2005/07/28 13:58:43 cbiere Exp $
+ * $Id: ggep.c,v 1.22 2005/09/10 08:17:28 daichik Exp $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,11 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: ggep.c,v 1.14 2005/07/28 13:58:43 cbiere Exp $");
+RCSID("$Id: ggep.c,v 1.22 2005/09/10 08:17:28 daichik Exp $");
 
 #include "ggep.h"
 #include "extensions.h"
 #include "lib/cobs.h"
+#include "lib/misc.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
 
@@ -47,6 +48,47 @@ RCSID("$Id: ggep.c,v 1.14 2005/07/28 13:58:43 cbiere Exp $");
 
 #include "lib/override.h"			/* Must be the last header included */
 
+/**
+ * Check whether a GGEP stream descriptor is valid.
+ */
+gboolean
+ggep_stream_is_valid(ggep_stream_t *gs)
+{
+	if (NULL == gs)
+		return FALSE;
+
+	if (gs->magic != GGEP_MAGIC_ID)
+		return FALSE;
+
+	if (NULL == gs->outbuf)
+		return FALSE;
+
+	if (NULL == gs->end || gs->end <= gs->outbuf)
+		return FALSE;
+
+	if (NULL == gs->o || gs->o < gs->outbuf || gs->o > gs->end)
+		return FALSE;
+
+	if (gs->magic_emitted != TRUE && gs->magic_emitted != FALSE)
+		return FALSE;
+
+	if (gs->begun != TRUE && gs->begun != FALSE)
+		return FALSE;
+
+	if (gs->begun) {
+		if ((gs->flags & GGEP_F_COBS) && !cobs_stream_is_valid(&gs->cs))
+			return FALSE;
+
+		if ((gs->flags & GGEP_F_DEFLATE) && NULL == gs->zd)
+			return FALSE;
+
+		if (!(gs->flags & GGEP_F_DEFLATE) && NULL != gs->zd)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * Initialize a GGEP stream object, capable of receiving multiple GGEP
  * extensions, written into the supplied buffer.
@@ -59,6 +101,9 @@ void
 ggep_stream_init(ggep_stream_t *gs, gpointer data, size_t len)
 {
 	g_assert(len <= INT_MAX);
+	g_assert(len != 0);
+
+	gs->magic = GGEP_MAGIC_ID;
 	gs->outbuf = data;
 	gs->size = len;
 	gs->end = &gs->outbuf[len];
@@ -67,6 +112,8 @@ ggep_stream_init(ggep_stream_t *gs, gpointer data, size_t len)
 	gs->magic_emitted = FALSE;
 	gs->begun = FALSE;
 	gs->zd = NULL;
+
+	g_assert(ggep_stream_is_valid(gs));
 }
 
 /**
@@ -78,7 +125,7 @@ ggep_stream_avail(ggep_stream_t *gs)
 {
 	size_t avail = gs->end - gs->o;
 	g_assert(avail <= gs->size);
-	return avail; 
+	return avail;
 }
 
 /**
@@ -88,6 +135,7 @@ ggep_stream_avail(ggep_stream_t *gs)
 static void
 ggep_stream_cleanup(ggep_stream_t *gs)
 {
+	g_assert(ggep_stream_is_valid(gs));
 	g_assert(gs->begun);
 
 	gs->begun = FALSE;
@@ -104,9 +152,11 @@ ggep_stream_cleanup(ggep_stream_t *gs)
  *
  * @return FALSE if there's not enough room in the output.
  */
-static gboolean
+static inline gboolean
 ggep_stream_appendc(ggep_stream_t *gs, gchar c)
 {
+	g_assert(ggep_stream_is_valid(gs));
+
 	if (0 == ggep_stream_avail(gs))
 		return FALSE;
 
@@ -120,9 +170,11 @@ ggep_stream_appendc(ggep_stream_t *gs, gchar c)
  *
  * @return FALSE if there's not enough room in the output.
  */
-static gboolean
-ggep_stream_append(ggep_stream_t *gs, gpointer data, size_t len)
+static inline gboolean
+ggep_stream_append(ggep_stream_t *gs, gconstpointer data, size_t len)
 {
+	g_assert(ggep_stream_is_valid(gs));
+
 	if (len > ggep_stream_avail(gs))
 		return FALSE;
 
@@ -143,11 +195,12 @@ ggep_stream_append(ggep_stream_t *gs, gpointer data, size_t len)
  * On error, the stream is left in a clean state.
  */
 gboolean
-ggep_stream_begin(ggep_stream_t *gs, gchar *id, guint32 wflags)
+ggep_stream_begin(ggep_stream_t *gs, const gchar *id, guint32 wflags)
 {
 	gint idlen;
 	guint8 flags = 0;
 
+	g_assert(ggep_stream_is_valid(gs));
 	g_assert(gs->outbuf != NULL);		/* Stream not closed */
 
 	/*
@@ -244,8 +297,14 @@ ggep_stream_begin(ggep_stream_t *gs, gchar *id, guint32 wflags)
 	 * All the data we'll now be appending count as payload data.
 	 */
 
+	g_assert(ggep_stream_is_valid(gs));
+
 	gs->begun = TRUE;
 
+	g_assert(!(gs->flags & GGEP_F_COBS) || cobs_stream_is_valid(&gs->cs));
+	g_assert(!(gs->flags & GGEP_F_DEFLATE) || gs->zd != NULL);
+	g_assert((gs->flags & GGEP_F_DEFLATE) || gs->zd == NULL);
+
 	return TRUE;
 
 cleanup:
@@ -259,11 +318,12 @@ cleanup:
  * @return TRUE if OK.  On error, the stream is brought back to a clean state.
  */
 gboolean
-ggep_stream_writev(ggep_stream_t *gs, struct iovec *iov, gint iovcnt)
+ggep_stream_writev(ggep_stream_t *gs, const struct iovec *iov, gint iovcnt)
 {
+	const struct iovec *xiov;
 	gint i;
-	struct iovec *xiov;
 
+	g_assert(ggep_stream_is_valid(gs));
 	g_assert(gs->begun);
 	g_assert(iovcnt >= 0);
 
@@ -317,15 +377,16 @@ cleanup:
  * @return TRUE if OK.  On error, the stream is brought back to a clean state.
  */
 gboolean
-ggep_stream_write(ggep_stream_t *gs, gpointer data, size_t len)
+ggep_stream_write(ggep_stream_t *gs, gconstpointer data, size_t len)
 {
 	struct iovec iov;
+	const struct iovec *p_iov = &iov;
 
 	g_assert(len <= INT_MAX);
-	iov.iov_base = data;
+	iov.iov_base = deconstify_gpointer(data);
 	iov.iov_len = len;
 
-	return ggep_stream_writev(gs, &iov, 1);
+	return ggep_stream_writev(gs, p_iov, 1);
 }
 
 /**
@@ -340,6 +401,7 @@ ggep_stream_end(ggep_stream_t *gs)
 	gint8 hlen[3];
 	size_t slen;
 
+	g_assert(ggep_stream_is_valid(gs));
 	g_assert(gs->begun);
 
 	/*
@@ -368,9 +430,9 @@ ggep_stream_end(ggep_stream_t *gs)
 			gpointer data;
 			gboolean ok;
 
-			if (ggep_debug)
+			if (ggep_debug > 2)
 				g_warning("GGEP \"%.*s\" not compressing %d bytes into %d",
-					*gs->fp & GGEP_F_IDLEN, gs->fp + 1,
+					(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
 					(gint) ilen, (gint) plen);
 
 			data = zlib_uncompress(zlib_deflater_out(gs->zd), plen, ilen);
@@ -401,9 +463,10 @@ ggep_stream_end(ggep_stream_t *gs)
 
 			if (!ok)
 				goto cleanup;
-		} else if (ggep_debug > 1)
+		} else if (ggep_debug > 2)
 			g_warning("GGEP \"%.*s\" compressed %d bytes into %d",
-				*gs->fp & GGEP_F_IDLEN, gs->fp + 1, (gint) ilen, (gint) plen);
+				(gint) (*gs->fp & GGEP_F_IDLEN),
+				gs->fp + 1, (gint) ilen, (gint) plen);
 	}
 
 	/*
@@ -449,9 +512,9 @@ ggep_stream_end(ggep_stream_t *gs)
 			if (NULL == cobs_decode(gs->lp + 1, plen, &ilen, TRUE))
 				goto cleanup;
 
-			if (ggep_debug)
+			if (ggep_debug > 2)
 				g_warning("GGEP \"%.*s\" no need to COBS %d bytes into %d",
-					*gs->fp & GGEP_F_IDLEN, gs->fp + 1,
+					(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
 					(gint) ilen, (gint) plen);
 
 			/*
@@ -470,9 +533,9 @@ ggep_stream_end(ggep_stream_t *gs)
 			gs->o = &gs->lp[1 + plen];	/* +1 to skip NUL "length" byte */
 			/* No overwriting */
 			g_assert((size_t) (gs->end - gs->o) <= gs->size);
-		} else if (ggep_debug > 1)
+		} else if (ggep_debug > 2)
 			g_message("GGEP \"%.*s\" COBS-ed into %d bytes",
-				*gs->fp & GGEP_F_IDLEN, gs->fp + 1, (gint) plen);
+				(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1, (gint) plen);
 	}
 
 	/*
@@ -490,9 +553,9 @@ ggep_stream_end(ggep_stream_t *gs)
 	 * stored at the beginning.  We only reserved 1 byte for the length.
 	 */
 
-	if (ggep_debug > 2)
+	if (ggep_debug > 3)
 		g_message("GGEP \"%.*s\" payload holds %d byte%s",
-			*gs->fp & GGEP_F_IDLEN, gs->fp + 1,
+			(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
 			(gint) plen, plen == 1 ? "" : "s");
 
 	if (plen <= 63) {
@@ -509,7 +572,7 @@ ggep_stream_end(ggep_stream_t *gs)
 		hlen[2] = GGEP_L_LAST | (plen & GGEP_L_VALUE);
 	} else {
 		g_warning("too large GGEP payload length (%d bytes) for \"%.*s\"",
-			(gint) plen, *gs->fp & GGEP_F_IDLEN, gs->fp + 1);
+			(gint) plen, (gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1);
 		goto cleanup;
 	}
 
@@ -589,10 +652,10 @@ ggep_stream_close(ggep_stream_t *gs)
  */
 gboolean
 ggep_stream_packv(ggep_stream_t *gs,
-	gchar *id, struct iovec *iov, gint iovcnt, guint32 wflags)
+	const gchar *id, const struct iovec *iov, gint iovcnt, guint32 wflags)
 {
 	g_assert(iovcnt >= 0);
-	
+
 	if (!ggep_stream_begin(gs, id, wflags))
 		return FALSE;
 
@@ -624,15 +687,19 @@ ggep_stream_packv(ggep_stream_t *gs,
  */
 gboolean
 ggep_stream_pack(ggep_stream_t *gs,
-	gchar *id, gchar *payload, size_t plen, guint32 wflags)
+	const gchar *id, gconstpointer payload, size_t plen, guint32 wflags)
 {
 	struct iovec iov;
+	const struct iovec *p_iov = &iov;
 
+	g_assert(id);
+	g_assert(0 == plen || NULL != payload);
 	g_assert(plen <= INT_MAX);
-	iov.iov_base = payload;
+
+	iov.iov_base = deconstify_gpointer(payload);
 	iov.iov_len = plen;
 
-	return ggep_stream_packv(gs, id, &iov, 1, wflags);
+	return ggep_stream_packv(gs, id, p_iov, 1, wflags);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
