@@ -1,10 +1,8 @@
 /*
- * $Id: ggep_type.h,v 1.5 2005/04/24 18:50:59 cbiere Exp $
+ * $Id: ggep_type.h,v 1.10 2005/08/17 13:44:05 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * GGEP type-specific routines.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,20 +23,31 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * GGEP type-specific routines.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_ggep_type_h_
 #define _core_ggep_type_h_
 
 #include <glib.h>
+#include "lib/host_addr.h"
 
-/*
+/**
  * Extraction interface return types.
  */
 
 typedef enum ggept_status {
-	GGEP_OK = 0,					/* OK, extracted what was asked */
-	GGEP_NOT_FOUND = 1,				/* OK, but did not find it */
-	GGEP_INVALID = 2,				/* Error, found something invalid */
-	GGEP_BAD_SIZE = 3				/* Error, buffer not correctly sized */
+	GGEP_OK = 0,					/**< OK, extracted what was asked */
+	GGEP_NOT_FOUND = 1,				/**< OK, but did not find it */
+	GGEP_INVALID = 2,				/**< Error, found something invalid */
+	GGEP_BAD_SIZE = 3				/**< Error, buffer not correctly sized */
 } ggept_status_t;
 
 /*
@@ -49,7 +58,8 @@ struct gnutella_host;
 
 ggept_status_t ggept_h_sha1_extract(extvec_t *exv, gchar *buf, gint len);
 
-struct ggep_gtkgv1 {				/* Decompiled payload of "GTKGV1" */
+/** Decompiled payload of "GTKGV1" */
+struct ggep_gtkgv1 {
 	guint8 major;
 	guint8 minor;
 	guint8 patch;
@@ -62,6 +72,7 @@ ggept_status_t ggept_gtkgv1_extract(extvec_t *exv, struct ggep_gtkgv1 *info);
 ggept_status_t ggept_hname_extract(extvec_t *exv, gchar *buf, gint len);
 ggept_status_t ggept_lf_extract(extvec_t *exv, guint64 *fs);
 ggept_status_t ggept_du_extract(extvec_t *exv, guint32 *uptime);
+ggept_status_t ggept_gtkg_ipv6_extract(extvec_t *exv, host_addr_t *addr);
 
 ggept_status_t ggept_alt_extract(extvec_t *exv,
 	struct gnutella_host **hvec, gint *hvcnt);
