@@ -1,10 +1,8 @@
 /*
- * $Id: gwcache.c,v 1.27 2005/05/10 23:26:37 cbiere Exp $
+ * $Id: gwcache.c,v 1.38 2005/09/11 13:39:57 rmanfredi Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
- * Gnutella Web Cache.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Gnutella Web Cache.
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: gwcache.c,v 1.27 2005/05/10 23:26:37 cbiere Exp $");
+RCSID("$Id: gwcache.c,v 1.38 2005/09/11 13:39:57 rmanfredi Exp $");
 
 #include "gwcache.h"
 #include "http.h"
@@ -42,6 +50,7 @@ RCSID("$Id: gwcache.c,v 1.27 2005/05/10 23:26:37 cbiere Exp $");
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/walloc.h"
 
@@ -65,13 +74,13 @@ static gchar gwc_tmp[1024];
  * to prevent insertion of duplicates in our cache.
  */
 
-#define MAX_GWC_URLS	200					/* Max URLs we store */
-#define MAX_GWC_REUSE	1					/* Max amount of uses for one URL */
+#define MAX_GWC_URLS	200		/**< Max URLs we store */
+#define MAX_GWC_REUSE	1		/**< Max amount of uses for one URL */
 
 struct gwc {
-	gchar *url;			/* atom */
-	time_t stamp;		/* time of last access */	
-} gwc_url[MAX_GWC_URLS];		/* Holds string atoms */
+	gchar *url;					/**< atom */
+	time_t stamp;				/**< time of last access */
+} gwc_url[MAX_GWC_URLS];		/**< Holds string atoms */
 
 static gint gwc_url_slot = -1;
 static GHashTable *gwc_known_url = NULL;
@@ -104,18 +113,18 @@ static GHashTable *gwc_failed_url = NULL;
  * . All requests include "client=GTKG" and "version" information.
  */
 
-static gchar *current_url = NULL;			/* Cache we're currently using */
-static gint current_reused = 0;				/* Amount of times we reused it */
+static gchar *current_url = NULL;			/**< Cache we're currently using */
+static gint current_reused = 0;				/**< Amount of times we reused it */
 
-#define MAX_URL_LINES	50					/* Max lines on a urlfile req */
-#define MAX_IP_LINES	150					/* Max lines on a hostfile req */
-#define MAX_OK_LINES	3					/* Max lines when expecting OK */
-#define MIN_IP_LINES	5					/* Min lines expected */
-#define MIN_URL_LINES	5					/* Min lines expected */
-#define HOUR_MS			(3600 * 1000)		/* Callout queue time in ms */
-#define URL_RETRY_MS	(20 * 1000)			/* Retry timer for urlfile, in ms */
-#define REFRESH_MS		(8 * HOUR_MS)		/* Refresh every 8 hours */
-#define REUSE_PERIOD	3600				/* Period between GET hostfile */
+#define MAX_URL_LINES	50					/**< Max lines on a urlfile req */
+#define MAX_IP_LINES	150					/**< Max lines on a hostfile req */
+#define MAX_OK_LINES	3					/**< Max lines when expecting OK */
+#define MIN_IP_LINES	5					/**< Min lines expected */
+#define MIN_URL_LINES	5					/**< Min lines expected */
+#define HOUR_MS			(3600 * 1000)		/**< Callout queue time in ms */
+#define URL_RETRY_MS	(20 * 1000)			/**< Retry timer for urlfile, in ms */
+#define REFRESH_MS		(8 * HOUR_MS)		/**< Refresh every 8 hours */
+#define REUSE_PERIOD	3600				/**< Period between GET hostfile */
 
 #define CLIENT_INFO "client=GTKG&version=" GTA_VERSION_NUMBER
 
@@ -132,17 +141,14 @@ static void gwc_get_urls(void);
 static void gwc_update_ip_url(void);
 static void gwc_seed_cache(gchar *cache_url);
 
-/*
+/**
  * The following URLs are there for bootstrapping purposes only.
  */
 
 static const gchar * const boot_url[] = {
 	"http://cache.kicks-ass.net:8000/",
-	"http://gwc.nonamer.ath.cx:8080/",
-	"http://kisama.ath.cx:8080/",
+	"http://galvatron.dyndns.org:59009/gwcache",
 	"http://krill.shacknet.nu:20095/gwc",
-	"http://overbeer.ghostwhitecrab.de/",
-	"http://starscream.dynalias.com/",
 };
 
 /**
@@ -235,7 +241,7 @@ gwc_pick(void)
 	gint count = g_hash_table_size(gwc_known_url);
 	gint idx, i;
 	gchar *url = NULL;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 
 	if (0 == count)
 		return NULL;
@@ -247,7 +253,7 @@ gwc_pick(void)
 	idx = random_value(count - 1);
 	for (i = 0; i < count; i++) {
 		time_t stamp = gwc_url[idx].stamp;
-	
+
 		if (0 == stamp || delta_time(now, stamp) > 3900) {
 			url = gwc_url[idx].url;
 			gwc_url[idx].stamp = now;
@@ -292,7 +298,7 @@ gwc_store(void)
 
 	for (j = 0; j < MAX_GWC_URLS; j++) {
 		gchar *url = gwc_url[i].url;
-		
+
 		i = (i + 1) % MAX_GWC_URLS;
 		if (url == NULL)
 			continue;
@@ -329,7 +335,7 @@ gwc_retrieve(void)
 	FILE *in;
 	gchar tmp[1024];
 
-	file_path_set(&fp[0], settings_config_dir(), gwc_bootfile);
+	file_path_set(&fp[0], settings_config_dir(), gwc_file);
 	file_path_set(&fp[1], PRIVLIB_EXP, gwc_bootfile);
 #ifndef OFFICIAL_BUILD
 	file_path_set(&fp[2], PACKAGE_SOURCE_DIR, gwc_bootfile);
@@ -622,11 +628,11 @@ gwc_close(void)
  ***/
 
 struct parse_context {
-	getline_t *getline;			/* Used to hold partially read line */
-	gpointer handle;			/* Request handle */
-	gint maxlines;				/* Maximum number of lines we want to process */
-	gint lines;					/* Amount of lines so far */
-	gint processed;				/* User callback can count retained lines */
+	getline_t *getline;			/**< Used to hold partially read line */
+	gpointer handle;			/**< Request handle */
+	gint maxlines;				/**< Maximum number of lines we want to process */
+	gint lines;					/**< Amount of lines so far */
+	gint processed;				/**< User callback can count retained lines */
 };
 
 typedef gboolean (parse_dispatch_t)
@@ -648,8 +654,8 @@ parse_context_free(gpointer obj)
 /**
  * Allocate new parsing context for handle and record it.
  *
- * `handle' is the asynchronous HTTP request handle.
- * `maxlines' is the max number of lines we want to parse.
+ * @param `handle'		the asynchronous HTTP request handle.
+ * @param `maxlines'	the max number of lines we want to parse.
  */
 static void
 parse_context_set(gpointer handle, gint maxlines)
@@ -763,7 +769,7 @@ static gboolean
 gwc_url_line(struct parse_context *ctx, gchar *buf, gint len)
 {
 	if (gwc_debug > 3)
-		printf("GWC URL line (%d bytes): %s\n", len, buf);
+		g_message("GWC URL line (%d bytes): %s", len, buf);
 
 	if (is_strprefix(buf, "ERROR")) {
 		g_warning("GWC cache \"%s\" returned %s",
@@ -791,7 +797,7 @@ static void
 gwc_url_eof(struct parse_context *ctx)
 {
 	if (gwc_debug > 2)
-		printf("GWC URL all done (%d/%d lines processed)\n",
+		g_message("GWC URL all done (%d/%d lines processed)",
 			ctx->processed, ctx->lines);
 
 	if (ctx->processed < MIN_URL_LINES) {
@@ -824,7 +830,7 @@ gwc_url_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_url_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, gwc_debug);
 
 	clear_current_url(TRUE);		/* This webcache is not good */
 
@@ -858,7 +864,7 @@ gwc_get_urls(void)
 		"%s?urlfile=1&%s", current_url, CLIENT_INFO);
 
 	if (gwc_debug > 2)
-		printf("GWC URL request: %s\n", gwc_tmp);
+		g_message("GWC URL request: %s", gwc_tmp);
 
 	/*
 	 * Launch the asynchronous request and attach parsing information.
@@ -900,8 +906,8 @@ gwc_is_waiting(void)
 static gboolean
 gwc_host_line(struct parse_context *ctx, gchar *buf, gint len)
 {
-	if (gwc_debug > 3)
-		printf("GWC host line (%d bytes): %s\n", len, buf);
+	if (gwc_debug > 3 || bootstrap_debug > 2)
+		g_message("BOOT GWC host line (%d bytes): %s", len, buf);
 
 	if (is_strprefix(buf, "ERROR")) {
 		g_warning("GWC cache \"%s\" returned %s",
@@ -911,12 +917,17 @@ gwc_host_line(struct parse_context *ctx, gchar *buf, gint len)
 	}
 
 	if (len) {
-		guint32 ip;
+		host_addr_t addr;
 		guint16 port;
 
-		if (gchar_to_ip_port(buf, &ip, &port)) {
+		if (string_to_host_addr_port(buf, NULL, &addr, &port)) {
 			ctx->processed++;
-			hcache_add_caught(HOST_ULTRA, ip, port, "GWC");
+			hcache_add_caught(HOST_ULTRA, addr, port, "GWC");
+
+			if (bootstrap_debug > 1)
+				g_message("BOOT collected %s from GWC %s",
+					host_addr_to_string(addr),
+					http_async_info(ctx->handle, NULL, NULL, NULL, NULL));
 		}
 	}
 
@@ -930,7 +941,7 @@ static void
 gwc_host_eof(struct parse_context *ctx)
 {
 	if (gwc_debug > 2)
-		printf("GWC host all done (%d/%d lines processed)\n",
+		g_message("GWC host all done (%d/%d lines processed)",
 			ctx->processed, ctx->lines);
 
 	/*
@@ -943,6 +954,10 @@ gwc_host_eof(struct parse_context *ctx)
 
 	gcu_statusbar_message(gwc_tmp);
 
+	if (bootstrap_debug)
+		g_message("BOOT got %d host%s from GWC %s",
+			ctx->processed, ctx->processed == 1 ? "" : "s", current_url);
+
 	/*
 	 * If we did not get enough addresses, try to feed the cache with ours.
 	 */
@@ -970,7 +985,7 @@ gwc_host_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_host_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, MAX(gwc_debug, bootstrap_debug));
 	hostfile_running = FALSE;
 
 	clear_current_url(TRUE);			/* This webcache is not good */
@@ -984,7 +999,7 @@ gwc_get_hosts(void)
 {
 	gpointer handle;
 	static time_t last_called = 0;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 
 	/*
 	 * Make sure we don't probe more than one webcache at a time.
@@ -1025,6 +1040,9 @@ gwc_get_hosts(void)
 
 	gcu_statusbar_message(gwc_tmp);
 
+	if (bootstrap_debug)
+		g_message("BOOT connecting to web cache %s", current_url);
+
 	/*
 	 * Launch the asynchronous request and attach parsing information.
 	 */
@@ -1095,7 +1113,7 @@ gwc_update_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_update_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, gwc_debug);
 	clear_current_url(TRUE);			/* This webcache is not good */
 }
 
@@ -1154,10 +1172,10 @@ gwc_update_this(gchar *cache_url)
 	if (
 		!is_firewalled &&
 		current_peermode == NODE_P_ULTRA &&
-		host_is_valid(listen_ip(), listen_port)
+		host_is_valid(listen_addr(), listen_port)
 	) {
 		rw += gm_snprintf(&gwc_tmp[rw], sizeof(gwc_tmp)-rw,
-			"ip=%s&", ip_port_to_gchar(listen_ip(), listen_port));
+			"ip=%s&", host_addr_port_to_string(listen_addr(), listen_port));
 		has_data = TRUE;
 	}
 
@@ -1219,7 +1237,7 @@ gwc_seed_cache(gchar *cache_url)
 		return;
 
 	if (gwc_debug > 2)
-		printf("GWC seeding cache \"%s\"\n", cache_url);
+		g_message("GWC seeding cache \"%s\"", cache_url);
 
 	gwc_update_this(cache_url);
 }
