@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.c,v 1.10 2005/05/04 17:37:22 graaff Exp $
+ * $Id: hosts.c,v 1.21 2005/09/11 13:39:57 rmanfredi Exp $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -28,16 +28,19 @@
  * @file
  *
  * Host management.
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2004
  */
 
 #include "common.h"
 
-RCSID("$Id: hosts.c,v 1.10 2005/05/04 17:37:22 graaff Exp $");
+RCSID("$Id: hosts.c,v 1.21 2005/09/11 13:39:57 rmanfredi Exp $");
 
 #include "sockets.h"
 #include "hosts.h"
 #include "nodes.h"
-#include "share.h"		/* For files_scanned and kbytes_scanned. */
+#include "share.h"			/* For files_scanned and kbytes_scanned. */
 #include "routing.h"
 #include "gmsg.h"
 #include "pcache.h"
@@ -50,9 +53,11 @@ RCSID("$Id: hosts.c,v 1.10 2005/05/04 17:37:22 graaff Exp $");
 #include "if/gnet_property_priv.h"
 
 #include "lib/endian.h"
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/glib-missing.h"
+#include "lib/misc.h"
+#include "lib/override.h"	/* Must be the last header included */
 
-gboolean host_low_on_pongs = FALSE;			/* True when less than 12% full */
+gboolean host_low_on_pongs = FALSE;			/**< True when less than 12% full */
 
 static gboolean in_shutdown = FALSE;
 
@@ -66,29 +71,28 @@ static gboolean in_shutdown = FALSE;
 guint
 host_hash(gconstpointer key)
 {
-	const gnet_host_t *host = (const gnet_host_t *) key;
+	const gnet_host_t *host = key;
 
-	return (guint) (host->ip ^ ((host->port << 16) | host->port));
+	return host_addr_hash(host->addr) ^ ((host->port << 16) | host->port);
 }
 
 /**
  * Compare function which returns TRUE if the hosts are equal.
  *
- * @note For use in g_hash_table_new
+ * @note For use in g_hash_table_new.
  */
 gint
 host_eq(gconstpointer v1, gconstpointer v2)
 {
-	const gnet_host_t *h1 = (const gnet_host_t *) v1;
-	const gnet_host_t *h2 = (const gnet_host_t *) v2;
+	const gnet_host_t *h1 = v1, *h2 = v2;
 
-	return h1->ip == h2->ip && h1->port == h2->port;
+	return host_addr_equal(h1->addr, h2->addr) && h1->port == h2->port;
 }
 
 /**
  * Compare function which returns 0 if the hosts are equal, otherwise 1.
  *
- * @note For use in g_list_find_custom
+ * @note For use in g_list_find_custom.
  */
 gint
 host_cmp(gconstpointer v1, gconstpointer v2)
@@ -110,7 +114,7 @@ host_timer(void)
 	static gint called = 0;
     guint count;
 	gint missing;
-	guint32 ip;
+	host_addr_t addr;
 	guint16 port;
 	host_type_t htype;
 	guint max_nodes;
@@ -201,8 +205,8 @@ host_timer(void)
             missing = to_add;
 
 			while (hcache_size(htype) && missing-- > 0) {
-				hcache_get_caught(htype, &ip, &port);
-				node_add(ip, port);
+				hcache_get_caught(htype, &addr, &port);
+				node_add(addr, port, 0);
 			}
 
 			if (missing > 0 && hcache_read_finished()) {
@@ -213,19 +217,19 @@ host_timer(void)
 						gwc_get_hosts(); 	/* Get from web host cache */
 					else
 						uhc_get_hosts();	/* Get from UDP pong caches */
-				} else if (dbg > 10)
-					g_message("host_timer - waiting for reply from %s",
+				} else if (bootstrap_debug > 2)
+					g_message("BOOT host_timer - waiting for reply from %s",
 						uhc_is_waiting() ? "UDP host cache" : "web cache");
 			}
 		}
 
 	} else if (use_netmasks) {
 		/* Try to find better hosts */
-		if (hcache_find_nearby(htype, &ip, &port)) {
+		if (hcache_find_nearby(htype, &addr, &port)) {
 			if (node_remove_worst(TRUE))
-				node_add(ip, port);
+				node_add(addr, port, 0);
 			else
-				hcache_add_caught(htype, ip, port, "nearby host");
+				hcache_add_caught(htype, addr, port, "nearby host");
 		}
 	}
 }
@@ -240,19 +244,37 @@ void host_init(void)
 }
 
 /**
- * Check whether host is connectible, i.e. that it has a valid port and that
- * its IP address is not private not bogus.
+ * @return the address:port of a host
+ */
+const gchar *
+host_to_string(const gnet_host_t *h)
+{
+	static gchar buf[64];
+	gchar host[48];
+
+	g_assert(h != NULL);
+
+	host_addr_to_string_buf(h->addr, host, sizeof host);
+	gm_snprintf(buf, sizeof buf, "%s:%u", host, h->port);
+	return buf;
+}
+
+/**
+ * Check whether host is connectible.
+ *
+ * i.e. that it has a valid port and that its IP address is not private
+ * not bogus.
  */
 gboolean
-host_is_valid(guint32 ip, guint16 port)
+host_is_valid(const host_addr_t addr, guint16 port)
 {
 	if (!port_is_valid(port))
 		return FALSE;
 
-	if (!ip_is_valid(ip))
+	if (!host_addr_is_routable(addr))
 		return FALSE;
 
-	if (bogons_check(ip))
+	if (bogons_check(addr))
 		return FALSE;
 
 	return TRUE;
@@ -260,12 +282,13 @@ host_is_valid(guint32 ip, guint16 port)
 
 /**
  * Add a new host to our pong reserve.
+ *
  * When `connect' is true, attempt to connect if we are low in Gnet links.
  */
 void
-host_add(guint32 ip, guint16 port, gboolean do_connect)
+host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 {
-	if (!hcache_add_caught(HOST_ANY, ip, port, "pong"))
+	if (!hcache_add_caught(HOST_ANY, addr, port, "pong"))
 		return;
 
 	/*
@@ -283,13 +306,13 @@ host_add(guint32 ip, guint16 port, gboolean do_connect)
 
 	if (do_connect) {
 		if (node_keep_missing() > 0)
-				node_add(ip, port);
+				node_add(addr, port, 0);
 		else {
 			/* If we are above the max connections, delete a non-nearby
 			 * connection before adding this better one
 			 */
-			if (use_netmasks && host_is_nearby(ip) && node_remove_worst(TRUE))
-				node_add(ip, port);
+			if (use_netmasks && host_is_nearby(addr) && node_remove_worst(TRUE))
+				node_add(addr, port, 0);
 		}
 
 	}
@@ -301,18 +324,18 @@ host_add(guint32 ip, guint16 port, gboolean do_connect)
  * may be unsuitable for Gnet connections.
  */
 void
-host_add_semi_pong(guint32 ip, guint16 port)
+host_add_semi_pong(const host_addr_t addr, guint16 port)
 {
 	g_assert(host_low_on_pongs);	/* Only used when low on pongs */
 
-    hcache_add_caught(HOST_ANY, ip, port, "semi-pong");
+    hcache_add_caught(HOST_ANY, addr, port, "semi-pong");
 }
 
 /* ---------- Netmask heuristic by Mike Perry -------- */
 struct network_pair
 {
-	struct in_addr mask;
-	struct in_addr net;
+	guint32 mask;
+	guint32 net;
 };
 
 struct network_pair *local_networks = NULL;
@@ -333,7 +356,7 @@ void free_networks(void)
  * the local_networks array. IP's are in network order.
  */
 void
-parse_netmasks(gchar * str)
+parse_netmasks(const gchar *str)
 {
 	gchar **masks = g_strsplit(str, ";", 0);
 	gchar *p;
@@ -355,56 +378,60 @@ parse_netmasks(gchar * str)
 		return;
     }
 
-	local_networks =
-		(struct network_pair *) g_malloc(i * sizeof(*local_networks));
+	local_networks = g_malloc(i * sizeof *local_networks);
 
 	for (i = 0; masks[i]; i++) {
 		/* Network is of the form ip/mask or ip/bits */
 		if ((p = strchr(masks[i], '/')) && *p) {
-			*p = 0;
-			p++;
+			*p++ = '\0';
+
 			if (strchr(p, '.')) {
 				/* get the network address from the user */
-				if (inet_aton(p, &local_networks[i].mask) == 0)
-					perror("inet_aton on netmasks");
+				if (!string_to_ip_strict(p, &local_networks[i].mask, NULL))
+					g_warning("parse_netmasks(): Invalid netmask: \"%s\"", p);
 			}
 			else {
-				errno = 0;
-				mask_div = strtol(p, NULL, 10);
-				if (mask_div > 32) {
-					mask_div = 32;
-				}
-				if (errno)
-					perror("netmask_div");
+				gint error;
+
+				mask_div = parse_uint32(p, NULL, 10, &error);
+				mask_div = MIN(32, mask_div);
+				if (error)
+					g_warning("parse_netmasks(): "
+						"Invalid CIDR prefixlen: \"%s\"", p);
 				else
-					WRITE_GUINT32_BE(~((1 << (32 - mask_div)) - 1),
-						&local_networks[i].mask.s_addr);
+					local_networks[i].mask = (guint32) -1 << (32 - mask_div);
 			}
 		}
 		else {
 			/* Assume single-host */
-			inet_aton("255.255.255.255", &local_networks[i].mask);
+			local_networks[i].mask = -1; /* 255.255.255.255 */
 		}
 		/* get the network address from the user */
-		if (inet_aton(masks[i], &local_networks[i].net) == 0)
-			perror("inet_aton on netmasks");
+		if (!string_to_ip_strict(masks[i], &local_networks[i].net, NULL))
+			g_warning("parse_netmasks(): Invalid netmask: \"%s\"", masks[i]);
 	}
 
 	g_strfreev(masks);
 }
 
 /**
- * Returns true if the ip is inside one of the local networks
+ * @returns true if the address is inside one of the local networks
  */
 gboolean
-host_is_nearby(guint32 ip)
+host_is_nearby(const host_addr_t addr)
 {
 	guint i;
 
-	for (i = 0; i < number_local_networks; i++) {
-		if ((ip & local_networks[i].mask.s_addr) ==
-				(local_networks[i].net.s_addr & local_networks[i].mask.s_addr))
-			return TRUE;
+	if (NET_TYPE_IPV4 == host_addr_net(addr)) {
+		for (i = 0; i < number_local_networks; i++) {
+			guint32 m_mask = local_networks[i].mask;
+			guint32 m_ip = local_networks[i].net;
+
+			if ((host_addr_ipv4(addr) & m_mask) == (m_ip & m_mask))
+				return TRUE;
+		}
+	} else if (NET_TYPE_IPV6 == host_addr_net(addr)) {
+		/* XXX: Implement this! */
 	}
 	return FALSE;
 }
