@@ -1,5 +1,5 @@
 /*
- * $Id: http.h,v 1.11 2005/07/01 09:35:35 daichik Exp $
+ * $Id: http.h,v 1.18 2005/11/09 19:16:53 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,6 +37,7 @@
 #define _core_http_h_
 
 #include "if/core/http.h"
+#include "lib/host_addr.h"
 
 #define HTTP_PORT		80		/**< Registered HTTP port */
 
@@ -46,7 +47,8 @@
 
 typedef enum {
 	HTTP_EXTRA_LINE,
-	HTTP_EXTRA_CALLBACK
+	HTTP_EXTRA_CALLBACK,
+	HTTP_EXTRA_BODY
 } http_extra_type_t;
 
 /**
@@ -162,20 +164,21 @@ extern guint http_async_errno;
  */
 
 typedef enum {
-	HTTP_URL_OK = 0,					/**< All OK */
-	HTTP_URL_NOT_HTTP,					/**< Not an http URI */
-	HTTP_URL_MULTIPLE_CREDENTIALS,		/**< More than one "<user>:<password>" */
-	HTTP_URL_BAD_CREDENTIALS,			/**< Truncated "<user>:<password>" */
-	HTTP_URL_BAD_PORT_PARSING,			/**< Could not parse port */
-	HTTP_URL_BAD_PORT_RANGE,			/**< Port value is out of range */
-	HTTP_URL_HOSTNAME_UNKNOWN,			/**< Could not resolve host into IP */
-	HTTP_URL_MISSING_URI				/**< URL has no URI part */
+	HTTP_URL_OK = 0,				/**< All OK */
+	HTTP_URL_NOT_HTTP,				/**< Not an http URI */
+	HTTP_URL_MULTIPLE_CREDENTIALS,	/**< More than one "<user>:<password>" */
+	HTTP_URL_BAD_CREDENTIALS,		/**< Truncated "<user>:<password>" */
+	HTTP_URL_BAD_PORT_PARSING,		/**< Could not parse port */
+	HTTP_URL_BAD_PORT_RANGE,		/**< Port value is out of range */
+	HTTP_URL_BAD_HOST_PART,			/**< Could not parse host */
+	HTTP_URL_HOSTNAME_UNKNOWN,		/**< Could not resolve host into IP */
+	HTTP_URL_MISSING_URI			/**< URL has no URI part */
 } http_url_error_t;
 
 extern http_url_error_t http_url_errno;
 
 /**
- * Callabck to notify about state changes in HTTP request.
+ * Callback to notify about state changes in HTTP request.
  */
 
 typedef void (*http_state_change_t)(gpointer handle, http_state_t newstate);
@@ -217,7 +220,7 @@ gint http_status_parse(const gchar *line,
 	const gchar *proto, const gchar **msg, guint *major, guint *minor);
 
 gboolean http_extract_version(
-	gchar *request, gint len, guint *major, guint *minor);
+	const gchar *request, gint len, guint *major, guint *minor);
 
 http_buffer_t *http_buffer_alloc(gchar *buf, size_t len, size_t written);
 void http_buffer_free(http_buffer_t *b);
@@ -228,13 +231,13 @@ http_content_range_parse(const gchar *buf,
 
 filesize_t http_range_size(const GSList *list);
 void http_range_free(GSList *list);
-GSList *http_range_parse(
-	const gchar *field, gchar *value, filesize_t size, const gchar *vendor);
+GSList *http_range_parse(const gchar *field, const gchar *value,
+		filesize_t size, const gchar *vendor);
 gboolean http_range_contains(GSList *ranges, filesize_t from, filesize_t to);
 
 const gchar *http_url_strerror(http_url_error_t errnum);
 gboolean http_url_parse(
-	const gchar *url, guint16 *port, gchar **host, gchar **path);
+	const gchar *url, guint16 *port, const gchar **host, const gchar **path);
 
 gpointer http_async_get(
 	gchar *url,
@@ -242,9 +245,9 @@ gpointer http_async_get(
 	http_data_cb_t data_ind,
 	http_error_cb_t error_ind);
 
-gpointer http_async_get_ip(
+gpointer http_async_get_addr(
 	gchar *path,
-	guint32 ip,
+	const host_addr_t,
 	guint16 port,
 	http_header_cb_t header_ind,
 	http_data_cb_t data_ind,
@@ -253,7 +256,7 @@ gpointer http_async_get_ip(
 const gchar *http_async_strerror(guint errnum);
 const gchar *http_async_info(
 	gpointer handle, const gchar **req, gchar **path,
-	guint32 *ip, guint16 *port);
+	host_addr_t *addr, guint16 *port);
 void http_async_connected(gpointer handle);
 void http_async_close(gpointer handle);
 void http_async_cancel(gpointer handle);
@@ -263,6 +266,8 @@ http_state_t http_async_state(gpointer handle);
 void http_async_set_opaque(gpointer handle, gpointer data, http_user_free_t fn);
 gpointer http_async_get_opaque(gpointer handle);
 void http_async_log_error(gpointer handle, http_errtype_t type, gpointer v);
+void http_async_log_error_dbg(
+	gpointer handle, http_errtype_t type, gpointer v, guint32 dbg_level);
 
 void http_async_on_state_change(gpointer handle, http_state_change_t fn);
 void http_async_allow_redirects(gpointer handle, gboolean allow);
