@@ -1,10 +1,8 @@
 /*
- * $Id: pproxy.c,v 1.14 2005/04/27 17:35:11 cbiere Exp $
+ * $Id: pproxy.c,v 1.27 2005/10/21 17:39:06 cbiere Exp $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * Push proxy HTTP management.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Push proxy HTTP management.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: pproxy.c,v 1.14 2005/04/27 17:35:11 cbiere Exp $");
+RCSID("$Id: pproxy.c,v 1.27 2005/10/21 17:39:06 cbiere Exp $");
 
 #include "pproxy.h"
 #include "http.h"
@@ -42,12 +50,14 @@ RCSID("$Id: pproxy.c,v 1.14 2005/04/27 17:35:11 cbiere Exp $");
 #include "routing.h"
 #include "gmsg.h"
 #include "uploads.h"
+#include "ggep.h"
+#include "ggep_type.h"
 #include "gnet_stats.h"
 #include "lib/url.h"
 
 /* Following extra needed for the client-side */
 
-#include "settings.h"		/* For listen_ip() */
+#include "settings.h"			/* For listen_addr() */
 #include "token.h"
 #include "downloads.h"
 
@@ -57,6 +67,7 @@ RCSID("$Id: pproxy.c,v 1.14 2005/04/27 17:35:11 cbiere Exp $");
 #include "lib/header.h"
 #include "lib/glib-missing.h"
 #include "lib/endian.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -67,7 +78,7 @@ RCSID("$Id: pproxy.c,v 1.14 2005/04/27 17:35:11 cbiere Exp $");
  *** Server-side of push-proxy
  ***/
 
-static GSList *pproxies = NULL;	/* Currently active push-proxy requests */
+static GSList *pproxies = NULL;	/**< Currently active push-proxy requests */
 
 static void send_pproxy_error(struct pproxy *pp, int code,
 	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
@@ -116,14 +127,15 @@ send_pproxy_error_v(
 	gint hevcnt = 0;
 
 	if (msg) {
-		gm_vsnprintf(reason, sizeof(reason), msg, ap);
+		gm_vsnprintf(reason, sizeof reason, msg, ap);
 	} else
 		reason[0] = '\0';
 
 	if (pp->error_sent) {
 		g_warning("push-proxy: already sent an error %d to %s, "
 			"not sending %d (%s)",
-			pp->error_sent, ip_to_gchar(pp->socket->ip), code, reason);
+			pp->error_sent,host_addr_to_string(pp->socket->addr),
+			code, reason);
 		return;
 	}
 
@@ -134,9 +146,9 @@ send_pproxy_error_v(
 	 */
 
 	if (ext) {
-		size_t slen = g_strlcpy(extra, ext, sizeof(extra));
+		size_t slen = g_strlcpy(extra, ext, sizeof extra);
 
-		if (slen < sizeof(extra)) {
+		if (slen < sizeof extra) {
 			hev[hevcnt].he_type = HTTP_EXTRA_LINE;
 			hev[hevcnt++].he_msg = extra;
 		} else
@@ -176,11 +188,11 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 	g_assert(pp != NULL);
 
 	if (reason) {
-		gm_vsnprintf(errbuf, sizeof(errbuf), reason, ap);
+		gm_vsnprintf(errbuf, sizeof errbuf , reason, ap);
 		logreason = errbuf;
 	} else {
 		if (pp->error_sent) {
-			gm_snprintf(errbuf, sizeof(errbuf), "HTTP %d", pp->error_sent);
+			gm_snprintf(errbuf, sizeof errbuf, "HTTP %d", pp->error_sent);
 			logreason = errbuf;
 		} else {
 			errbuf[0] = '\0';
@@ -190,7 +202,7 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 
 	if (dbg > 1) {
 		printf("push-proxy: ending request from %s (%s): %s\n",
-			pp->socket ? ip_to_gchar(pp->socket->ip) : "<no socket>",
+			pp->socket ? host_addr_to_string(pp->socket->addr) : "<no socket>",
 			pproxy_vendor_str(pp),
 			logreason);
 	}
@@ -202,9 +214,9 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 	}
 
 	pproxy_free_resources(pp);
-	wfree(pp, sizeof(*pp));
+	wfree(pp, sizeof *pp);
 
-	pproxies = g_slist_remove(pproxies, (gpointer) pp);
+	pproxies = g_slist_remove(pproxies, pp);
 }
 
 /**
@@ -255,7 +267,7 @@ pproxy_timer(time_t now)
 	GSList *to_remove = NULL;
 
 	for (sl = pproxies; sl; sl = g_slist_next(sl)) {
-		struct pproxy *pp = (struct pproxy *) sl->data;
+		struct pproxy *pp = sl->data;
 
 		/*
 		 * We can't call pproxy_remove() since it will remove the structure
@@ -268,7 +280,7 @@ pproxy_timer(time_t now)
 	}
 
 	for (sl = to_remove; sl; sl = g_slist_next(sl)) {
-		struct pproxy *pp = (struct pproxy *) sl->data;
+		struct pproxy *pp = sl->data;
 		pproxy_error_remove(pp, 408, "Request timeout");
 	}
 
@@ -283,10 +295,11 @@ pproxy_create(struct gnutella_socket *s)
 {
 	struct pproxy *pp;
 
-	pp = walloc0(sizeof(*pp));
+	pp = walloc0(sizeof *pp);
 
 	pp->socket = s;
-	pp->last_update = time(NULL);
+	pp->flags = 0; /* XXX: TLS? */
+	pp->last_update = tm_time();
 	s->resource.pproxy = pp;
 
 	return pp;
@@ -298,7 +311,7 @@ pproxy_create(struct gnutella_socket *s)
  *
  * Fills the GUID atom into `guid_atom' and the file index into `file_idx'.
  *
- * Returns TRUE if OK, FALSE if we could not figure it out, in which case
+ * @returns TRUE if OK, FALSE if we could not figure it out, in which case
  * we also return an error to the calling party.
  */
 static gboolean
@@ -321,14 +334,24 @@ get_params(struct pproxy *pp, gchar *request,
 	url_params_t *up;
 	guint i;
 
+	g_assert(pp);
+	g_assert(request);
+	g_assert(guid_atom);
+	g_assert(file_idx);
+	
 	/*
 	 * Move to the start of the requested path.  Note that sizeof("GET")
 	 * accounts for the trailing NUL in the string.
 	 */
 
-	uri = request + ((request[0] == 'G') ? sizeof("GET") : sizeof("HEAD"));
-	while (is_ascii_blank(*uri))
-		uri++;
+	uri = is_strprefix(request, "GET");
+	if (!uri)
+		uri = is_strprefix(request, "HEAD");
+	if (!uri) {
+		pproxy_error_remove(pp, 501, "Not Implemented");
+		return FALSE;
+	}
+	uri = skip_ascii_blanks(uri);
 
 	/*
 	 * Go patch the first space we encounter before HTTP to be a NUL.
@@ -340,7 +363,7 @@ get_params(struct pproxy *pp, gchar *request,
 	 */
 
 	p = strrchr(uri, ' ');
-	if (p && p[1]=='H' && p[2]=='T' && p[3]=='T' && p[4]=='P' && p[5]=='/')
+	if (p && is_strprefix(&p[1], "HTTP/"))
 		*p = '\0';
 
 	/*
@@ -354,7 +377,7 @@ get_params(struct pproxy *pp, gchar *request,
 	attr = NULL;
 	for (i = 0; i < G_N_ELEMENTS(req_types); i++) {
 		gchar *q;
-		
+
 		if (NULL != (q = is_strprefix(req_types[i].req, uri))) {
 			attr = req_types[i].attr;
 			uri = q;
@@ -417,7 +440,7 @@ get_params(struct pproxy *pp, gchar *request,
 
 		*guid_atom = atom_guid_get(guid);
 	} else if (0 == strcmp(attr, "guid")) {
-		gchar guid[16];
+		gchar guid[GUID_RAW_SIZE];
 
 		/*
 		 * GUID in hexadecimal: valid length is 32.
@@ -440,19 +463,21 @@ get_params(struct pproxy *pp, gchar *request,
 		}
 
 		*guid_atom = atom_guid_get(guid);
-	} else
+	} else {
 		g_error("unhandled parameter \"%s\"", attr);
+	}
 
 	/*
 	 * Extract the optional "file=" parameter.
 	 */
 
 	value = url_params_get(up, "file");
+	{
+		gint error;
 
-	if (value == NULL)
-		*file_idx = 0;
-	else
-		*file_idx = (guint32) atoi(value);		/* Silent about errors */
+		/* Ignore errors; parse_uint32() returns 0 on error. */
+		*file_idx = value ? parse_uint32(value, NULL, 10, &error) : 0;
+	}
 
 	url_params_free(up);
 	return TRUE;
@@ -463,28 +488,116 @@ error:
 }
 
 /**
- * Build a push request to send.  We set TTL=max_ttl-1 and hops=1 since
+ * Builds a push request to send.  We set TTL=max_ttl-1 and hops=1 since
  * it does not come from our node really.  The file ID may be set to 0, but
  * it should be ignored when the GIV is received anyway.
+ *
+ * @param size_ptr no document
+ * @param ttl the TTL to use for the packet header.
+ * @param hops the hops value to use for the packet header.
+ * @param guid the hops value to use for the packet header.
+ * @param addr the host address the receiving peer should connect to.
+ * @param port the port number the receiving peer should connect to.
+ * @param file_idx the file index this push is for.
+ * @return	A pointer to a static buffer holding the created Gnutella PUSH
+ *			packet on success, NULL on failure.
  */
-static void
-build_push(struct gnutella_msg_push_request *m,
-	gchar *guid, guint32 ip, guint16 port, guint32 file_idx)
+const gchar *
+build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
+	host_addr_t addr, guint16 port, guint32 file_idx)
 {
-	message_set_muid(&m->header, GTA_MSG_PUSH_REQUEST);
+	static union {
+		struct gnutella_msg_push_request m;
+		gchar data[1024];
+	} packet;
+	gchar *p = packet.data;
+	size_t len = 0, size = sizeof packet;
+	ggep_stream_t gs;
+	guint32 ip;
 
-	m->header.function = GTA_MSG_PUSH_REQUEST;
-	m->header.ttl = max_ttl - 1;
-	m->header.hops = 1;
+	g_assert(size_ptr);
+	g_assert(guid);
+	g_assert(0 != port);
 
-	STATIC_ASSERT(49 == sizeof *m);
-	WRITE_GUINT32_LE(sizeof m->request, m->header.size);
+	message_set_muid(&packet.m.header, GTA_MSG_PUSH_REQUEST);
 
-	memcpy(m->request.guid, guid, 16);
+	packet.m.header.function = GTA_MSG_PUSH_REQUEST;
+	packet.m.header.ttl = ttl;
+	packet.m.header.hops = hops;
+	memcpy(packet.m.request.guid, guid, 16);
 
-	WRITE_GUINT32_LE(file_idx, m->request.file_id);
-	WRITE_GUINT32_BE(ip, m->request.host_ip);
-	WRITE_GUINT16_LE(port, m->request.host_port);
+	STATIC_ASSERT(49 == sizeof packet.m);
+	p += sizeof packet.m;
+	size -= sizeof packet.m;
+	len += sizeof packet.m.request;	/* Exclude the header size */
+
+	ggep_stream_init(&gs, p, size);
+
+#ifdef HAS_GNUTLS
+	if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0)) {
+		g_warning("could not write GGEP \"GTKG.TLS\" extension into PUSH");
+			ggep_stream_close(&gs);
+			return NULL;
+	}
+#endif /* HAS_GNUTLS */
+
+	ip = 0;
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV6:
+#ifndef USE_IPV6
+		g_assert_not_reached();
+		break;
+#else /* USE_IPV6 */
+		{
+			const host_addr_t from = addr;
+
+			if (!host_addr_convert(from, &addr, NET_TYPE_IPV4)) {
+				const guint8 *ipv6 = host_addr_ipv6(&from);
+
+				if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0)) {
+					g_warning("could not write GGEP \"GTKG.IPV6\" extension "
+						"into PUSH");
+					ggep_stream_close(&gs);
+					return NULL;
+				}
+
+				break;
+			}
+		}
+
+#endif	/* !USE_IPV6 */
+
+		/* FALL THROUGH */
+	case NET_TYPE_IPV4:
+		ip = host_addr_ipv4(addr);
+		break;
+	case NET_TYPE_NONE:
+		g_assert_not_reached();
+		break;
+	}
+
+	{
+		size_t glen;
+
+		glen = ggep_stream_close(&gs);
+		g_assert(size >= glen);
+
+		size -= glen;
+		len += glen;
+		p += glen;
+	}
+	g_assert(len < size);
+	g_assert(len < sizeof packet);
+
+	poke_le32(packet.m.request.file_id, file_idx == URN_INDEX ? 0 : file_idx);
+	poke_be32(packet.m.request.host_ip, ip);
+	poke_le16(packet.m.request.host_port, port);
+	poke_le32(packet.m.header.size, len);
+
+	message_add(packet.m.header.muid, GTA_MSG_PUSH_REQUEST, NULL);
+
+	*size_ptr = p - packet.data;
+	return packet.data;
 }
 
 /**
@@ -493,12 +606,12 @@ build_push(struct gnutella_msg_push_request *m,
  * @return atom, or NULL.
  */
 static gchar *
-validate_vendor(gchar *vendor, gchar *token, guint32 ip)
+validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 {
 	gchar *result = NULL;
 
 	if (vendor) {
-		gboolean faked = !version_check(vendor, token, ip);
+		gboolean faked = !version_check(vendor, token, addr);
 
 		if (faked) {
 			gchar name[1024];
@@ -526,11 +639,11 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	gchar *buf;
 	gchar *token;
 	gchar *user_agent;
-	struct gnutella_msg_push_request m;
 	GSList *nodes;
 
 	if (dbg > 2) {
-		printf("----Push-proxy request from %s:\n", ip_to_gchar(s->ip));
+		printf("----Push-proxy request from %s:\n",
+			host_addr_to_string(s->addr));
 		printf("%s\n", request);
 		header_dump(header, stdout);
 		printf("----\n");
@@ -544,7 +657,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	token = header_get(header, "X-Token");
 	user_agent = header_get(header, "User-Agent");
 
-	pp->user_agent = validate_vendor(user_agent, token, s->ip);
+	pp->user_agent = validate_vendor(user_agent, token, s->addr);
 
 	/*
 	 * Determine the servent ID.
@@ -555,7 +668,8 @@ pproxy_request(struct pproxy *pp, header_t *header)
 
 	if (dbg > 2)
 		printf("PUSH-PROXY: %s requesting a push to %s for file #%d\n",
-			ip_to_gchar(s->ip), guid_hex_str(pp->guid), pp->file_idx);
+			host_addr_to_string(s->addr), guid_hex_str(pp->guid),
+			pp->file_idx);
 
 	/*
 	 * Make sure they provide an X-Node header so we know whom to set up
@@ -570,23 +684,23 @@ pproxy_request(struct pproxy *pp, header_t *header)
 		return;
 	}
 
-	if (!gchar_to_ip_port(buf, &pp->ip, &pp->port)) {
+	if (!string_to_host_addr_port(buf, NULL, &pp->addr, &pp->port)) {
 		pproxy_error_remove(pp, 400,
 			"Malformed push-proxy request: cannot parse X-Node");
 		return;
 	}
 
-	if (!host_is_valid(pp->ip, pp->port)) {
+	if (!host_is_valid(pp->addr, pp->port)) {
 		pproxy_error_remove(pp, 400,
 			"Malformed push-proxy request: supplied address %s unreachable",
-			ip_port_to_gchar(pp->ip, pp->port));
+			host_addr_port_to_string(pp->addr, pp->port));
 		return;
 	}
 
-	if (pp->ip != s->ip)
+	if (!host_addr_equal(pp->addr, s->addr))
 		g_warning("push-proxy request from %s (%s) said node was at %s",
-			ip_to_gchar(s->ip), pproxy_vendor_str(pp),
-			ip_port_to_gchar(pp->ip, pp->port));
+			host_addr_to_string(s->addr), pproxy_vendor_str(pp),
+			host_addr_port_to_string(pp->addr, pp->port));
 
 	/*
 	 * Locate a route to that servent.
@@ -595,19 +709,32 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	n = route_proxy_find(pp->guid);
 
 	if (n != NULL) {
-		build_push(&m, pp->guid, pp->ip, pp->port, pp->file_idx);
-		message_add(m.header.muid, GTA_MSG_PUSH_REQUEST, NULL);
+		const gchar *packet;
+		size_t size;
 
-		STATIC_ASSERT(49 == sizeof m);
-		gmsg_sendto_one(n, &m, sizeof m);
-		gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
+		/*
+ 		 * We set TTL=max_ttl-1 and hops=1 since
+		 * it does not come from our node really.
+		 */
 
-		http_send_status(pp->socket, 202, FALSE, NULL, 0,
-			"Push-proxy: message sent to node");
+		packet = build_push(&size, max_ttl - 1, 1, pp->guid,
+					pp->addr, pp->port, pp->file_idx);
 
-		pp->error_sent = 202;
-		pproxy_remove(pp, "Push sent directly to node GUID %s",
-			guid_hex_str(pp->guid));
+		if (NULL == packet) {
+			g_warning("Failed to send push to %s (index=%lu)",
+				host_addr_port_to_string(pp->addr, pp->port),
+				(gulong) pp->file_idx);
+		} else {
+			gmsg_sendto_one(n, &packet, size);
+			gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
+
+			http_send_status(pp->socket, 202, FALSE, NULL, 0,
+					"Push-proxy: message sent to node");
+
+			pp->error_sent = 202;
+			pproxy_remove(pp, "Push sent directly to node GUID %s",
+					guid_hex_str(pp->guid));
+		}
 
 		return;
 	}
@@ -616,28 +743,40 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 * Bad luck, no direct connection.  Look for a Gnutella route.
 	 */
 
-	nodes = route_towards_guid(pp->guid);
+	if (NULL != (nodes = route_towards_guid(pp->guid))) {
+		const gchar *packet;
+		size_t size;
 
-	if (nodes != NULL) {
-		gint cnt;
+		/*
+ 		 * We set TTL=max_ttl-1 and hops=1 since
+		 * it does not come from our node really.
+		 */
 
-		build_push(&m, pp->guid, pp->ip, pp->port, pp->file_idx);
-		message_add(m.header.muid, GTA_MSG_PUSH_REQUEST, NULL);
+		packet = build_push(&size, max_ttl - 1, 1,
+					pp->guid, pp->addr, pp->port, pp->file_idx);
 
-		STATIC_ASSERT(49 == sizeof m);
-		gmsg_sendto_all(nodes, &m, sizeof m);
-		gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
+		if (NULL == packet) {
+			g_warning("Failed to send push to %s (index=%lu)",
+				host_addr_port_to_string(pp->addr, pp->port),
+				(gulong) pp->file_idx);
+		} else {
+			gint cnt;
 
-		cnt = g_slist_length(nodes);
-		g_slist_free(nodes);
+			gmsg_sendto_all(nodes, packet, size);
+			gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
 
-		http_send_status(pp->socket, 203, FALSE, NULL, 0,
-			"Push-proxy: message sent through Gnutella (via %d node%s)",
-			cnt, cnt == 1 ? "" : "s");
+			cnt = g_slist_length(nodes);
+			g_slist_free(nodes);
+			nodes = NULL;
 
-		pp->error_sent = 203;
-		pproxy_remove(pp, "Push sent via Gnutella (%d node%s) for GUID %s",
-			cnt, cnt == 1 ? "" : "s", guid_hex_str(pp->guid));
+			http_send_status(pp->socket, 203, FALSE, NULL, 0,
+					"Push-proxy: message sent through Gnutella (via %d node%s)",
+					cnt, cnt == 1 ? "" : "s");
+
+			pp->error_sent = 203;
+			pproxy_remove(pp, "Push sent via Gnutella (%d node%s) for GUID %s",
+					cnt, cnt == 1 ? "" : "s", guid_hex_str(pp->guid));
+		}
 
 		return;
 	}
@@ -648,7 +787,8 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 */
 
 	if (guid_eq(pp->guid, servent_guid)) {
-		upload_send_giv(pp->ip, pp->port, 0, 1, 0, "<from push-proxy>", FALSE);
+		upload_send_giv(pp->addr, pp->port, 0, 1, 0,
+			"<from push-proxy>", FALSE, pp->flags);
 
 		http_send_status(pp->socket, 202, FALSE, NULL, 0,
 			"Push-proxy: you found the target GUID %s",
@@ -847,7 +987,7 @@ cproxy_http_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 
 /**
  * HTTP async callback for header reception notification.
- * Returns whether processing can continue.
+ * @returns whether processing can continue.
  */
 static gboolean
 cproxy_http_header_ind(gpointer handle, header_t *header,
@@ -870,7 +1010,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 	if (server == NULL)
 		server = header_get(header, "User-Agent");
 
-	cp->server = validate_vendor(server, token, cp->ip);
+	cp->server = validate_vendor(server, token, cp->addr);
 
 	/*
 	 * Don't continue past headers, we don't expect data, and besides the
@@ -901,7 +1041,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 		break;
 	case 400:
 		g_warning("push-proxy at %s (%s) for %s file #%u reported HTTP %d: %s",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx, code, message);
 		/* FALL THROUGH */
 	case 410:
@@ -910,7 +1050,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 	default:
 		g_warning("push-proxy at %s (%s) for %s file #%u "
 			"sent unexpected HTTP %d: %s",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx, code, message);
 		download_proxy_failed(cp->d);
 		break;
@@ -918,7 +1058,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 
 	if (dbg > 2 && cp->sent)
 		printf("PUSH-PROXY at %s (%s) sent PUSH for %s file #%u %s\n",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx,
 			cp->directly ? "directly" : "via Gnet");
 
@@ -956,7 +1096,7 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
 		"\r\n",
 		verb, path, version_string,
 		tok_version(),
-		ip_port_to_gchar(listen_ip(), listen_port));
+		host_addr_port_to_string(listen_addr(), listen_port));
 }
 
 /**
@@ -981,7 +1121,7 @@ cproxy_http_newstate(gpointer handle, http_state_t newstate)
  * @returns NULL if problem during connection.
  */
 struct cproxy *
-cproxy_create(struct download *d, guint32 ip, guint16 port,
+cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
 	gchar *guid, guint32 file_idx)
 {
 	struct cproxy *cp;
@@ -995,21 +1135,21 @@ cproxy_create(struct download *d, guint32 ip, guint16 port,
 	 * Try to connect immediately: if we can't connect, no need to continue.
 	 */
 
-	handle = http_async_get_ip(path, ip, port,
+	handle = http_async_get_addr(path, addr, port,
 		cproxy_http_header_ind, NULL, cproxy_http_error_ind);
 
 	if (handle == NULL) {
 		g_warning("can't connect to push-proxy %s for GUID %s: %s",
-			ip_port_to_gchar(ip, port), guid_hex_str(guid),
+			host_addr_port_to_string(addr, port), guid_hex_str(guid),
 			http_async_strerror(http_async_errno));
 		return NULL;
 	}
 
-	cp = walloc0(sizeof(*cp));
+	cp = walloc0(sizeof *cp);
 
 	cp->magic = CPROXY_MAGIC;
 	cp->d = d;
-	cp->ip = ip;
+	cp->addr = addr;
 	cp->port = port;
 	cp->guid = atom_guid_get(guid);
 	cp->file_idx = file_idx;
