@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c,v 1.21 2005/05/04 17:37:23 graaff Exp $
+ * $Id: qhit.c,v 1.38 2005/11/21 21:12:27 rmanfredi Exp $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -28,11 +28,14 @@
  * @file
  *
  * Query hit packet management.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: qhit.c,v 1.21 2005/05/04 17:37:23 graaff Exp $");
+RCSID("$Id: qhit.c,v 1.38 2005/11/21 21:12:27 rmanfredi Exp $");
 
 #include "qhit.h"
 #include "gnutella.h"
@@ -51,19 +54,19 @@ RCSID("$Id: qhit.c,v 1.21 2005/05/04 17:37:23 graaff Exp $");
 #include "lib/getdate.h"
 #include "lib/endian.h"
 #include "lib/misc.h"
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/tm.h"
+#include "lib/override.h"			/* Must be the last header included */
 
-#define QHIT_SIZE_THRESHOLD	2016	/* Flush query hits larger than this */
-#define QHIT_SIZE_OOB		645		/* Flush OOB query hits larger than this */
-#define QHIT_MAX_RESULTS	255		/* Maximum amount of hits in a query hit */
-#define QHIT_MAX_ALT		10		/* Send out 10 alt-locs per entry, max */
-#define QHIT_MAX_PROXIES	5		/* Send out 5 push-proxies at most */
-#define QHIT_MAX_GGEP		512		/* Allocated room for trailing GGEP */
+#define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
+#define QHIT_MAX_RESULTS	255		/**< Maximum amount of hits in a query hit */
+#define QHIT_MAX_ALT		15		/**< Send out 15 alt-locs per entry, max */
+#define QHIT_MAX_PROXIES	5		/**< Send out 5 push-proxies at most */
+#define QHIT_MAX_GGEP		512		/**< Allocated room for trailing GGEP */
 
 /*
  * Minimal trailer length is our code NAME, the open flags, and the GUID.
  */
-#define QHIT_MIN_TRAILER_LEN	(4+3+16)	/* NAME + open flags + GUID */
+#define QHIT_MIN_TRAILER_LEN	(4+3+16)	/**< NAME + open flags + GUID */
 
 /*
  * Buffer where query hit packet is built.
@@ -75,15 +78,15 @@ RCSID("$Id: qhit.c,v 1.21 2005/05/04 17:37:23 graaff Exp $");
  */
 
 struct found_struct {
-	gchar data[64 * 1024];		/* data */
-	size_t pos;					/* current write position */
-	size_t files;				/* amount of file entries */
-	size_t max_size;			/* max query hit size */
-	gboolean use_ggep_h;		/* whether to use GGEP "H" to send SHA1 */
-	gchar *muid;				/* the MUID to put in all query hits */
-	qhit_process_t process;		/* processor once query hit is built */
-	gpointer udata;				/* processor argument */
-	gboolean open;				/* Set if found_open() was used */
+	gchar data[64 * 1024];		/**< data */
+	size_t pos;					/**< current write position */
+	size_t files;				/**< amount of file entries */
+	size_t max_size;			/**< max query hit size */
+	gboolean use_ggep_h;		/**< whether to use GGEP "H" to send SHA1 */
+	const gchar *muid;			/**< the MUID to put in all query hits */
+	qhit_process_t process;		/**< processor once query hit is built */
+	gpointer udata;				/**< processor argument */
+	gboolean open;				/**< Set if found_open() was used */
 };
 
 static struct found_struct *
@@ -221,7 +224,7 @@ found_set_header(void)
 	connect_speed /= MAX(1, max_uploads);	/* Upload speed expected per slot */
 
 	WRITE_GUINT16_LE(listen_port, search_head->host_port);
-	WRITE_GUINT32_BE(listen_ip(), search_head->host_ip);
+	WRITE_GUINT32_BE(host_addr_ipv4(listen_addr()), search_head->host_ip);
 	WRITE_GUINT32_LE(connect_speed, search_head->host_speed);
 }
 
@@ -247,7 +250,7 @@ found_process(void)
 }
 
 static void
-found_init(size_t max_size, gchar *xuid, gboolean ggep_h,
+found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
 	qhit_process_t proc, gpointer udata)
 {
 	struct found_struct *f = found_get();
@@ -280,11 +283,11 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 			(gint) found_file_count(),
 			found_file_count() == 1 ? "y" : "ies",
 			(gint) found_size(),
-			node_ip(n));
+			node_addr(n));
 	}
 
 	g_assert(len <= INT_MAX);
-	
+
 	/*
 	 * We limit the TTL to the minimal possible value, then add a margin
 	 * of 5 to account for re-routing abilities some day.  We then trim
@@ -379,7 +382,7 @@ flush_match(void)
 		WRITE_GUINT32_BE(start_stamp, &start);
 
 		ok =
-			ggep_stream_begin(&gs, "GTKGV1", 0) &&
+			ggep_stream_begin(&gs, GGEP_NAME(GTKGV1), 0) &&
 			ggep_stream_write(&gs, &major, 1) &&
 			ggep_stream_write(&gs, &minor, 1) &&
 			ggep_stream_write(&gs, &patch, 1) &&
@@ -406,18 +409,20 @@ flush_match(void)
 			gchar proxy[6];
 			gboolean ok;
 
-			ok = ggep_stream_begin(&gs, "PUSH", 0);
+			ok = ggep_stream_begin(&gs, GGEP_NAME(PUSH), 0);
 
 			for (
 				l = nodes, count = 0;
 				ok && l && count < QHIT_MAX_PROXIES;
 				l = g_slist_next(l), count++
 			) {
-				struct gnutella_node *n = (struct gnutella_node *) l->data;
+				struct gnutella_node *n = l->data;
 
-				WRITE_GUINT32_BE(n->proxy_ip, &proxy[0]);
-				WRITE_GUINT16_LE(n->proxy_port, &proxy[4]);
-				ok = ggep_stream_write(&gs, proxy, sizeof(proxy));
+				if (NET_TYPE_IPV4 == host_addr_net(n->proxy_addr)) {
+					WRITE_GUINT32_BE(host_addr_ipv4(n->proxy_addr), &proxy[0]);
+					WRITE_GUINT16_LE(n->proxy_port, &proxy[4]);
+					ok = ggep_stream_write(&gs, proxy, sizeof proxy);
+				}
 			}
 
 			ok = ok && ggep_stream_end(&gs);
@@ -433,23 +438,52 @@ flush_match(void)
 	 * server's hostname.
 	 */
 
-	if (!is_firewalled && give_server_hostname && 0 != *server_hostname) {
+	if (!is_firewalled && give_server_hostname && '\0' != server_hostname[0]) {
 		gboolean ok;
 
-		ok = ggep_stream_pack(&gs, "HNAME",
-				(gchar *) server_hostname, strlen(server_hostname), 0);
+		ok = ggep_stream_pack(&gs, GGEP_NAME(HNAME),
+				server_hostname, strlen(server_hostname), 0);
 
 		if (!ok)
 			g_warning("could not write GGEP \"HNAME\" extension "
 				"in query hit");
 	}
 
+#ifdef USE_IPV6
+	if (
+		NET_TYPE_IPV6 == host_addr_net(listen_addr()) &&
+		!host_addr_can_convert(listen_addr(), NET_TYPE_IPV4)
+	) {
+		const host_addr_t addr = listen_addr();
+		const guint8 *ipv6 = host_addr_ipv6(&addr);
+
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0))
+			g_warning("could not write GGEP \"GTKG.IPV6\" extension "
+				"into query hit");
+	}
+#endif /* USE_IPV6 */
+
+#ifdef HAS_GNUTLS
+	if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
+		g_warning("could not write GGEP \"GTKG.TLS\" extension into query hit");
+#endif /* HAS_GNUTLS */
+
+
+	/*
+	 * Advertise the Browse Host extension in the results if the feature is
+	 * enabled.
+     */
+
+	if (browse_host_enabled) {
+		if (!ggep_stream_pack(&gs, GGEP_NAME(BH), NULL, 0, 0))
+			g_warning("could not write GGEP \"BH\" extension into query hit");
+	}
+
 	ggep_len = ggep_stream_close(&gs);
 	found_close(ggep_len);
 
-	if (ggep_len > 0) {
+	if (ggep_len > 0)
 		trailer_start[6] |= 0x20;		/* Has GGEP extensions in trailer */
-	}
 
 	/*
 	 * Store the GUID in the last 16 bytes of the query hit.
@@ -471,11 +505,11 @@ failure:
 /**
  * Add file to current query hit.
  *
- * Returns TRUE if we inserted the record, FALSE if we refused it due to
+ * @returns TRUE if we inserted the record, FALSE if we refused it due to
  * lack of space.
  */
 static gboolean
-add_file(struct shared_file *sf)
+add_file(const struct shared_file *sf)
 {
 	size_t needed = 8 + 2 + sf->name_nfc_len;		/* size of hit entry */
 	gboolean sha1_available;
@@ -486,6 +520,7 @@ add_file(struct shared_file *sf)
 	gboolean ok;
 	ggep_stream_t gs;
 	size_t left;
+	gpointer start;
 
 	g_assert(sf->fi == NULL);	/* Cannot match partially downloaded files */
 
@@ -515,18 +550,16 @@ add_file(struct shared_file *sf)
 	if (
 		found_size() + needed + QHIT_MIN_TRAILER_LEN
 			> search_answers_forward_size
-	) {
+	)
 		return FALSE;
-	}
 
 	/*
 	 * Grow buffer by the size of the search results header 8 bytes,
 	 * plus the string length - NULL, plus two NULL's
 	 */
 
-	if (needed > found_left()) {
+	if (needed > found_left())
 		return FALSE;
-	}
 
 	/*
 	 * If size is greater than 2^31-1, we store ~0 as the file size and will
@@ -534,24 +567,20 @@ add_file(struct shared_file *sf)
 	 */
 
 	fs32 = sf->file_size >= (1U << 31) ? ~0U : sf->file_size;
-	
+
 	WRITE_GUINT32_LE(sf->file_index, &idx_le);
-	if (!found_write(&idx_le, sizeof idx_le)) {
+	if (!found_write(&idx_le, sizeof idx_le))
 		return FALSE;
-	}
 	WRITE_GUINT32_LE(fs32, &fs32_le);
-	if (!found_write(&fs32_le, sizeof fs32_le)) {
+	if (!found_write(&fs32_le, sizeof fs32_le))
 		return FALSE;
-	}
-	if (!found_write(sf->name_nfc, sf->name_nfc_len)) {
+	if (!found_write(sf->name_nfc, sf->name_nfc_len))
 		return FALSE;
-	}
 
 	/* Position equals the next byte to be written to */
 
-	if (!found_write("", 1)) {
+	if (!found_write("", 1))
 		return FALSE;
-	}
 
 	/*
 	 * We're now between the two NULs at the end of the hit entry.
@@ -566,12 +595,10 @@ add_file(struct shared_file *sf)
 		const gchar *b32 = sha1_base32(sf->sha1_digest);
 		/* Good old way: ASCII URN */
 
-		if (!found_write(urnsha1, CONST_STRLEN(urnsha1))) {
+		if (!found_write(urnsha1, CONST_STRLEN(urnsha1)))
 			return FALSE;
-		}
-		if (!found_write(b32, SHA1_BASE32_SIZE)) {
+		if (!found_write(b32, SHA1_BASE32_SIZE))
 			return FALSE;
-		}
 	}
 
 	/*
@@ -579,7 +606,8 @@ add_file(struct shared_file *sf)
 	 */
 
 	left = found_left();
-	ggep_stream_init(&gs, found_open(), left);
+	start = found_open();
+	ggep_stream_init(&gs, start, left);
 
 	/*
 	 * Emit the SHA1 as GGEP "H" if they said they understand it.
@@ -590,7 +618,7 @@ add_file(struct shared_file *sf)
 		guint8 type = GGEP_H_SHA1;
 
 		ok =
-			ggep_stream_begin(&gs, "H", GGEP_W_COBS) &&
+			ggep_stream_begin(&gs, GGEP_NAME(H), GGEP_W_COBS) &&
 			ggep_stream_write(&gs, &type, 1) &&
 			ggep_stream_write(&gs, sf->sha1_digest, SHA1_RAW_SIZE) &&
 			ggep_stream_end(&gs);
@@ -612,7 +640,7 @@ add_file(struct shared_file *sf)
 
 		g_assert(len > 0 && len <= (gint) sizeof buf);
 
-		ok = ggep_stream_pack(&gs, "LF", buf, len, GGEP_W_COBS);
+		ok = ggep_stream_pack(&gs, GGEP_NAME(LF), buf, len, GGEP_W_COBS);
 		if (!ok)
 			g_warning("could not write GGEP \"LF\" extension in query hit");
 	}
@@ -628,12 +656,15 @@ add_file(struct shared_file *sf)
 
 		g_assert(hcnt <= QHIT_MAX_ALT);
 
-		ok = ggep_stream_begin(&gs, "ALT", GGEP_W_COBS);
+		ok = ggep_stream_begin(&gs, GGEP_NAME(ALT), GGEP_W_COBS);
 
 		for (i = 0; ok && i < hcnt; i++) {
-			WRITE_GUINT32_BE(hvec[i].ip, &alt[0]);
-			WRITE_GUINT16_LE(hvec[i].port, &alt[4]);
-			ok = ggep_stream_write(&gs, alt, sizeof(alt));
+			g_assert(start == gs.outbuf);
+			if (NET_TYPE_IPV4 == host_addr_net(hvec[i].addr)) {
+				WRITE_GUINT32_BE(host_addr_ipv4(hvec[i].addr), &alt[0]);
+				WRITE_GUINT16_LE(hvec[i].port, &alt[4]);
+				ok = ggep_stream_write(&gs, alt, sizeof alt);
+			}
 		}
 
 		ok = ok && ggep_stream_end(&gs);
@@ -642,18 +673,17 @@ add_file(struct shared_file *sf)
 			g_warning("could not write GGEP \"ALT\" extension in query hit");
 	}
 
-	ggep_len = ggep_stream_close(&gs);
 	/*
 	 * Because we don't know exactly the size of the GGEP extension
 	 * (could be COBS-encoded or not), we need to adjust the real
 	 * extension size now that the entry is fully written.
 	 */
+
+	ggep_len = ggep_stream_close(&gs);
 	found_close(ggep_len);
 
-	/* Append terminating NUL */
-	if (!found_write("", 1)) {
+	if (!found_write("", 1))		/* Append terminating NUL */
 		return FALSE;
-	}
 
 	found_add_files(1);
 
@@ -693,7 +723,7 @@ add_file(struct shared_file *sf)
  * have to be sent out-of-bound
  */
 static void
-found_reset(size_t max_size, gchar *muid,
+found_reset(size_t max_size, const gchar *muid,
 	gboolean use_ggep_h, qhit_process_t process, gpointer udata)
 {
 	g_assert(process != NULL);
@@ -709,17 +739,26 @@ found_reset(size_t max_size, gchar *muid,
  * @param n				the node where we should send results to
  * @param files			the list of shared_file_t entries that make up results
  * @param count			the amount of results
+ * @param muid			the query's MUID
  * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
 qhit_send_results(
-	struct gnutella_node *n, GSList *files, gint count, gboolean use_ggep_h)
+	struct gnutella_node *n, GSList *files, gint count,
+	const gchar *muid, gboolean use_ggep_h)
 {
 	GSList *sl;
 	gint sent = 0;
 
-	found_reset(QHIT_SIZE_THRESHOLD, n->header.muid,
-		use_ggep_h, qhit_send_node, n);
+	/*
+	 * We can't use n->header.muid as the query's MUID but must rely on the
+	 * parameter we're given.  Indeed, we're delivering a local hit here,
+	 * but the query can have been OOB-proxified already and therefore the
+	 * n->header.muid data have been mangled (since that is what we're going
+	 * to forward to other nodes).
+	 */
+
+	found_reset(QHIT_SIZE_THRESHOLD, muid, use_ggep_h, qhit_send_node, n);
 
 	for (sl = files; sl; sl = g_slist_next(sl)) {
 		shared_file_t *sf = (shared_file_t *) sl->data;
@@ -728,48 +767,63 @@ qhit_send_results(
 		shared_file_unref(sf);
 	}
 
-	if (0 != found_file_count())			/* Still some unflushed results */
-		flush_match();			/* Send last packet */
+	if (0 != found_file_count())	/* Still some unflushed results */
+		flush_match();				/* Send last packet */
 
 	g_slist_free(files);
 
 	if (dbg > 3)
-		g_message("sent %d/%d hits to %s\n", sent, count, node_ip(n));
+		g_message("sent %d/%d hits to %s\n", sent, count, node_addr(n));
 }
 
 /**
- * Build query hit results for later out-of-band delivery.
+ * Build query hit results for later delivery.
+ *
+ * Results are held in the `files' list.  They are packed in hits until
+ * the message reaches the `max_msgsize' limit at which time the packet
+ * is flushed and given the the `cb' callback for processing (sending,
+ * queueing, whatever).
+ *
+ * The callback is invoked as
+ *
+ *		cb(data, len, udata)
+ *
+ * where the query hit message is held in the `len' bytes starting at `data'.
+ * The `udata' parameter is simply user-supplied data, opaque for us.
  *
- * @param cb			the processor callback to invoke on each built hit
- * @param udata			argument to pass to callback (OOB recording strcuture)
- * @param muid			the MUID to use on each generated hit
  * @param files			the list of shared_file_t entries that make up results
- * @param count			the amount of results
+ * @param count			the amount of results to deliver (first `count' files)
+ * @param max_msgsize	the targeted maximum hit size before flushing
+ * @param cb			the processor callback to invoke on each built hit
+ * @param udata			argument to pass to callback
+ * @param muid			the MUID to use on each generated hit
  * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
 qhit_build_results(
+	GSList *files, gint count,
+	size_t max_msgsize,
 	qhit_process_t cb, gpointer udata,
-	gchar *muid, GSList *files, gint count, gboolean use_ggep_h)
+	const gchar *muid, gboolean use_ggep_h)
 {
-	GSList *sl;
+	const GSList *sl;
 	gint sent;
 
 	g_assert(cb != NULL);
-	found_reset(QHIT_SIZE_OOB, muid, use_ggep_h, cb, udata);
+	found_reset(max_msgsize, muid, use_ggep_h, cb, udata);
 
 	for (sl = files, sent = 0; sl && sent < count; sl = g_slist_next(sl)) {
-		shared_file_t *sf = (shared_file_t *) sl->data;
+		const shared_file_t *sf = sl->data;
 
 		if (add_file(sf))
 			sent++;
 	}
 
-	if (0 != found_file_count())			/* Still some unflushed results */
-		flush_match();			/* Send last packet */
+	if (0 != found_file_count())	/* Still some unflushed results */
+		flush_match();				/* Send last packet */
 
 	/*
-	 * Nothing to free, since everything is the property of the OOB module.
+	 * Nothing to free, since everything is the property of the calling module.
 	 */
 }
 
@@ -779,7 +833,7 @@ qhit_build_results(
 void
 qhit_init(void)
 {
-	release_date = date2time(GTA_RELEASE, time(NULL));
+	release_date = date2time(GTA_RELEASE, tm_time());
 }
 
 /**
