@@ -1,10 +1,8 @@
 /*
- * $Id: settings.c,v 1.22 2005/05/14 04:11:04 cbiere Exp $
+ * $Id: settings.c,v 1.51 2005/12/08 00:49:08 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
- * gtk-gnutella configuration.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,22 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Gtk-Gnutella configuration.
+ *
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #include "common.h"
 
+#ifdef I_NETDB
 #include <netdb.h>
+#endif
 
 #include "lib/eval.h"
 
@@ -43,6 +54,7 @@
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"		/* For debugging() */
 #include "if/core/net_stats.h"
 
 #include "if/bridge/c2ui.h"
@@ -50,17 +62,19 @@
 #include "lib/cq.h"
 #include "lib/file.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: settings.c,v 1.22 2005/05/14 04:11:04 cbiere Exp $");
+RCSID("$Id: settings.c,v 1.51 2005/12/08 00:49:08 cbiere Exp $");
 
 #define debug dbg
 
 static const gchar config_file[] = "config_gnet";
 static const gchar ul_stats_file[] = "upload_stats";
 
-#define CONFIG_DIR_MODE	/* 0755 */ \
-	(S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+#define PID_FILE_MODE	(S_IRUSR | S_IWUSR) /* 0600 */
+#define CONFIG_DIR_MODE /* 0750 */ \
+    (S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP)
 
 static gchar *home_dir = NULL;
 static gchar *config_dir = NULL;
@@ -89,105 +103,186 @@ static void update_uptimes(void);
 
 /* ----------------------------------------- */
 
-/*
- * ensure_unicity
- *
+static host_addr_t
+listen_address(gboolean update)
+{
+	static host_addr_t addr;
+
+	if (update) {
+		const gchar *val;
+
+		val = force_local_ip ? forced_local_ip : local_ip;
+		addr = string_to_host_addr(val, NULL);
+	}
+	return addr;
+}
+
+static inline void
+listen_address_update(void)
+{
+	listen_address(TRUE);
+}
+
+/**
+ * @return the currently used local listening address.
+ */
+host_addr_t
+listen_addr(void)
+{
+	return listen_address(FALSE);
+}
+
+/**
  * Look for any existing PID file. If found, look at the pid recorded
  * there and make sure it has died. Abort operations if it hasn't...
+ *
+ * @returns -1 on failure, the file descriptor of the pidfile otherwise.
  */
-static void
+static gint
 ensure_unicity(const gchar *file)
 {
-	FILE *f;
-	pid_t pid;
-	guint64 pid_value;
-	gint error;
-	gchar buf[16];
+	gint fd, e;
+	gboolean locking_failed;
 
-	f = file_fopen_missing(file, "r");
-	if (f == NULL)
-		return;				/* Assume it's missing if can't be opened */
+	g_assert(file);
 
-	buf[0] = '\0';
-	if (NULL == fgets(buf, sizeof(buf), f))
-		return;
-	fclose(f);
-
-	pid_value = parse_uint64(buf, NULL, 10, &error);
-	if (error)
-		return;				/* Can't read it back correctly */
-
-	pid = pid_value;
-
-	/*
-	 * Existence check relies on the existence of signal 0. The kernel
-	 * won't actually send anything, but will perform all the existence
-	 * checks inherent to the kill() syscall for us...
-	 */
-
-	if (-1 == kill(pid, 0)) {
-		if (errno != ESRCH)
-			g_warning("kill() return unexpected error: %s", g_strerror(errno));
-		return;
+	fd = file_create(file, O_RDWR, PID_FILE_MODE);
+	if (-1 == fd) {
+		e = errno;
+		g_warning("could not access \"%s\": %s", file, g_strerror(e));
+		return -1;
 	}
 
-	fprintf(stderr,
-		"You seem to have left another gtk-gnutella running (pid = %ld)\n",
-		(glong) pid);
-	exit(1);
+/* FIXME: These might be enums, a compile-time check would be better */
+#if defined(F_SETLK) && defined(F_WRLCK)
+	{
+		static const struct flock zero_flock;
+		struct flock fl;
+
+		fl = zero_flock;
+		fl.l_type = F_WRLCK;
+		fl.l_whence = SEEK_SET;
+		/* l_start and l_len are zero, which means the whole is locked */
+
+		locking_failed = -1 == fcntl(fd, F_SETLK, &fl);
+		if (locking_failed) {
+			e = errno;
+
+			locking_failed = TRUE;
+			g_warning("fcntl(%d, F_SETLK, ...) failed for \"%s\": %s",
+				fd, file, g_strerror(e));
+
+			/*
+			 * Use F_GETLK to determing the PID of the process, the
+			 * reinitialization of "fl" might be unnecessary but who
+			 * knows.
+			 */
+			fl = zero_flock;
+			fl.l_type = F_WRLCK;
+			fl.l_whence = SEEK_SET;
+
+			if (-1 != fcntl(fd, F_GETLK, &fl)) {
+				g_warning("another gtk-gnutella process seems to "
+					"be still running (pid=%lu)", (gulong) fl.l_pid);
+			}
+		}
+	}
+#else
+	locking_failed = TRUE;
+	errno = 0;
+#endif /* F_SETLK && F_WRLCK */
+
+	if (!locking_failed)
+		goto done;
+
+	if (VAL_EAGAIN == e || EACCES == e)
+		goto failed;		/* The file appears to be locked */
+
+	/* Maybe F_SETLK is not supported by the OS or filesystem,
+	 * fall back to weaker PID locking */
+	{
+		ssize_t r;
+		gchar buf[33];
+
+		r = read(fd, buf, sizeof buf - 1);
+		e = errno;
+
+		if ((ssize_t) -1 == r) {
+			/* This would be odd */
+			g_warning("could not read pidfile \"%s\": %s", file, g_strerror(e));
+			goto failed;
+		}
+
+		/* Check the PID in the file */
+		{
+			guint64 u;
+			gint error;
+
+			g_assert(r >= 0 && (size_t) r < sizeof buf);
+			buf[r] = '\0';
+
+			u = parse_uint64(buf, NULL, 10, &error);
+
+			/* If the pidfile seems to be corrupt, ignore it */
+			if (!error && u > 1) {
+				pid_t pid = u;
+
+				if (0 == kill(pid, 0)) {
+					g_warning("another gtk-gnutella process seems to "
+						"be still running (pid=%lu)", (gulong) pid);
+					goto failed;
+				}
+			}
+		}
+	}
+
+done:
+	/* Keep the fd open, otherwise the lock is lost */
+	return fd;
+
+failed:
+	close(fd);
+	return -1;
 }
 
 /**
  * Write our pid to the pidfile.
  */
 static void
-save_pid(const gchar *file)
+save_pid(gint fd)
 {
-	FILE *f;
+	size_t len;
+	gchar buf[32];
 
-	f = file_fopen(file, "w");
-	if (f == NULL)
+	g_assert(-1 != fd);
+
+	gm_snprintf(buf, sizeof buf, "%lu\n", (gulong) getpid());
+	len = strlen(buf);
+
+	if (-1 == ftruncate(fd, 0))	{
+		g_warning("ftruncate() failed for pidfile: %s", g_strerror(errno));
 		return;
+	}
 
-	fprintf(f, "%lu\n", (gulong) getpid());
+	if (0 != lseek(fd, 0, SEEK_SET))	{
+		g_warning("lseek() failed for pidfile: %s", g_strerror(errno));
+		return;
+	}
 
-	if (0 != fclose(f))
-		g_warning("could not flush pidfile \"%s\": %s",
-			file, g_strerror(errno));
+	if (len != (size_t) write(fd, buf, len))
+		g_warning("could not flush pidfile: %s", g_strerror(errno));
 }
 
 /* ----------------------------------------- */
 
-#if !defined(_SC_PAGE_SIZE) && defined(_SC_PAGESIZE)
-#define _SC_PAGE_SIZE _SC_PAGESIZE
-#endif
-
-#if defined (_SC_PAGE_SIZE) && defined (_SC_PHYS_PAGES)
-static glong
-settings_getpagesize(void)
-{
-	glong ret;
-
-	errno = 0;
-	ret = sysconf(_SC_PAGE_SIZE);
-	if ((glong) -1 == ret && 0 != errno) {
-		g_warning("sysconf(_SC_PHYS_PAGES) failed: %s", g_strerror(errno));
-		return 0;
-	}
-	return ret;
-}
-#else
-#define settings_getpagesize() ((glong) getpagesize())
-#endif /* _SC_PAGE_SIZE && _SC_PHYS_PAGES */
-
 /**
  * @return the amount of physical RAM in KB, or zero in case of failure
  */
-static gulong
+static guint64
 settings_getphysmemsize(void)
-{
 #if defined (_SC_PHYS_PAGES)
-	guint32 pagesize = settings_getpagesize();
+{
+	size_t pagesize = compat_pagesize();
 	glong pages;
 
 	errno = 0;
@@ -197,12 +292,29 @@ settings_getphysmemsize(void)
 		return 0;
 	}
 	return (pagesize >> 10) * (gulong) pages;
+}
+#elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM64)
+{
+/* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
+	int mib[2] = { CTL_HW, HW_USERMEM64 };
+	guint64 amount = 0;
+	size_t len = sizeof amount;
 
+	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
+		g_warning(
+			"settings_getphysmemsize: sysctl() for HW_USERMEM64 failed: %s",
+			g_strerror(errno));
+		return 0;
+	}
+
+	return amount / 1024;
+}
 #elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM)
+{
 /* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
 	int mib[2] = { CTL_HW, HW_USERMEM };
-	int amount = 0;
-	size_t len = sizeof(amount);
+	guint32 amount = 0;
+	size_t len = sizeof amount;
 
 	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
 		g_warning(
@@ -212,8 +324,9 @@ settings_getphysmemsize(void)
 	}
 
 	return amount / 1024;
-
+}
 #elif defined (HAS_GETINVENT)
+{
 	inventory_t *inv;
 	long physmem = 0;
 
@@ -239,38 +352,38 @@ settings_getphysmemsize(void)
 	}
 
 	return physmem * 1024;
-
+}
 #else /* ! _SC_PHYS_PAGES && ! HAS_SYSCTL && ! HAS_GETINVENT */
-
+{
 	g_warning("Unable to determine amount of physical RAM");
 	return 0;
-
-#endif /* _SC_PHYS_PAGES */
 }
+#endif /* _SC_PHYS_PAGES */
 
 void
 settings_init(void)
 {
-	guint32 maxfd = (guint32) sysconf(_SC_OPEN_MAX);
-	gulong memory = settings_getphysmemsize();
-	guint32 amount = (guint32) memory;
-	struct rlimit lim;
+	guint64 memory = settings_getphysmemsize();
+	guint64 amount = memory; 
 	char *path = NULL;
+	guint max_fd;
 
-	if (debug > 0) {
-		g_message("detected amount of physical RAM: %lu KB", memory);
-		g_message("process can use %u file descriptors", maxfd);
-	}
-
-	if (-1 != getrlimit(RLIMIT_DATA, &lim)) {
-		guint32 maxdata = lim.rlim_cur >> 10;
-		amount = MIN(amount, maxdata);		/* For our purposes */
+#ifdef RLIMIT_DATA 
+	{
+		struct rlimit lim;
+	
+		if (-1 != getrlimit(RLIMIT_DATA, &lim)) {
+			guint32 maxdata = lim.rlim_cur >> 10;
+			amount = MIN(amount, maxdata);		/* For our purposes */
+		}
 	}
+#endif /* RLIMIT_DATA */
 
     properties = gnet_prop_init();
-
-	gnet_prop_set_guint32_val(PROP_SYS_NOFILE, maxfd);
-	gnet_prop_set_guint32_val(PROP_SYS_PHYSMEM, amount);
+	max_fd = compat_max_fd();
+	
+	gnet_prop_set_guint32_val(PROP_SYS_NOFILE, max_fd);
+	gnet_prop_set_guint64_val(PROP_SYS_PHYSMEM, amount);
 
 	memset(deconstify_gchar(servent_guid), 0, sizeof servent_guid);
 	config_dir = g_strdup(getenv("GTK_GNUTELLA_DIR"));
@@ -286,8 +399,7 @@ settings_init(void)
 
 	if (!is_directory(config_dir)) {
 		g_warning(_("creating configuration directory \"%s\""), config_dir);
-
-		if (mkdir(config_dir, CONFIG_DIR_MODE) == -1) {
+		if (-1 == compat_mkdir(config_dir, CONFIG_DIR_MODE)) {
 			g_warning("mkdir(\"%s\") failed: \"%s\"",
 				config_dir, g_strerror(errno));
 			goto no_config_dir;
@@ -298,13 +410,29 @@ settings_init(void)
 	/* Ensure we're the only instance running */
 
 	path = make_pathname(config_dir, pidfile);
-	ensure_unicity(path);
-	save_pid(path);
+	{
+		gint fd;
+
+		if (-1 == (fd = ensure_unicity(path))) {
+			g_warning(
+				_("You seem to have left another gtk-gnutella running\n"));
+			exit(EXIT_FAILURE);
+		}
+		save_pid(fd);
+		/* The file descriptor must be kept open */
+	}
 	G_FREE_NULL(path);
 
 		/* Parse the configuration */
 	prop_load_from_file(properties, config_dir, config_file);
 
+	if (debugging(0)) {
+		g_message("detected amount of physical RAM: %s", short_kb_size(memory));
+		g_message("process can use at maximum: %s", short_kb_size(amount));
+		g_message("process can use %u file descriptors", max_fd);
+		g_message("max I/O vector size is %d items", MAX_IOV_COUNT);
+	}
+
 	path = make_pathname(config_dir, ul_stats_file);
 	upload_stats_load_history(path);	/* Loads the upload statistics */
 	G_FREE_NULL(path);
@@ -372,15 +500,19 @@ settings_remove_pidfile(void)
 
 /**
  * This routine is called when we determined that our IP was no longer the
- * one we computed.  We base this on some headers sent back when we handshake
- * with other nodes, and as a result, cannot trust the information.
+ * one we computed.
+ *
+ * We base this on some headers sent back when we handshake with other nodes,
+ * and as a result, cannot trust the information.
  *
  * What we do henceforth is trust 3 successive indication that our IP changed,
  * provided we get the same information each time.
  *
  *		--RAM, 13/01/2002
  *
- * ``peer_ip'' is the IP address of peer which reported the new IP address.
+ * @param `new_addr' the newly suggested address.
+ * @param `peer' the IP address of peer which reported the new IP address.
+ *
  * There must be 3 peers from 3 different /16 networks before a change is
  * accepted. Otherwise, it would be very easy to confuse GTKG by connecting
  * 3 times in a row and submitting a *wrong* IP address.
@@ -388,24 +520,23 @@ settings_remove_pidfile(void)
  *		--cbiere, 2004-08-01
  */
 void
-settings_ip_changed(guint32 new_ip, guint32 peer_ip)
+settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer)
 {
-	static guint32 last_ip_seen = 0;
-	static guint same_ip_count = 0;
-	static guint32 peers[3];
+	static guint same_addr_count = 0;
+	static host_addr_t peers[3], last_addr_seen;
 	guint i;
 
-	g_assert(!force_local_ip);		/* Must be called when IP isn't forced */
-	g_assert(new_ip != 0);			/* The new IP must be valid */
+	g_assert(!force_local_ip);	  /* Must be called when IP isn't forced */
+	g_assert(is_host_addr(new_addr)); /* The new IP must be valid */
 
-	if (!ip_is_valid(new_ip) || !ip_is_valid(peer_ip))
+	if (!host_addr_is_routable(new_addr) || !host_addr_is_routable(peer))
 		return;
 
 	/*
 	 * Don't accept updates for private addresses from non-private addresses
 	 * and vice-versa.
 	 */
-	if (is_private_ip(new_ip) ^ is_private_ip(peer_ip))
+	if (is_private_addr(new_addr) ^ is_private_addr(peer))
 		return;
 
 	/*
@@ -413,40 +544,41 @@ settings_ip_changed(guint32 new_ip, guint32 peer_ip)
 	 * network; addresses are in host byte order.
 	 */
 	if (
-		is_private_ip(new_ip) &&
-		is_private_ip(peer_ip) &&
-		(peer_ip & 0xffff0000) != (new_ip & 0xffff0000)
+		is_private_addr(new_addr) &&
+		is_private_addr(peer) &&
+		!host_addr_matches(peer, new_addr, 16) /* CIDR /16 */
 	) {
 		return;
 	}
 
-	peer_ip &= 0xffff0000;	/* One vote per /16 network; host byte order! */
+	/* FIXME: Correct this for IPv6 */
 	for (i = 0; i < G_N_ELEMENTS(peers); i++) {
-		if (peers[i] == peer_ip)
+		if (host_addr_matches(peer, new_addr, 16)) /* CIDR /16 */
 			return;
 	}
 
-	if (new_ip != last_ip_seen) {
-		last_ip_seen = new_ip;
-		same_ip_count = 1;
-		peers[0] = peer_ip;
+	if (!host_addr_equal(new_addr, last_addr_seen)) {
+		last_addr_seen = new_addr;
+		same_addr_count = 1;
+		peers[0] = peer;
 		return;
 	}
 
-	g_assert(same_ip_count > 0 && same_ip_count < G_N_ELEMENTS(peers));
-	peers[same_ip_count] = peer_ip;
+	g_assert(same_addr_count > 0 && same_addr_count < G_N_ELEMENTS(peers));
+	peers[same_addr_count] = peer;
 
-	if (++same_ip_count < G_N_ELEMENTS(peers))
+	if (++same_addr_count < G_N_ELEMENTS(peers))
 		return;
 
-	last_ip_seen = 0;
-	same_ip_count = 0;
-	memset(peers, 0, sizeof peers);
+	last_addr_seen = zero_host_addr;
+	same_addr_count = 0;
+	for (i = 0; i < G_N_ELEMENTS(peers); i++)
+		peers[0] = zero_host_addr;
 
-	if (new_ip == local_ip)
+	if (host_addr_equal(new_addr, string_to_host_addr(local_ip, NULL)))
 		return;
 
-    gnet_prop_set_guint32_val(PROP_LOCAL_IP, new_ip);
+    gnet_prop_set_string(PROP_LOCAL_IP, host_addr_to_string(new_addr));
 }
 
 /**
@@ -476,7 +608,7 @@ settings_max_msg_size(void)
  * Ask them to set a property to be able to run.
  */
 void
-settings_ask_for_property(gchar *name, gchar *value)
+settings_ask_for_property(const gchar *name, const gchar *value)
 {
 	fprintf(stderr, "\n*** ANCIENT VERSION DETECTED! ***\n\n");
 	fprintf(stderr,
@@ -596,17 +728,16 @@ gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *s)
 guint32
 get_average_ip_lifetime(time_t now)
 {
-	guint32 current_ip_stamp;
+	time_t current_ip_stamp;
 	guint32 average_ip_uptime;
-	gint32 lifetime;
+	glong lifetime;
 
-	gnet_prop_get_guint32_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
+	gnet_prop_get_timestamp_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
 	gnet_prop_get_guint32_val(PROP_AVERAGE_IP_UPTIME, &average_ip_uptime);
 
 	if (current_ip_stamp) {
 		lifetime = delta_time(now, current_ip_stamp);
-		if (lifetime < 0)
-			lifetime = 0;
+		lifetime = MAX(0, lifetime);
 	} else
 		lifetime = 0;
 
@@ -627,44 +758,37 @@ get_average_ip_lifetime(time_t now)
 static void
 update_address_lifetime(void)
 {
-	static guint32 old_ip = 0;
-	gboolean force_local_ip;
-	guint32 current_ip;
-	guint32 current_ip_stamp;
-	time_t now;
+	static host_addr_t old_addr;
+	time_t current_ip_stamp;
+	host_addr_t addr;
 
-	gnet_prop_get_boolean_val(PROP_FORCE_LOCAL_IP, &force_local_ip);
-	if (force_local_ip)
-		gnet_prop_get_guint32_val(PROP_FORCED_LOCAL_IP, &current_ip);
-	else
-		gnet_prop_get_guint32_val(PROP_LOCAL_IP, &current_ip);
+	addr = listen_addr();
 
-	if (old_ip == 0) {				/* First time */
-		old_ip = current_ip;
-		gnet_prop_get_guint32_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
-		if (current_ip_stamp == 0) {
-			now = time(NULL);
-			gnet_prop_set_guint32_val(PROP_CURRENT_IP_STAMP, (guint32) now);
-		}
+	if (!is_host_addr(old_addr)) {				/* First time */
+		old_addr = addr;
+		gnet_prop_get_timestamp_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
+		if (0 == current_ip_stamp)
+			gnet_prop_set_timestamp_val(PROP_CURRENT_IP_STAMP, tm_time());
 	}
 
-	if (old_ip == current_ip)
-		return;
+	if (!host_addr_equal(old_addr, addr)) {
+		time_t now;
 
-	/*
-	 * IP address changed, update lifetime information.
-	 */
+		/*
+		 * IP address changed, update lifetime information.
+		 */
 
-	now = time(NULL);
-	old_ip = current_ip;
+		now = tm_time();
+		old_addr = addr;
 
-	gnet_prop_get_guint32_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
+		gnet_prop_get_timestamp_val(PROP_CURRENT_IP_STAMP, &current_ip_stamp);
 
-	if (current_ip_stamp)
-		gnet_prop_set_guint32_val(PROP_AVERAGE_IP_UPTIME,
-			get_average_ip_lifetime(now));
+		if (current_ip_stamp)
+			gnet_prop_set_guint32_val(PROP_AVERAGE_IP_UPTIME,
+					get_average_ip_lifetime(now));
 
-	gnet_prop_set_guint32_val(PROP_CURRENT_IP_STAMP, (guint32) now);
+		gnet_prop_set_timestamp_val(PROP_CURRENT_IP_STAMP, now);
+	}
 }
 
 /**
@@ -674,17 +798,15 @@ update_address_lifetime(void)
 guint32
 get_average_servent_uptime(time_t now)
 {
-	time_t start_stamp;
-	gint32 uptime;
-	guint32 val, avg_servent_uptime;
+	guint32 avg_servent_uptime;
+	time_t start;
+	glong uptime;
 
 	gnet_prop_get_guint32_val(PROP_AVERAGE_SERVENT_UPTIME, &avg_servent_uptime);
-	gnet_prop_get_guint32_val(PROP_START_STAMP, &val);
+	gnet_prop_get_timestamp_val(PROP_START_STAMP, &start);
 
-	start_stamp = val;
-	uptime = delta_time(now, start_stamp);
-	if (uptime < 0)
-		uptime = 0;
+	uptime = delta_time(now, start);
+	uptime = MAX(0, uptime);
 
 	/*
 	 * The average uptime is computed as an EMA on 7 terms.
@@ -703,7 +825,7 @@ get_average_servent_uptime(time_t now)
 static void
 update_uptimes(void)
 {
-	time_t now = time(NULL);
+	time_t now = tm_time();
 
 	gnet_prop_set_guint32_val(PROP_AVERAGE_SERVENT_UPTIME,
 		get_average_servent_uptime(now));
@@ -784,7 +906,7 @@ enable_udp_changed(property_t prop)
     gnet_prop_get_boolean_val(prop, &enabled);
 	if (enabled) {
 		if (s_udp_listen == NULL && listen_port)
-			s_udp_listen = socket_udp_listen(0, listen_port);
+			s_udp_listen = socket_udp_listen(zero_host_addr, listen_port);
 	} else {
 		if (s_udp_listen) {
 			socket_free(s_udp_listen);
@@ -821,9 +943,11 @@ listen_port_changed(property_t prop)
 
 	random_port = listen_port == 1;
 	memset(tried, 0, sizeof tried);
-	memset(tried, 0xff, 1024 / 8); /* Mark ports below 1024 as already tried */
+	memset(tried, 0xff, 1024 / 8);	/* Mark ports below 1024 as already tried */
 
 	do {
+		host_addr_t listen_ha = zero_host_addr;
+
 		if (random_port) {
 			guint32 i, b, r;
 
@@ -863,8 +987,27 @@ listen_port_changed(property_t prop)
 		 * If the new port != 0, open the new port
 		 */
 
-		if (listen_port)
-			s_tcp_listen = socket_tcp_listen(0, listen_port, SOCK_TYPE_CONTROL);
+		if (listen_port) {
+
+			switch (network_protocol) {
+			case NET_USE_BOTH:
+			case NET_USE_IPV4:
+				listen_ha = host_addr_set_ipv4(INADDR_ANY);
+				break;
+#ifdef USE_IPV6
+			case NET_USE_IPV6:
+				{
+					static const guint8 zero_ipv6_addr[16];
+
+					host_addr_set_ipv6(&listen_ha, zero_ipv6_addr);
+				}
+				break;
+#endif /* USE_IPV6 */
+			}
+
+			s_tcp_listen = socket_tcp_listen(listen_ha,
+							listen_port, SOCK_TYPE_CONTROL);
+		}
 
 		/*
 		 * If UDP is enabled, also listen on the same UDP port.
@@ -872,7 +1015,7 @@ listen_port_changed(property_t prop)
 
 		if (enable_udp) {
 			if (listen_port)
-				s_udp_listen = socket_udp_listen(0, listen_port);
+				s_udp_listen = socket_udp_listen(listen_ha, listen_port);
 			if (random_port && s_udp_listen == NULL) {
 				socket_free(s_tcp_listen);
 				s_tcp_listen = NULL;
@@ -902,6 +1045,23 @@ listen_port_changed(property_t prop)
     return FALSE;
 }
 
+static gboolean
+network_protocol_changed(property_t prop)
+{
+	guint32 port;
+
+	(void) prop;
+    gnet_prop_get_guint32_val(PROP_LISTEN_PORT, &port);
+	/*
+	 * Trigger a change of the listening port to create a new listening
+	 * socket.
+	 */
+    gnet_prop_set_guint32_val(PROP_LISTEN_PORT, 0);
+    gnet_prop_set_guint32_val(PROP_LISTEN_PORT, port);
+	return FALSE;
+}
+
+
 static gboolean
 bw_http_in_enabled_changed(property_t prop)
 {
@@ -1016,36 +1176,18 @@ scan_extensions_changed(property_t prop)
 static gboolean
 file_path_changed(property_t prop)
 {
-    gchar *s = gnet_prop_get_string(prop, NULL, 0);
+    gchar *s;
 
+	s = gnet_prop_get_string(prop, NULL, 0);
 	g_assert(s != NULL);
 
 	if (!is_directory(s)) {
 		g_message("Attempt to create directory \"%s\"", s);
-		if (create_directory(s))
+
+		if (0 != create_directory(s))
 			g_message("Attempt failed: \"%s\"", g_strerror(errno));
 	}
 
-    if (!is_directory(s)) {
-		gchar *path;
-
-        if (prop == PROP_SAVE_FILE_PATH) {
-			prop_def_t *def = gnet_prop_get_def(prop);
-			path = g_strdup(eval_subst(*def->data.string.def));
-			prop_free_def(def);
-		} else
-			path = gnet_prop_get_string(PROP_SAVE_FILE_PATH, NULL, 0);
-
-        g_warning("property \"%s\": directory %s is not available, "
-            "using %s instead", gnet_prop_name(prop), s, path);
-
-        gnet_prop_set_string(prop, path);
-
-		G_FREE_NULL(path);
-        G_FREE_NULL(s);
-        return TRUE;
-    }
-
     G_FREE_NULL(s);
     return FALSE;
 }
@@ -1235,17 +1377,19 @@ lib_debug_changed(property_t unused_prop)
 }
 
 static gboolean
-force_local_ip_changed(property_t prop)
+force_local_addr_changed(property_t prop)
 {
 	g_assert(PROP_FORCE_LOCAL_IP == prop);
+	listen_address_update();
 	update_address_lifetime();
     return FALSE;
 }
 
 static gboolean
-local_ip_changed(property_t prop)
+local_addr_changed(property_t prop)
 {
 	g_assert(PROP_LOCAL_IP == prop);
+	listen_address_update();
 	update_address_lifetime();
     return FALSE;
 }
@@ -1345,7 +1489,7 @@ node_rx_size_changed(property_t prop)
 static gpointer ev_file_descriptor_shortage = NULL;
 static gpointer ev_file_descriptor_runout = NULL;
 
-#define RESET_PROP_TM	(10*60*1000)	/* 10 minutes in ms */
+#define RESET_PROP_TM	(10*60*1000)	/**< 10 minutes in ms */
 
 /**
  * callout queue callback
@@ -1410,39 +1554,16 @@ file_descriptor_x_changed(property_t prop)
     return FALSE;
 }
 
-/**
- * This is only necessary to migrate the old PROP_PROXY_IP to
- * PROP_PROXY_HOSTNAME and should be removed in a future release.
- *    -- cbiere, 2004-06-29
- */
-static gboolean
-proxy_ip_changed(property_t prop)
-{
-	guint32 ip;
-
-	gnet_prop_get_guint32_val(prop, &ip);
-	if (ip) {
-		gchar *hostname = gnet_prop_get_string(PROP_PROXY_HOSTNAME, NULL, 0);
-
-		if (hostname[0] == '\0')
-			gnet_prop_set_string(PROP_PROXY_HOSTNAME, ip_to_gchar(ip));
-    	G_FREE_NULL(hostname);
-	}
-    return FALSE;
-}
-
 /***
  *** Property-to-callback map
  ***/
 
 typedef struct prop_map {
-    property_t prop;            /* property handle */
-    prop_changed_listener_t cb; /* callback function */
-    gboolean init;              /* init widget with current value */
+    property_t prop;            /**< property handle */
+    prop_changed_listener_t cb; /**< callback function */
+    gboolean init;              /**< init widget with current value */
 } prop_map_t;
 
-#define IGNORE NULL
-
 static prop_map_t property_map[] = {
     {
         PROP_NODE_SENDQUEUE_SIZE,
@@ -1489,6 +1610,11 @@ static prop_map_t property_map[] = {
         listen_port_changed,
         TRUE
     },
+    {
+        PROP_NETWORK_PROTOCOL,
+        network_protocol_changed,
+        FALSE
+    },
     {
         PROP_BW_HTTP_IN_ENABLED,
         bw_http_in_enabled_changed,
@@ -1606,12 +1732,12 @@ static prop_map_t property_map[] = {
     },
 	{
 		PROP_FORCE_LOCAL_IP,
-		force_local_ip_changed,
+		force_local_addr_changed,
 		TRUE,
 	},
 	{
 		PROP_LOCAL_IP,
-		local_ip_changed,
+		local_addr_changed,
 		TRUE,
 	},
 	{
@@ -1644,11 +1770,6 @@ static prop_map_t property_map[] = {
 		file_descriptor_x_changed,
 		FALSE,
 	},
-	{
-		PROP_PROXY_IP,
-		proxy_ip_changed,
-		TRUE,
-	},
 	{
 		PROP_ENABLE_UDP,
 		enable_udp_changed,
@@ -1687,7 +1808,7 @@ settings_callbacks_init(void)
             g_warning("settings_callbacks_init:"
                 " property %d already mapped", n);
 
-        if (property_map[n].cb != IGNORE) {
+        if (property_map[n].cb) {
             gnet_prop_add_prop_changed_listener(
                 property_map[n].prop,
                 property_map[n].cb,
@@ -1722,7 +1843,7 @@ settings_callbacks_shutdown(void)
 	}
 
     for (n = 0; n < PROPERTY_MAP_SIZE; n ++) {
-        if (property_map[n].cb != IGNORE) {
+        if (property_map[n].cb) {
             gnet_prop_remove_prop_changed_listener(
                 property_map[n].prop,
                 property_map[n].cb);
