@@ -1,5 +1,5 @@
 /*
- * $Id: search.h,v 1.12 2005/06/25 01:37:42 daichik Exp $
+ * $Id: search.h,v 1.19 2005/11/27 04:37:45 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -29,6 +29,7 @@
 #include "common.h"
 
 #include "lib/misc.h"
+#include "lib/vendors.h"
 #include "if/core/nodes.h"
 
 /***
@@ -39,8 +40,11 @@ typedef guint32 gnet_search_t;
 /*
  * Flags for search_new()
  */
-#define SEARCH_PASSIVE	 0x01 /**< start a passive ssearch */
-#define SEARCH_ENABLED	 0x02 /**< start an enabled search */
+enum {
+	SEARCH_F_PASSIVE 	= 1 << 0,	/**< Start a passive ssearch */
+	SEARCH_F_ENABLED 	= 1 << 1,	/**< Start an enabled search */
+	SEARCH_F_BROWSE		= 1 << 2	/**< Start a browse-host search */
+};
 
 /*
  * Host vectors held in query hits.
@@ -53,22 +57,24 @@ typedef struct gnet_host_vec {
 /*
  * Result sets `status' flags.
  */
-#define ST_KNOWN_VENDOR			0x8000		/**< Found known vendor code */
-#define ST_PARSED_TRAILER		0x4000		/**< Was able to parse trailer */
-#define ST_UDP					0x2000		/**< Got hit via UDP */
-#define ST_BOGUS				0x1000		/**< Bogus IP address */
-#define ST_PUSH_PROXY			0x0010		/**< Listed some push proxies */
-#define ST_GGEP					0x0008		/**< Trailer has a GGEP extension */
-#define ST_UPLOADED				0x0004		/**< Is "stable", people downloaded */
-#define ST_BUSY					0x0002		/**< Has currently no slots */
-#define ST_FIREWALL				0x0001		/**< Is behind a firewall */
+#define ST_TLS				(1 << 10)	/**< Indicated support for TLS */
+#define ST_BH				(1 << 9)	/**< Browse Host support */
+#define ST_KNOWN_VENDOR		(1 << 8)	/**< Found known vendor code */
+#define ST_PARSED_TRAILER	(1 << 7)	/**< Was able to parse trailer */
+#define ST_UDP				(1 << 6)	/**< Got hit via UDP */
+#define ST_BOGUS			(1 << 5)	/**< Bogus IP address */
+#define ST_PUSH_PROXY		(1 << 4)	/**< Listed some push proxies */
+#define ST_GGEP				(1 << 3)	/**< Trailer has a GGEP extension */
+#define ST_UPLOADED			(1 << 2)	/**< Is "stable", people downloaded */
+#define ST_BUSY				(1 << 1)	/**< Has currently no slots */
+#define ST_FIREWALL			(1 << 0)	/**< Is behind a firewall */
 
 /*
  * Processing of ignored files.
  */
-#define SEARCH_IGN_DISPLAY_AS_IS	0		/**< Display normally */
-#define SEARCH_IGN_DISPLAY_MARKED	1		/**< Display marked (lighter color) */
-#define SEARCH_IGN_NO_DISPLAY		2		/**< Don't display */
+#define SEARCH_IGN_DISPLAY_AS_IS	0	/**< Display normally */
+#define SEARCH_IGN_DISPLAY_MARKED	1	/**< Display marked (lighter color) */
+#define SEARCH_IGN_NO_DISPLAY		2	/**< Don't display */
 
 /**
  * A results_set structure factorizes the common information from a Query Hit
@@ -80,18 +86,18 @@ typedef struct gnet_host_vec {
  */
 typedef struct gnet_results_set {
 	gchar *guid;				/**< Servent's GUID (atom) */
-	guint32 ip;
+	host_addr_t addr;
 	guint16 port;
 	guint16 status;				/**< Parsed status bits from trailer */
 	guint32 speed;
 	time_t  stamp;				/**< Reception time of the hit */
-	guchar  vendor[4];			/**< Vendor code */
+	union vendor_code vcode;	/**< Vendor code */
 	gchar *version;				/**< Version information (atom) */
 	gint country;				/**< Country code -- encoded ISO3166 */
     flag_t  flags;
 	gnet_host_vec_t *proxies;	/**< Optional: known push proxies */
 	gchar *hostname;			/**< Optional: server's hostname */
-	guint32 udp_ip;				/**< IP of delivering node, if hit from UDP */
+	host_addr_t udp_addr;		/**< IP of delivering node, if hit from UDP */
 
 	GSList *records;
 	guint32 num_recs;
@@ -132,7 +138,8 @@ typedef void (*search_got_results_listener_t)
 
 #ifdef CORE_SOURCES
 
-gnet_search_t search_new(const gchar *, guint32 timeout, flag_t flags);
+gnet_search_t search_new(const gchar *, time_t create_time, guint lifetime,
+		guint32 timeout, flag_t flags);
 void search_close(gnet_search_t sh);
 
 void search_start(gnet_search_t sh);
@@ -146,16 +153,25 @@ void search_reissue(gnet_search_t sh);
 void search_add_kept(gnet_search_t sh, guint32 kept);
 
 gboolean search_is_passive(gnet_search_t sh);
+gboolean search_is_active(gnet_search_t sh);
 gboolean search_is_frozen(gnet_search_t sh);
+gboolean search_is_expired(gnet_search_t sh);
 
 void search_set_reissue_timeout(gnet_search_t sh, guint32 timeout);
 guint32 search_get_reissue_timeout(gnet_search_t sh);
+guint search_get_lifetime(gnet_search_t sh);
+time_t search_get_create_time(gnet_search_t sh);
+void search_set_create_time(gnet_search_t sh, time_t t);
 
 void search_free_alt_locs(gnet_record_t *rc);
 void search_free_proxies(gnet_results_set_t *rs);
 
 void search_update_items(gnet_search_t sh, guint32 items);
 
+gboolean search_browse(gnet_search_t sh,
+	const gchar *hostname, host_addr_t addr, guint16 port,
+	const gchar *guid, gboolean push, const gnet_host_vec_t *proxies);
+
 #endif /* CORE_SOURCES */
 #endif /* _if_core_search_h_ */
 
