@@ -1,5 +1,5 @@
 /*
- * $Id: prop.c,v 1.17 2005/05/18 18:48:13 cbiere Exp $
+ * $Id: prop.c,v 1.32 2005/11/13 12:00:26 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,31 +25,30 @@
 
 #include "common.h"
 
-RCSID("$Id: prop.c,v 1.17 2005/05/18 18:48:13 cbiere Exp $");
+RCSID("$Id: prop.c,v 1.32 2005/11/13 12:00:26 cbiere Exp $");
 
 #include "prop.h"
 #include "file.h"
 #include "misc.h"
 #include "glib-missing.h"
+#include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
 #define debug track_props
-static guint32 track_props = 0;	/* XXX need to init lib's props--RAM */
-static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
+static guint32 track_props = 0;	/**< XXX need to init lib's props--RAM */
+static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
 
-/*
- * Checks if a property is part of a property set.
- */
-#define prop_in_range(ps, p) ((p >= ps->offset) && (p < ps->size+ps->offset))
-
-const gchar * const prop_type_str[] = {
-	"boolean",
-	"guint32",
-	"guint64",
-	"string",
-	"ip",
-	"storage",
-	"multichoice"
+const struct {
+	const gchar *name; 
+} prop_type_str[] = {
+	{ "boolean" 	},
+	{ "guint32" 	},
+	{ "guint64" 	},
+	{ "ip" 			},
+	{ "multichoice" },
+	{ "storage" 	},
+	{ "string"		},
+	{ "timestamp"	},
 };
 
 /***
@@ -64,193 +63,273 @@ G_STMT_START {														\
 	}																\
 } G_STMT_END
 
-/**
- * Parse comma delimited numeric vector.
- */
-static void
-prop_parse_ip_vector(const gchar *name,
-	const gchar *str, size_t size, guint32 *t)
+typedef gint (* prop_parse_func_t)(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i);
+
+static gint
+prop_parse_guint64(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
 {
-	/* Parse comma delimited settings */
-
-	gchar **h;
-	size_t i;
-
-	g_assert(str != NULL);
-	g_assert(t != NULL);
-
-	h = g_strsplit(str, ",", size + 1);
-
-	for (i = 0; i < size; i++) {
-		if (!h[i])
-			break;
-
-		t[i] = gchar_to_ip(h[i]);
+	gint error;
+	guint64 u;
+	
+	u = parse_uint64(str, endptr, 10, &error);
+	if (error) {
+		g_warning("prop_parse_guint64: (prop=\"%s\") "
+			"str=\"%s\": \"%s\"", name, str, g_strerror(error));
+	} else if (vec) {
+		((guint64 *) vec)[i] = u;
 	}
 
-	if (i < size)
-		g_warning("prop_parse_ip_vector: (prop=\"%s\") "
-			"target initialization incomplete!", name);
-
-	g_strfreev(h);
+	return error;
 }
 
-/**
- * Parse comma delimited numeric vector.
- */
-static void
-prop_parse_guint64_vector(const gchar *name,
-	const gchar *str, size_t size, guint64 *t)
+static gint
+prop_parse_timestamp(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
 {
-	/* Parse comma delimited settings */
+	const gchar *ep;
+	gint error = 0;
+	guint64 u;
+	time_t t;
 
-	gchar **h;
-	size_t i;
-
-	g_assert(str != NULL);
-	g_assert(t != NULL);
-
-	h = g_strsplit(str, ",", size + 1);
-
-	for (i = 0; i < size; i++) {
-		int error;
-		gulong v;
-
-		if (!h[i])
-			break;
-
-		/* TODO: long is most likely a 32-bit integer! */
-		v = gm_atoul(h[i], NULL, &error);
-		if (error) {
-			g_warning("prop_parse_guint64_vector: (prop=\"%s\") "
-				"h[i]=\"%s\": \"%s\"", name, h[i], g_strerror(error));
-			continue;
-		}
-
-		t[i] = v;
+	u = parse_uint64(str, &ep, 10, &error);
+	if (error) {
+		g_warning("prop_parse_timestamp: (prop=\"%s\") "
+			"str=\"%s\": \"%s\"", name, str, g_strerror(error));
 	}
+	if (*ep != '-') {
+		t = u;	/* For backwards-compatibility accept raw numeric timestamps */
+	} else {
+		static const struct tm zero_tm;
+		struct tm tm;
 
-	if (i < size)
-		g_warning("prop_parse_guint64_vector: (prop=\"%s\") "
-			"target initialization incomplete!", name);
+		t = 0;
+		tm = zero_tm;
+		tm.tm_year = u - 1900;
 
-	g_strfreev(h);
+		/* Parse an ISO 8601 date of the variant YYYY-MM-DD HH:MM:SS */
+		if (!error && *ep == '-') {
+			guint16 v;
+			
+			str = &ep[1];
+			v = parse_uint16(str, &ep, 10, &error);
+			if (!error && (v < 1 || v > 12))
+				error = ERANGE;
+			else
+				tm.tm_mon = v - 1; 
+		}
+		if (!error && *ep == '-') {
+			guint16 v;
+
+			str = &ep[1];
+			v = parse_uint16(str, &ep, 10, &error);
+			if (!error && (v < 1 || v > 31))
+				error = ERANGE;
+			else
+				tm.tm_mday = v;
+		}
+		if (!error && (is_ascii_blank(*ep) || *ep == 'T')) {
+			str = skip_ascii_blanks(ep);
+			if (str[0] == 'T')
+				str++;
+			tm.tm_hour = parse_uint16(str, &ep, 10, &error);
+			if (!error && tm.tm_hour > 23)
+				error = ERANGE;
+		}
+		if (!error && *ep == ':') {
+			str = &ep[1];
+			tm.tm_min = parse_uint16(str, &ep, 10, &error);
+			if (!error && tm.tm_min > 59)
+				error = ERANGE;
+		}
+		if (!error && *ep == ':') {
+			str = &ep[1];
+			tm.tm_sec = parse_uint16(str, &ep, 10, &error);
+			if (!error && tm.tm_sec > 61)
+				error = ERANGE;
+		}
+		if (!error) {
+			errno = 0;
+			if ((time_t) -1 == (t = mktime(&tm)))
+				error = errno ? errno : EINVAL;
+		}
+	}
+	
+	if (!error && vec)
+		((time_t *) vec)[i] = t;
+
+	if (endptr)
+		*endptr = ep;
+
+	return error;
 }
 
-/**
- * Parse comma delimited numeric vector.
- */
-static void
-prop_parse_guint32_vector(const gchar *name,
-	const gchar *str, size_t size, guint32 *t)
+
+static gint
+prop_parse_guint32(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
 {
-	/* Parse comma delimited settings */
-
-	gchar **h;
-	size_t i;
-
-	g_assert(str != NULL);
-	g_assert(t != NULL);
-
-	h = g_strsplit(str, ",", size + 1);
-
-	for (i = 0; i < size; i++) {
-		int error;
-		gulong v;
-
-		if (!h[i])
-			break;
-
-		v = gm_atoul(h[i], NULL, &error);
-		if (!error && (guint32) v != v)
-			error = ERANGE;
-
-		if (error) {
-			g_warning("prop_parse_guint32_vector: "
-				"(prop=\"%s\") h[i]=\"%s\": \"%s\"",
-				name, h[i], g_strerror(error));
-			continue;
-		}
-
-		t[i] = v;
+	gint error;
+	guint32 u;
+	
+	u = parse_uint32(str, endptr, 10, &error);
+	if (error) {
+		g_warning("prop_parse_guint32: (prop=\"%s\") "
+			"str=\"%s\": \"%s\"", name, str, g_strerror(error));
+	} else if (vec) {
+		((guint32 *) vec)[i] = u;
 	}
 
-	if (i < size)
-		g_warning("prop_parse_guint32_vector: (prop=\"%s\") "
-			"target initialization incomplete!", name);
+	return error;
+}
 
-	g_strfreev(h);
+static gint
+prop_parse_ip(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
+{
+	guint32 ip;
+	gint error;
+	
+	g_assert(name);
+	g_assert(str);
+
+	error = string_to_ip_strict(str, &ip, endptr) ? 0 : EINVAL;
+	if (error) {
+		g_warning("prop_parse_ip: (prop=\"%s\") "
+			"str=\"%s\": \"%s\"", name, str, g_strerror(error));
+	} else if (vec) {
+		((guint32 *) vec)[i] = ip;
+	}
+	
+	return error;
 }
 
 /**
  * Parse comma delimited boolean vector (TRUE/FALSE list).
  */
-static void
-prop_parse_boolean_vector(const gchar *name,
-	const gchar *str, size_t size, gboolean *t)
+static gint
+prop_parse_boolean(const gchar *name,
+	const gchar *str, const gchar **endptr, gpointer vec, size_t i)
 {
-	/* Parse comma delimited settings */
+	static const struct {
+		const gchar *s;
+		const gboolean v;
+	} tab[] = {
+		{ "0",		FALSE },
+		{ "1",		TRUE },
+		{ "FALSE",	FALSE },
+		{ "TRUE",	TRUE },
+	};
+	gboolean b = FALSE;
+	const gchar *p = NULL;
+	guint j;
+	gint error = 0;
+	
+	g_assert(name);
+	g_assert(str);
 
-	const gchar *a, *b, *p = str;
-	size_t i;
-
-	g_assert(str != NULL);
-	g_assert(t != NULL);
-
-	for (i = 0; *p != '\0' && i < size; i++) {
-		size_t l;
-		gboolean valid;
-
-		p = skip_ascii_spaces(p);
-		if (*p == '\0')
+	for (j = 0; j < G_N_ELEMENTS(tab); j++) {
+		if (NULL != (p = is_strcaseprefix(str, tab[j].s))) {
+			b = tab[j].v;
 			break;
-
-		a = p;
-		b = strchr(p, ',');
-		if (!b)
-			b = strchr(p, '\0');
-		p = b--;
-		g_assert(*p == ',' || *p == '\0');
-		p++;
-
-		while (b > a && is_ascii_space(*b))
-			b--;
-
-		l = (b - a) + 1;
-		valid = FALSE;
-		switch (l) {
-		case CONST_STRLEN("TRUE"):
-			if (0 == ascii_strncasecmp(a, "TRUE", l)) {
-				t[i] = TRUE;
-				valid = TRUE;
-			}
-			break;
-		case CONST_STRLEN("FALSE"):
-			if (0 == ascii_strncasecmp(a, "FALSE", l)) {
-				t[i] = FALSE;
-				valid = TRUE;
-			}
-			break;
-		case CONST_STRLEN("1"):
-			if (*a == '1') {
-				t[i] = TRUE;
-				valid = TRUE;
-			} else if (*a == '0') {
-				t[i] = FALSE;
-				valid = TRUE;
-			}
-			break;
-		}
-		if (!valid) {
-			g_warning("Not a boolean value (prop=\"%s\"): \"%s\"", name, a);
 		}
 	}
 
-	if (i != size)
-		g_warning("prop_parse_boolean_vector: (prop=\"%s\") "
+	if (!p) {
+		p = str;
+		error = EINVAL;
+	}
+
+	if (error) {
+		g_warning("Not a boolean value (prop=\"%s\"): \"%s\"", name, str);
+	} else if (vec) {
+		((gboolean *) vec)[i] = b;
+	}
+
+	if (endptr)
+		*endptr = p;
+
+	return error;
+}
+
+/**
+ * Parse prop vector.
+ */
+static void
+prop_parse_vector(const gchar *name, const gchar *str,
+	size_t size, gpointer vec, prop_parse_func_t parser)
+{
+	const gchar *p = str;
+	size_t i;
+
+	g_assert(str != NULL);
+	g_assert(vec != NULL);
+
+	for (i = 0; i < size && p; i++) {
+		const gchar *endptr;
+		gint error;
+
+		p = skip_ascii_spaces(p);
+		if ('\0' == *p)
+			break;
+
+		error = (*parser)(name, p, &endptr, vec, i);
+		endptr = skip_ascii_spaces(endptr);
+		if (!error)
+			error = ('\0' != *endptr && ',' != *endptr) ? EINVAL : 0;
+
+		if (error)
+			g_warning("prop_parse_vector: (prop=\"%s\") "
+				"str=\"%s\": \"%s\"", name, p, g_strerror(error));
+
+		p = strchr(endptr, ',');
+		if (p)
+			p++;
+	}
+
+	if (i < size)
+		g_warning("prop_parse_vector: (prop=\"%s\") "
 			"target initialization incomplete!", name);
 }
 
+static void
+prop_parse_guint64_vector(const gchar *name, const gchar *str,
+	size_t size, guint64 *vec)
+{
+	prop_parse_vector(name, str, size, vec, prop_parse_guint64);
+}
+
+static void
+prop_parse_timestamp_vector(const gchar *name, const gchar *str,
+	size_t size, time_t *vec)
+{
+	prop_parse_vector(name, str, size, vec, prop_parse_timestamp);
+}
+
+
+static void
+prop_parse_guint32_vector(const gchar *name, const gchar *str,
+	size_t size, guint32 *vec)
+{
+	prop_parse_vector(name, str, size, vec, prop_parse_guint32);
+}
+
+static void
+prop_parse_ip_vector(const gchar *name, const gchar *str,
+	size_t size, guint32 *vec)
+{
+	prop_parse_vector(name, str, size, vec, prop_parse_ip);
+}
+
+static void
+prop_parse_boolean_vector(const gchar *name, const gchar *str,
+	size_t size, gboolean *vec)
+{
+	prop_parse_vector(name, str, size, vec, prop_parse_boolean);
+}
+
+
 /**
  * Parse a hex string into a gchar array.
  */
@@ -275,7 +354,7 @@ prop_parse_storage(const gchar *name, const gchar *str, size_t size, gchar *t)
 				"storage is damaged: \"%s\"", name, str);
 			return;
 		}
-		t[i] = (hex2dec(h) << 4) + hex2dec(l);
+		t[i] = (hex2int(h) << 4) + hex2int(l);
 	}
 }
 
@@ -290,6 +369,7 @@ prop_parse_storage(const gchar *name, const gchar *str, size_t size, gchar *t)
  * Use the prop_free_def call to free the memory again. A simple g_free
  * won't do, since there are lot's of pointers to allocated memory
  * in the definition structure.
+ *
  * The prop_changed_listeners field will always be NULL in the copy.
  */
 prop_def_t *
@@ -307,7 +387,7 @@ prop_get_def(prop_set_t *ps, property_t p)
 	buf->desc = g_strdup(PROP(ps, p).desc);
 	buf->ev_changed = NULL;
 
-	switch(buf->type) {
+	switch (buf->type) {
 	case PROP_TYPE_BOOLEAN:
 		buf->data.boolean.def = g_memdup(
 			PROP(ps,p).data.boolean.def,
@@ -348,6 +428,7 @@ prop_get_def(prop_set_t *ps, property_t p)
 			PROP(ps,p).data.guint32.value,
 			sizeof(guint32) * PROP(ps,p).vector_size);
 		break;
+
 	case PROP_TYPE_GUINT64:
 		buf->data.guint64.def = g_memdup(
 			PROP(ps,p).data.guint64.def,
@@ -355,15 +436,31 @@ prop_get_def(prop_set_t *ps, property_t p)
 		buf->data.guint64.value = g_memdup(
 			PROP(ps,p).data.guint64.value,
 			sizeof(guint64) * PROP(ps,p).vector_size);
+		break;
+
+	case PROP_TYPE_TIMESTAMP:
+		buf->data.timestamp.def = g_memdup(
+			PROP(ps,p).data.timestamp.def,
+			sizeof(time_t) * PROP(ps,p).vector_size);
+		buf->data.timestamp.value = g_memdup(
+			PROP(ps,p).data.timestamp.value,
+			sizeof(time_t) * PROP(ps,p).vector_size);
+		break;
+
 	case PROP_TYPE_STRING:
 		buf->data.string.def	= g_new(gchar*, 1);
 		*buf->data.string.def   = g_strdup(*PROP(ps,p).data.string.def);
 		buf->data.string.value  = g_new(gchar*, 1);
 		*buf->data.string.value = g_strdup(*PROP(ps,p).data.string.value);
 		break;
+
 	case PROP_TYPE_STORAGE:
 		buf->data.storage.value = g_memdup
 			(PROP(ps,p).data.storage.value, PROP(ps,p).vector_size);
+		break;
+		
+	case NUM_PROP_TYPES:
+		g_assert_not_reached();
 	}
 
 	return buf;
@@ -399,6 +496,10 @@ prop_free_def(prop_def_t *d)
 		G_FREE_NULL(d->data.guint64.value);
 		G_FREE_NULL(d->data.guint64.def);
 		break;
+	case PROP_TYPE_TIMESTAMP:
+		G_FREE_NULL(d->data.timestamp.value);
+		G_FREE_NULL(d->data.timestamp.def);
+		break;
 	case PROP_TYPE_STRING:
 		if (*d->data.string.value)
 			G_FREE_NULL(*d->data.string.value);
@@ -409,6 +510,9 @@ prop_free_def(prop_def_t *d)
 		break;
 	case PROP_TYPE_STORAGE:
 		G_FREE_NULL(d->data.storage.value);
+		break;
+	case NUM_PROP_TYPES:
+		g_assert_not_reached();
 	}
 	G_FREE_NULL(d->name);
 	G_FREE_NULL(d->desc);
@@ -485,8 +589,8 @@ prop_set_boolean(prop_set_t *ps, property_t prop, const gboolean *src,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_BOOLEAN]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_BOOLEAN].name);
 
 	if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -538,8 +642,8 @@ prop_get_boolean(prop_set_t *ps, property_t prop, gboolean *t,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_BOOLEAN]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_BOOLEAN].name);
 
 	if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -568,8 +672,8 @@ prop_set_guint64(prop_set_t *ps, property_t prop, const guint64 *src,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_GUINT64]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_GUINT64].name);
 
 	if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -652,8 +756,8 @@ prop_get_guint64(prop_set_t *ps, property_t prop, guint64 *t,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_GUINT64]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_GUINT64].name);
 
    if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -684,10 +788,10 @@ prop_set_guint32(prop_set_t *ps, property_t prop, const guint32 *src,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s, %s or %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_GUINT32],
-			prop_type_str[PROP_TYPE_IP],
-			prop_type_str[PROP_TYPE_MULTICHOICE]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_GUINT32].name,
+			prop_type_str[PROP_TYPE_IP].name,
+			prop_type_str[PROP_TYPE_MULTICHOICE].name);
 
 	if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -722,9 +826,10 @@ prop_set_guint32(prop_set_t *ps, property_t prop, const guint32 *src,
 			}
 
 			if (invalid) {
-				g_warning("prop_set_guint32: [%s] new value is invalid "
-					"choice %u (leaving at %u)",
-					PROP(ps,prop).name, newval, *src);
+				g_warning("prop_set_guint32: [%s] new value is invalid choice "
+					"%u (leaving at %u)",
+					PROP(ps,prop).name, newval,
+					*PROP(ps,prop).data.guint32.value);
 			} else {
 				*PROP(ps,prop).data.guint32.value = newval;
 			}
@@ -767,7 +872,7 @@ prop_set_guint32(prop_set_t *ps, property_t prop, const guint32 *src,
 
 		for (n = 0; n < PROP(ps,prop).vector_size; n++) {
 			if (PROP(ps,prop).type == PROP_TYPE_IP) {
-				printf("%s%s ", ip_to_gchar(
+				printf("%s%s ", ip_to_string(
 					PROP(ps,prop).data.guint32.value[n]),
 					(n < (PROP(ps,prop).vector_size-1)) ? "," : "");
 			} else {
@@ -799,10 +904,10 @@ prop_get_guint32(prop_set_t *ps, property_t prop, guint32 *t,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s, %s or %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_GUINT32],
-			prop_type_str[PROP_TYPE_IP],
-			prop_type_str[PROP_TYPE_MULTICHOICE]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_GUINT32].name,
+			prop_type_str[PROP_TYPE_IP].name,
+			prop_type_str[PROP_TYPE_MULTICHOICE].name);
 
    	if (length == 0)
 		length = PROP(ps,prop).vector_size;
@@ -816,6 +921,121 @@ prop_get_guint32(prop_set_t *ps, property_t prop, guint32 *t,
 	return target;
 }
 
+void
+prop_set_timestamp(prop_set_t *ps, property_t prop, const time_t *src,
+	size_t offset, size_t length)
+{
+	gboolean differ = FALSE;
+
+	g_assert(ps != NULL);
+	g_assert(src != NULL);
+
+	if (!prop_in_range(ps, prop))
+		g_error("prop_set_timestamp: unknown property %d", prop);
+	if ((PROP(ps,prop).type != PROP_TYPE_TIMESTAMP) )
+		g_error("Type mismatch setting value for [%s] of type"
+			" %s when %s was expected",
+			PROP(ps,prop).name,
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_TIMESTAMP].name);
+
+	if (length == 0)
+		length = PROP(ps,prop).vector_size;
+
+	prop_assert(ps, prop, offset + length <= PROP(ps,prop).vector_size);
+
+	differ = 0 != memcmp(&PROP(ps,prop).data.timestamp.value[offset], src,
+					length * sizeof *src);
+
+	if (!differ)
+		return;
+
+	/*
+	 * Only do bounds-checking on non-vector properties.
+	 */
+	if (PROP(ps,prop).vector_size == 1) {
+		/*
+		 * Either check multiple choices or min/max.
+		 */
+			prop_assert(ps, prop, PROP(ps,prop).data.timestamp.choices == NULL);
+
+			if (
+				(PROP(ps,prop).data.timestamp.min <= *src) &&
+				(PROP(ps,prop).data.timestamp.max >= *src)
+			) {
+				*PROP(ps,prop).data.timestamp.value = *src;
+			} else {
+				gchar buf[64];
+				time_t newval = *src;
+
+				if (newval > PROP(ps,prop).data.timestamp.max)
+					newval = PROP(ps,prop).data.timestamp.max;
+				if (newval < PROP(ps,prop).data.timestamp.min)
+					newval = PROP(ps,prop).data.timestamp.min;
+
+				concat_strings(buf, sizeof buf,
+					uint64_to_string(PROP(ps,prop).data.timestamp.min), "/",
+					uint64_to_string2(PROP(ps,prop).data.timestamp.max),
+					(void *) 0);
+				g_warning("prop_set_timestamp: [%s] new value out of bounds "
+					"(%s): %s (adjusting to %s)", PROP(ps,prop).name, buf,
+					uint64_to_string(*src), uint64_to_string2(newval));
+
+				*PROP(ps,prop).data.timestamp.value = newval;
+			}
+	} else {
+		memcpy(&PROP(ps,prop).data.timestamp.value[offset], src,
+			length * sizeof *src);
+	}
+
+	if (debug >= 5) {
+		size_t n;
+
+		printf("updated property [%s] = ( ", PROP(ps,prop).name);
+
+		for (n = 0; n < PROP(ps,prop).vector_size; n++) {
+			printf("%s%s ",
+				uint64_to_string(PROP(ps,prop).data.timestamp.value[n]),
+				n < (PROP(ps,prop).vector_size-1) ? "," : "");
+		}
+
+		printf(")\n");
+	}
+
+	prop_emit_prop_changed(ps, prop);
+}
+
+time_t *
+prop_get_timestamp(prop_set_t *ps, property_t prop, time_t *t,
+	size_t offset, size_t length)
+{
+	time_t *target;
+	size_t n;
+
+	g_assert(ps != NULL);
+
+	if (!prop_in_range(ps, prop))
+		g_error("prop_get_timestamp: unknown property %d", prop);
+	if ((PROP(ps,prop).type != PROP_TYPE_TIMESTAMP))
+		g_error("Type mismatch setting value for [%s] of type"
+			" %s when %s was expected",
+			PROP(ps,prop).name,
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_TIMESTAMP].name);
+
+   if (length == 0)
+		length = PROP(ps,prop).vector_size;
+
+	prop_assert(ps, prop, offset + length <= PROP(ps,prop).vector_size);
+
+	n = length * sizeof *target;
+	target = t != NULL ? (gpointer) t : g_malloc(n);
+	memcpy(target, &PROP(ps,prop).data.timestamp.value[offset], n);
+
+	return target;
+}
+
+
 void
 prop_set_storage(prop_set_t *ps, property_t prop, const gchar *src,
 	size_t length)
@@ -831,8 +1051,8 @@ prop_set_storage(prop_set_t *ps, property_t prop, const gchar *src,
 		g_error("Type mismatch setting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_STORAGE]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_STORAGE].name);
 
 	prop_assert(ps, prop, length == PROP(ps,prop).vector_size);
 
@@ -866,8 +1086,8 @@ prop_get_storage(prop_set_t *ps, property_t prop, gchar *t, size_t length)
 		g_error("Type mismatch getting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_STORAGE]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_STORAGE].name);
 
 	prop_assert(ps, prop, length == PROP(ps,prop).vector_size);
 
@@ -891,8 +1111,8 @@ prop_set_string(prop_set_t *ps, property_t prop, const gchar *val)
 		g_error("Type mismatch getting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_STRING]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_STRING].name);
 
 	prop_assert(ps, prop, PROP(ps,prop).vector_size == 1);
 
@@ -933,6 +1153,7 @@ prop_set_string(prop_set_t *ps, property_t prop, const gchar *val)
  * (t != NULL), then this is used. The size indicates the size of the given
  * string buffer and may not be 0 in this case. The pointer which is
  * returned will point to the given buffer.
+ *
  * If no string buffer is given (t == NULL), new memory is allocated and
  * returned. This memory must be free'ed later. The size parameter has
  * no effect in this case.
@@ -954,8 +1175,8 @@ prop_get_string(prop_set_t *ps, property_t prop, gchar *t, size_t size)
 		g_error("Type mismatch getting value for [%s] of type"
 			" %s when %s was expected",
 			PROP(ps,prop).name,
-			prop_type_str[PROP(ps,prop).type],
-			prop_type_str[PROP_TYPE_STRING]);
+			prop_type_str[PROP(ps,prop).type].name,
+			prop_type_str[PROP_TYPE_STRING].name);
 
 	s = *PROP(ps,prop).data.string.value;
 
@@ -998,8 +1219,22 @@ prop_description(prop_set_t *ps, property_t prop)
 	return PROP(ps,prop).desc;
 }
 
+const gchar *
+prop_type_to_string(prop_set_t *ps, property_t prop)
+{
+	g_assert(PROP(ps,prop).type < NUM_PROP_TYPES);
+	STATIC_ASSERT(NUM_PROP_TYPES == G_N_ELEMENTS(prop_type_str));
+	return prop_type_str[PROP(ps,prop).type].name;
+}
+
+gboolean
+prop_is_saved(prop_set_t *ps, property_t prop)
+{
+	return PROP(ps,prop).save;
+}
+
 /**
- * Helper function for update_label() and update_entry()
+ * Fetches the value of property as a string.
  */
 const gchar *
 prop_to_string(prop_set_t *ps, property_t prop)
@@ -1009,74 +1244,146 @@ prop_to_string(prop_set_t *ps, property_t prop)
 	g_assert(ps != NULL);
 
 	if (!prop_in_range(ps, prop))
-		g_error("prop_get_gchar: unknown property %d", prop);
+		g_error("prop_get_gchar: unknown property %u", prop);
 
 	switch (PROP(ps,prop).type) {
-		case PROP_TYPE_GUINT32: {
+	case PROP_TYPE_GUINT32:
+		{
 			guint32 val;
 
 			prop_get_guint32(ps, prop, &val, 0, 1);
 			gm_snprintf(s, sizeof(s), "%u", val);
-			break;
 		}
-		case PROP_TYPE_GUINT64: {
+		break;
+	case PROP_TYPE_GUINT64:
+		{
 			guint64 val;
 
 			prop_get_guint64(ps, prop, &val, 0, 1);
-			uint64_to_string_buf(s, sizeof s, val);
-			break;
+			uint64_to_string_buf(val, s, sizeof s);
 		}
-		case PROP_TYPE_STRING: {
+		break;
+	case PROP_TYPE_TIMESTAMP:
+		{
+			time_t val;
+
+			prop_get_timestamp(ps, prop, &val, 0, 1);
+			timestamp_to_string_buf(val, s, sizeof s);
+		}
+		break;
+	case PROP_TYPE_STRING:
+		{
 			gchar *buf = prop_get_string(ps, prop, NULL, 0);
-			g_strlcpy(s, buf ? buf : "", sizeof(s));
+
+			g_strlcpy(s, buf ? buf : "", sizeof s);
 			G_FREE_NULL(buf);
-			break;
 		}
-		case PROP_TYPE_IP: {
+		break;
+	case PROP_TYPE_IP:
+		{
 			guint32 val;
 
 			prop_get_guint32(ps, prop, &val, 0, 1);
-
-			g_strlcpy(s, ip_to_gchar(val), sizeof(s));
-			break;
+			g_strlcpy(s, ip_to_string(val), sizeof s);
 		}
-		case PROP_TYPE_BOOLEAN: {
+		break;
+	case PROP_TYPE_BOOLEAN:
+		{
 			gboolean val;
 
 			prop_get_boolean(ps, prop, &val, 0, 1);
-
-			g_strlcpy(s, val ? "TRUE" : "FALSE", sizeof(s));
-			break;
+			g_strlcpy(s, val ? "TRUE" : "FALSE", sizeof s);
 		}
-		case PROP_TYPE_MULTICHOICE: {
+		break;
+	case PROP_TYPE_MULTICHOICE:
+		{
 			guint n = 0;
 
 			while (
 				(PROP(ps, prop).data.guint32.choices[n].title != NULL) &&
 				(PROP(ps, prop).data.guint32.choices[n].value !=
-					*(PROP(ps, prop).data.guint32.value))
-				)
+				 *(PROP(ps, prop).data.guint32.value))
+			  )
 				n++;
 
 			if (PROP(ps, prop).data.guint32.choices[n].title != NULL)
-				gm_snprintf(s, sizeof(s), "%d: %s",
-					*(PROP(ps, prop).data.guint32.value),
-					PROP(ps,prop).data.guint32.choices[n].title);
+				gm_snprintf(s, sizeof s, "%u: %s",
+						*(PROP(ps, prop).data.guint32.value),
+						PROP(ps,prop).data.guint32.choices[n].title);
 			else
-				gm_snprintf(s, sizeof(s),
-					"%d: No descriptive string found for this value",
-					*(PROP(ps, prop).data.guint32.value));
-			break;
+				gm_snprintf(s, sizeof s,
+						"%u: No descriptive string found for this value",
+						*(PROP(ps, prop).data.guint32.value));
 		}
-		case PROP_TYPE_STORAGE: {
-			g_strlcpy(s, guid_hex_str(prop_get_storage(ps, prop, NULL,
-							PROP(ps,prop).vector_size)), sizeof(s));
-			break;
+		break;
+	case PROP_TYPE_STORAGE:
+		g_strlcpy(s, guid_hex_str(prop_get_storage(ps, prop, NULL,
+			PROP(ps,prop).vector_size)), sizeof s);
+		break;
+	default:
+		s[0] = '\0';
+		g_error("update_entry_gnet: incompatible type %s",
+			prop_type_str[PROP(ps,prop).type].name);
+	}
+
+	return s;
+}
+
+/**
+ * Fetches the default value of property as a string.
+ */
+const gchar *
+prop_default_to_string(prop_set_t *ps, property_t prop)
+{
+	static gchar s[4096];
+	const prop_def_t *p = &PROP(ps, prop);
+	
+	switch (p->type) {
+	case PROP_TYPE_GUINT32:
+		gm_snprintf(s, sizeof s, "%u", (guint) p->data.guint32.def[0]);
+		break;
+	case PROP_TYPE_GUINT64:
+		uint64_to_string_buf(p->data.guint64.def[0], s, sizeof s);
+		break;
+	case PROP_TYPE_TIMESTAMP:
+		uint64_to_string_buf(p->data.timestamp.def[0], s, sizeof s);
+		break;
+	case PROP_TYPE_STRING:
+		g_strlcpy(s, *p->data.string.def ? *p->data.string.def : "", sizeof s);
+		break;
+	case PROP_TYPE_IP:
+		g_strlcpy(s, ip_to_string(p->data.guint32.def[0]), sizeof s);
+		break;
+	case PROP_TYPE_BOOLEAN:
+		g_strlcpy(s, p->data.boolean.def[0] ? "TRUE" : "FALSE", sizeof s);
+		break;
+	case PROP_TYPE_MULTICHOICE:
+		{
+			guint n = 0;
+
+			while (
+				p->data.guint32.choices[n].title != NULL &&
+				p->data.guint32.choices[n].value != *p->data.guint32.def
+			  )
+				n++;
+
+			if (p->data.guint32.choices[n].title != NULL)
+				gm_snprintf(s, sizeof s, "%u: %s",
+					*(p->data.guint32.def), p->data.guint32.choices[n].title);
+			else
+				gm_snprintf(s, sizeof s,
+					"%u: No descriptive string found for this value",
+					*(p->data.guint32.def));
 		}
-		default:
-			s[0] = '\0';
-			g_error("update_entry_gnet: incompatible type %s",
-				prop_type_str[PROP(ps,prop).type]);
+		break;
+	case PROP_TYPE_STORAGE:
+		g_strlcpy(s, guid_hex_str(prop_get_storage(ps, prop, NULL,
+			PROP(ps,prop).vector_size)), sizeof s);
+		break;
+	default:
+		s[0] = '\0';
+		g_error("update_entry_gnet: incompatible type %s",
+			prop_type_str[PROP(ps,prop).type].name);
 	}
 
 	return s;
@@ -1084,7 +1391,7 @@ prop_to_string(prop_set_t *ps, property_t prop)
 
 
 /**
- * Returns "TRUE" or "FALSE" depending on the given boolean value.
+ * @return "TRUE" or "FALSE" depending on the given boolean value.
  */
 static const gchar *
 config_boolean(gboolean b)
@@ -1152,7 +1459,7 @@ prop_save_to_file_if_dirty(prop_set_t *ps, const gchar *dir,
 
 /**
  * Read the all properties from the given property set and stores them
- * along with thier description to the given file in the given directory.
+ * along with their description to the given file in the given directory.
  * If this file was modified since the property set was read from it at
  * startup, the modifies file will be renamed to [filename].old before
  * saving.
@@ -1161,8 +1468,7 @@ void
 prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 {
 	FILE *config;
-	time_t mtime = 0;
-	struct stat buf;
+	struct stat sb;
 	gchar *newfile;
 	gchar *pathname;
 	guint n;
@@ -1180,23 +1486,24 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 	pathname = make_pathname(dir, filename);
 	g_return_if_fail(NULL != pathname);
 
-	if (-1 == stat(pathname, &buf))
+	if (-1 == stat(pathname, &sb)) {
 		g_warning("could not stat \"%s\": %s", pathname, g_strerror(errno));
-	else
-		mtime = buf.st_mtime;
+	} else {
+		/*
+		 * Rename old config file if they changed it whilst we were running.
+		 */
 
-	/*
-	 * Rename old config file if they changed it whilst we were running.
-	 */
-
-	if (ps->mtime && mtime > ps->mtime) {
-		gchar *old = g_strconcat(pathname, ".old", (void *) 0);
-		g_warning("config file \"%s\" changed whilst I was running", pathname);
-		if (-1 == rename(pathname, old))
-			g_warning("unable to rename as \"%s\": %s", old, g_strerror(errno));
-		else
-			g_warning("renamed old copy as \"%s\"", old);
-		G_FREE_NULL(old);
+		if (ps->mtime && delta_time(sb.st_mtime, ps->mtime) > 0) {
+			gchar *old = g_strconcat(pathname, ".old", (void *) 0);
+			g_warning("config file \"%s\" changed whilst I was running",
+				pathname);
+			if (-1 == rename(pathname, old))
+				g_warning("unable to rename as \"%s\": %s",
+					old, g_strerror(errno));
+			else
+				g_warning("renamed old copy as \"%s\"", old);
+			G_FREE_NULL(old);
+		}
 	}
 
 	/*
@@ -1246,8 +1553,8 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		case PROP_TYPE_BOOLEAN:
 			for (i = 0; i < p->vector_size; i++) {
 				gboolean v;
-			
-				v = p->data.boolean.value[i];	
+
+				v = p->data.boolean.value[i];
 				if (v != p->data.boolean.def[i])
 					defaultvalue = FALSE;
 				vbuf[i] = g_strdup(config_boolean(v));
@@ -1278,12 +1585,29 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 				v = p->data.guint64.value[i];
 				if (v != p->data.guint64.def[i])
 					defaultvalue = FALSE;
-				vbuf[i] = g_strdup(uint64_to_string_buf(sbuf, sizeof sbuf, v));
+
+				uint64_to_string_buf(v, sbuf, sizeof sbuf);
+				vbuf[i] = g_strdup(sbuf);
 			}
 			vbuf[p->vector_size] = NULL;
 
 			val = g_strjoinv(",", vbuf);
 			break;
+		case PROP_TYPE_TIMESTAMP:
+			for (i = 0; i < p->vector_size; i++) {
+				time_t t;
+
+				t = p->data.timestamp.value[i];
+				if (t != p->data.timestamp.def[i])
+					defaultvalue = FALSE;
+
+				timestamp_to_string_buf(t, sbuf, sizeof sbuf);
+				vbuf[i] = g_strdup(sbuf);
+			}
+			vbuf[p->vector_size] = NULL;
+			val = g_strjoinv(",", vbuf);
+			quotes = TRUE;
+			break;
 		case PROP_TYPE_STRING:
 			val = g_strdup(*p->data.string.value);
 			if (
@@ -1301,7 +1625,7 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 				v = p->data.guint32.value[i];
 				if (v != p->data.guint32.def[i])
 					defaultvalue = FALSE;
-				vbuf[i] = g_strdup(ip_to_gchar(v));
+				vbuf[i] = g_strdup(ip_to_string(v));
 			}
 			vbuf[p->vector_size] = NULL;
 
@@ -1325,6 +1649,8 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 			/* No default values for storage type properties. */
 			defaultvalue = FALSE;
 			break;
+		case NUM_PROP_TYPES:
+			g_assert_not_reached();
 		}
 
 		g_assert(val != NULL);
@@ -1347,7 +1673,7 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		if (-1 == rename(newfile, pathname))
 			g_warning("could not rename %s as %s: %s",
 				newfile, pathname, g_strerror(errno));
-		ps->mtime = time(NULL);
+		ps->mtime = tm_time_exact();
 	} else
 		g_warning("could not flush %s: %s", newfile, g_strerror(errno));
 
@@ -1369,15 +1695,16 @@ prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
 		gboolean	boolean[100];
 		guint32		uint32[100];
 		guint64		uint64[100];
+		time_t		timestamp[100];
 	} vecbuf;
 
 	g_assert(NULL != ps);
 	g_assert(NULL != val);
 	g_return_if_fail(prop >= ps->offset && prop < ps->offset + ps->size);
-	
+
 	p = &PROP(ps, prop);
 	g_return_if_fail(NULL != p);
-	
+
 	if (!p->save && saved_only) {
 		g_warning("Refusing to load run-time only property \"%s\"", p->name);
 		return;
@@ -1414,6 +1741,16 @@ prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
 		prop_parse_guint64_vector(p->name, val, p->vector_size, vecbuf.uint64);
 		stub->guint64.set(prop, vecbuf.uint64, 0, 0);
 		break;
+	case PROP_TYPE_TIMESTAMP:
+		prop_assert(ps, prop,
+			p->vector_size * sizeof(time_t) < sizeof(vecbuf.timestamp));
+
+		/* Initialize vector with defaults */
+		stub->timestamp.get(prop, vecbuf.timestamp, 0, 0);
+		prop_parse_timestamp_vector(p->name, val,
+			p->vector_size, vecbuf.timestamp);
+		stub->timestamp.set(prop, vecbuf.timestamp, 0, 0);
+		break;
 	case PROP_TYPE_STRING:
 		stub->string.set(prop, val);
 		break;
@@ -1444,6 +1781,8 @@ prop_set_from_string(prop_set_t *ps, property_t prop, const gchar *val,
 			G_FREE_NULL(d);
 		}
 		break;
+	case NUM_PROP_TYPES:
+		g_assert_not_reached();
 	}
 
 	G_FREE_NULL(stub);
@@ -1617,7 +1956,7 @@ prop_get_by_regex(prop_set_t *ps, const gchar *pattern, gint *error)
 			*error = ret;
 		return NULL;
 	}
-	
+
 	for (i = 0; i < ps->size; i++) {
 		if (0 == regexec(&re, ps->props[i].name, 0, NULL, 0))
 			sl = g_slist_prepend(sl, GUINT_TO_POINTER(ps->offset + i));
