@@ -1,9 +1,7 @@
 /*
- * $Id: tm.h,v 1.3 2005/05/04 01:50:42 cbiere Exp $
+ * $Id: tm.h,v 1.6 2005/09/16 21:51:11 rmanfredi Exp $
  *
- * Copyright (c) 2003, Raphael Manfredi
- *
- * Time manipulation routines.
+ * Copyright (c) 2003-2005, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -25,26 +23,36 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Time manipulation and caching routines.
+ *
+ * @author Raphael Manfredi
+ * @date 2003-2005
+ */
+
 #ifndef _tm_h_
 #define _tm_h_
 
 #include "common.h"
 
-/*
+/**
  * tm_zero
  *
  * Returns true if time is zero.
  */
 #define tm_zero(t)	((t)->tv_sec == 0 && (t)->tv_usec == 0)
 
-/*
+/**
  * tm2f
  *
  * Convert timeval description into floating point representatiion.
  */
 #define tm2f(t)		((double) (t)->tv_sec + (t)->tv_usec / 1000000.0)
 
-/*
+/**
  * tm2ms
  *
  * Convert timeval description into milliseconds.
@@ -58,7 +66,12 @@ void tm_elapsed(tm_t *elapsed, const tm_t *last, const tm_t *old);
 void tm_sub(tm_t *tm, const tm_t *dec);
 void tm_add(tm_t *tm, const tm_t *inc);
 int tm_cmp(const tm_t *a, const tm_t *b);
+
 void tm_now(tm_t *tm);
+void tm_now_exact(tm_t *tm);
+time_t tm_time(void);
+time_t tm_time_exact(void);
+gdouble tm_cputime(gdouble *user, gdouble *sys);
 
 guint tm_hash(gconstpointer key);
 gint tm_equal(gconstpointer a, gconstpointer b);
