@@ -1,5 +1,5 @@
 /*
- * $Id: utf8.h,v 1.21 2005/07/26 20:49:19 cbiere Exp $
+ * $Id: utf8.h,v 1.40 2005/11/18 22:31:04 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -56,13 +56,37 @@ typedef enum {
 	NUM_UNI_NORM
 } uni_norm_t;
 
+/*
+ * Gtk+ renderers want UTF-8 NFC
+ */
+#define UNI_NORM_GUI UNI_NORM_NFC
+
+/*
+ * NFC is more dense than NFD, thus it's the normalization of choice when
+ * passing text over the wire.
+ */
+#define UNI_NORM_NETWORK UNI_NORM_NFC
+
+#if defined(__APPLE__) && defined(__MACH__) /* Darwin */
+/* Mac OS X (Darwin) wants filenames always in UTF-8 NFD */
+#define UNI_NORM_FILESYSTEM UNI_NORM_NFD
+#else /* !Darwin */
+/* Unix systems usually use NFC for UTF-8 filenames */
+#define UNI_NORM_FILESYSTEM UNI_NORM_NFC
+#endif /* Darwin */
+
 void locale_init(void);
 void locale_close(void);
 const gchar *locale_get_charset(void);
-gint utf8_is_valid_char(const gchar *s);
-size_t utf8_is_valid_string(const gchar *s, size_t len);
+const gchar * locale_get_language(void);
+guint utf8_char_len(const gchar *s);
+gboolean is_ascii_string(const gchar *str);
+gboolean utf8_is_valid_string(const gchar *s);
+gboolean utf8_is_valid_data(const gchar *s, size_t n);
+size_t utf8_char_count(const gchar *s);
 size_t utf8_strlcpy(gchar *dst, const gchar *src, size_t dst_size);
-guint32 utf8_decode_char(const gchar *s, gint len, gint *retlen, gboolean warn);
+guint32 utf8_decode_char_fast(const gchar *s, guint *retlen)
+	NON_NULL_PARAM((1,2));
 gint utf8_to_iso8859(gchar *s, gint len, gboolean space);
 size_t utf8_strlower(gchar *dst, const gchar *src, size_t size);
 gchar *utf8_strlower_copy(const gchar *src);
@@ -100,7 +124,7 @@ utf8_decode_lookahead(const gchar *s, size_t len)
  * See also RFC 2781.
  *
  * @param uc the unicode character to encode.
- * @returns 0 if the unicode character is invalid. Otherwise the
+ * @returns (guint32) -1 if the unicode character is invalid. Otherwise the
  *         	UTF-16 encoded character is returned in a compact form:
  *			The lower 16 bits are the first UTF-16 character, the
  *			upper 16 bits are the second one. If the upper bits are
@@ -119,35 +143,53 @@ utf16_encode_char_compact(guint32 uc)
 		w2 = (uc & 0x3ff) | 0xdc00;
 		return (w2 << 16) | w1;
 	}
-	return 0;
+	return (guint32) -1;
 }
 
-/*
- * Necessary for GTK+ 2.x version because it expects almost any string
- * to be encoded as UTF-8.
+/**
+ * Lazy converters either return a pointer to a static buffer or manage
+ * the allocated memory themselves. They may also return the original
+ * pointer. Copy the result before calling them again unless you don't
+ * need the previous result anymore.
  */
-const gchar *lazy_iso8859_1_to_utf8(const gchar *s);
-const gchar *lazy_locale_to_utf8(const gchar *str);
-const gchar *locale_to_utf8(const gchar *str, size_t len);
-gchar *locale_to_utf8_full(const gchar *str);
+const gchar *lazy_iso8859_1_to_utf8(const gchar *src);
 
-/* Necessary for Mac OS X, as it requires filenames to be UTF-8 encoded
- * with all characters decomposed (NFD).
- */
+const gchar *lazy_ui_string_to_utf8(const gchar *src);
+const gchar *lazy_utf8_to_ui_string(const gchar *src);
+
+const gchar *lazy_utf8_to_locale(const gchar *src);
+const gchar *lazy_locale_to_utf8(const gchar *src);
+
+const gchar *lazy_locale_to_ui_string(const gchar *src);
+const gchar *lazy_locale_to_ui_string2(const gchar *src);
+
+const gchar *lazy_locale_to_utf8_normalized(const gchar *src, uni_norm_t norm);
+const gchar *lazy_unknown_to_utf8_normalized(const gchar *src, uni_norm_t norm,
+				const gchar **charset_ptr);
+
+gchar *iso8859_1_to_utf8(const gchar *str);
+gchar *iso8859_1_to_utf8_normalized(const gchar *str, uni_norm_t norm);
+
+gchar *utf8_to_ui_string(const gchar *src);
+gchar *ui_string_to_utf8(const gchar *src);
+
+gchar *utf8_to_locale(const gchar *s);
+gchar *locale_to_utf8(const gchar *str);
 gchar *locale_to_utf8_normalized(const gchar *str, uni_norm_t norm);
 
+gchar *utf8_to_filename(const gchar *s);
+gchar *filename_to_utf8_normalized(const gchar *str, uni_norm_t norm);
 
-/*
- * Necessary for GTK+ 1.2 version because it expects almost any string
- * to be in locale, but xml is stored in utf-8
- */
+gchar *unknown_to_utf8(const gchar *str, const gchar **charset_ptr);
+gchar *unknown_to_utf8_normalized(const gchar *src, uni_norm_t norm,
+			const gchar **charset_ptr);
 
-gboolean is_ascii_string(const gchar *str);
-const gchar *utf8_to_locale(const gchar *str, size_t len);
-const gchar *lazy_utf8_to_locale(const gchar *str);
+size_t ascii_enforce(gchar *dst, size_t size, const gchar *src);
+size_t utf8_enforce(gchar *dst, size_t size, const gchar *src);
 
 gboolean icu_enabled(void);
-gboolean is_latin_locale(void);
+gboolean locale_is_latin(void);
+gboolean locale_is_utf8(void);
 
 #if 0  /* xxxUSE_ICU */
 
