@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.234 2005/07/29 01:42:09 cbiere Exp $
+ * $Id: main.c,v 1.264 2005/11/29 22:14:55 rmanfredi Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -72,28 +72,34 @@
 #include "core/pcache.h"
 #include "core/pproxy.h"
 #include "core/routing.h"
+#include "core/rx.h"
 #include "core/search.h"
 #include "core/settings.h"
 #include "core/share.h"
 #include "core/sockets.h"
 #include "core/sq.h"
 #include "core/tsync.h"
+#include "core/tx.h"
 #include "core/uhc.h"
 #include "core/upload_stats.h"
 #include "core/verify.h"
 #include "core/version.h"
+#include "core/vmsg.h"
 #include "core/whitelist.h"
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/bg.h"
 #include "lib/cq.h"
 #include "lib/crc.h"
+#include "lib/dbus_util.h"
 #include "lib/eval.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/pattern.h"
 #include "lib/tiger.h"
+#include "lib/tm.h"
 #include "lib/utf8.h"
+#include "lib/vendors.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 #include "lib/wordvec.h"
@@ -112,7 +118,7 @@
 #include "if/ui/gtk/search.h"
 #endif /* GTK */
 
-
+#include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
 
@@ -122,18 +128,22 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: main.c,v 1.234 2005/07/29 01:42:09 cbiere Exp $");
+RCSID("$Id: main.c,v 1.264 2005/11/29 22:14:55 rmanfredi Exp $");
 
-#define SLOW_UPDATE_PERIOD		20	/**< Updating period for `main_slow_update' */
+#define SLOW_UPDATE_PERIOD		20	/**< Update period for `main_slow_update' */
 #define EXIT_GRACE				30	/**< Seconds to wait before exiting */
 #define ATEXIT_TIMEOUT			20	/**< Final cleanup must not take longer */
 
+#define LOAD_HIGH_WATERMARK		90	/**< % amount over which we're overloaded */
+#define LOAD_LOW_WATERMARK		55	/**< lower threshold to clear condition */
+
 static guint main_slow_update = 0;
 static gboolean exiting = FALSE;
 static gboolean from_atexit = FALSE;
 static gint signal_received = 0;
 static jmp_buf atexit_env;
 static volatile gchar *exit_step = "gtk_gnutella_exit";
+static tm_t start_time;
 
 void gtk_gnutella_exit(gint n);
 
@@ -167,6 +177,47 @@ sig_malloc(int n)
 }
 #endif /* MALLOC_STATS */
 
+/**
+ * Are we debugging anything at a level greater than some threshold "t"?
+ */
+gboolean
+debugging(guint t)
+{
+	return
+		ban_debug > t ||
+		bitzi_debug > t ||
+		bootstrap_debug > t ||
+		dbg > t ||
+		dh_debug > t ||
+		dmesh_debug > t ||
+		download_debug > t ||
+		dq_debug > t ||
+		fileinfo_debug > t ||
+		ggep_debug > t ||
+		gmsg_debug > t ||
+		gwc_debug > t ||
+		hsep_debug > t ||
+		http_debug > t ||
+		lib_debug > t ||
+		node_debug > t ||
+		parq_debug > t ||
+		pcache_debug > t ||
+		qrp_debug > t ||
+		query_debug > t ||
+		routing_debug > t ||
+		search_debug > t ||
+		share_debug > t ||
+		socket_debug > t ||
+		tls_debug > t ||
+		udp_debug > t ||
+		upload_debug > t ||
+		url_debug > t ||
+		vmsg_debug > t ||
+
+		/* Above line left blank for easy "!}sort" under vi */
+		0;
+}
+
 /**
  * Invoked as an atexit() callback when someone does an exit().
  */
@@ -183,7 +234,10 @@ gtk_gnutella_atexit(void)
 	if (!exiting) {
 		g_warning("trapped foreign exit(), cleaning up...");
 		from_atexit = TRUE;
+
+#ifdef SIGALRM
 		set_signal(SIGALRM, sig_alarm);
+#endif
 		if (setjmp(atexit_env)) {
 			g_warning("cleanup aborted while in %s().", exit_step);
 			return;
@@ -195,6 +249,48 @@ gtk_gnutella_atexit(void)
 	}
 }
 
+/**
+ * Log cpu used since last time.
+ *
+ * @param since_time	time at which the measurement period started, updated
+ * @param prev_user		previous total user time, updated if not NULL
+ * @param prev_sys		previous total system time, updated if not NULL
+ */
+static void
+log_cpu_usage(tm_t *since_time, gdouble *prev_user, gdouble *prev_sys)
+{
+	gdouble user;
+	gdouble sys;
+	gdouble total;
+	tm_t cur_time;
+	tm_t elapsed_time;
+	gdouble elapsed;
+
+	tm_now_exact(&cur_time);
+	total = tm_cputime(&user, &sys);
+	if (prev_user) {
+		gdouble v = *prev_user;
+		*prev_user = user;
+		user -= v;
+		total -= v;
+	}
+	if (prev_sys) {
+		gdouble v = *prev_sys;
+		*prev_sys = sys;
+		sys -= v;
+		total -= v;
+	}
+
+	tm_elapsed(&elapsed_time, &cur_time, since_time);
+	elapsed = tm2f(&elapsed_time);
+	*since_time = cur_time;
+
+	g_message("average CPU used: %.3f%% over %.2f secs",
+		100.0 * total / elapsed, elapsed);
+	g_message("CPU usage: total: %.2fs (user: %.2f, sys: %.2f)",
+		total, user, sys);
+}
+
 /**
  * Exit program, return status `n' to parent process.
  *
@@ -206,9 +302,17 @@ gtk_gnutella_exit(gint n)
 {
 	time_t exit_time = time(NULL);
 	gint exit_grace = EXIT_GRACE;
+	static gboolean safe_to_exit = FALSE;
 
-	if (exiting)
-		return;			/* Already exiting, must be in loop below */
+	if (exiting) {
+		if (safe_to_exit) {
+			g_warning("forced exit(%d), good bye.", n);
+			exit(n);
+		}
+		g_warning("ignoring re-entrant exit(%d), unsafe now (in %s)",
+			n, exit_step);
+		return;
+	}
 
 	exiting = TRUE;
 
@@ -240,6 +344,8 @@ gtk_gnutella_exit(gint n)
 	DO(settings_save_if_dirty);
 	DO(settings_gui_save_if_dirty);
 
+	safe_to_exit = TRUE;	/* Will immediately exit if re-entered */
+
 	if (from_atexit)
 		return;
 
@@ -257,6 +363,16 @@ gtk_gnutella_exit(gint n)
 
 	settings_gui_shutdown();
 
+	/*
+	 * Show total CPU used, and the amount spent in user / kernel, before
+	 * we start the grace period...
+	 */
+
+	if (debugging(0)) {
+		tm_t since = start_time;
+		log_cpu_usage(&since, NULL, NULL);
+	}
+
 	/*
 	 * Wait at most EXIT_GRACE seconds, so that BYE messages can go through.
 	 * This amount of time is doubled when running in Ultra mode since we
@@ -313,6 +429,7 @@ gtk_gnutella_exit(gint n)
 	iso3166_close();
 	atom_str_free(start_rfc822_date);
 	adns_close();
+	dbus_util_close();  /* After adns_close() to avoid strange crashes */
 	atoms_close();
 	wdestroy();
 	locale_close();
@@ -321,8 +438,8 @@ gtk_gnutella_exit(gint n)
 #endif
 
 
-	if (dbg)
-		printf("gtk-gnutella shut down cleanly.\n\n");
+	if (debugging(0) || signal_received)
+		g_message("gtk-gnutella shut down cleanly.");
 
 #if defined(USE_GTK1) || defined(USE_GTK2)
 	gtk_exit(n);
@@ -353,6 +470,17 @@ slow_main_timer(time_t now)
 	static guint i = 0;
 	static time_t last_warn = 0;
 
+	if (cpu_debug) {
+		static tm_t since = { 0, 0 };
+		static gdouble user = 0.0;
+		static gdouble sys = 0.0;
+
+		if (since.tv_sec == 0)
+			since = start_time;
+
+		log_cpu_usage(&since, &user, &sys);
+	}
+
 	switch (i) {
 	case 0:
 		dmesh_store();
@@ -373,13 +501,15 @@ slow_main_timer(time_t now)
 		hcache_store_if_dirty(HOST_ULTRA);
 		break;
 	default:
-		g_assert(0);
+		g_assert_not_reached();
 	}
 	i = (i + 1) % 5;
 
 	download_store_if_dirty();		/* Important, so always attempt it */
 	settings_save_if_dirty();		/* Nice to have, and file is small */
 	settings_gui_save_if_dirty();	/* Ditto */
+	tx_collect();					/* Collect freed TX stacks */
+	rx_collect();					/* Idem for freed RX stacks */
 
 	node_slow_timer(now);
 	ignore_timer(now);
@@ -394,10 +524,107 @@ slow_main_timer(time_t now)
 void icon_timer(void);
 #endif /* USE_TOPLESS */
 
+/**
+ * Check CPU usage.
+ *
+ * @return current (exact) system time.
+ */
+static time_t
+check_cpu_usage(void)
+{
+	static tm_t last_tm;
+	static gdouble last_cpu = 0.0;
+	static gint ticks = 0;
+	static gint load_avg = 0;		/* 100 * cpu% for integer arithmetic */
+	static gint avg = 0;			/* cpu% */
+	tm_t cur_tm;
+	tm_t elapsed_tm;
+	gint load = 0;
+	gdouble cpu;
+	gdouble elapsed;
+	gdouble cpu_percent;
+	gdouble coverage;
+
+	/*
+	 * Compute CPU time used this period.
+	 */
+
+	tm_now_exact(&cur_tm);
+	cpu = tm_cputime(NULL, NULL);
+	tm_elapsed(&elapsed_tm, &cur_tm, &last_tm);
+
+	elapsed = tm2f(&elapsed_tm);
+	if (elapsed == 0.0)
+		elapsed = 0.000001;			/* Paranoid: avoid division by zero */
+
+	cpu_percent = 100.0 * (cpu - last_cpu) / elapsed,
+
+	coverage = callout_queue_coverage(ticks);
+	if (coverage == 0.0)
+		coverage = 0.001;			/* Paranoid again */
+
+	/*
+	 * Correct the percentage of CPU that would have been actually used
+	 * if we had had 100% of the CPU scheduling time.  We use the callout
+	 * queue as a rough estimation of the CPU running time we had: the less
+	 * ticks were received by the callout queue, the busier the CPU was
+	 * running other things.  But we can be busy running our own code,
+	 * not really because the CPU is used by other processes, so we cannot
+	 * just divide by the coverage ratio.
+	 *
+	 * The average load is computed using a medium exponential moving average.
+	 */
+
+	if (coverage <= 0.1)
+		cpu_percent *= 4;
+	else if (coverage <= 0.2)
+		cpu_percent *= 3;
+	else if (coverage <= 0.5)
+		cpu_percent *= 1.5;
+
+	load = (gint) cpu_percent * 100;
+	load_avg += (load >> 3) - (load_avg >> 3);
+	avg = load_avg / 100;
+
+	if (dbg > 1 && last_cpu)
+		g_message("CPU: %.3f secs in %.3f secs (~%.3f%% @ cover=%.2f) avg=%d%%",
+			cpu - last_cpu, elapsed, cpu_percent, coverage, avg);
+
+	/*
+	 * Update for next time.
+	 */
+
+	last_cpu = cpu;
+	last_tm = cur_tm;		/* Struct copy */
+	ticks = cq_ticks(callout_queue);
+
+	/*
+	 * Check whether we're overloaded, or if we were, whether we decreased
+	 * the average load enough to disable the "overloaded" condition.
+	 */
+
+	if (avg >= LOAD_HIGH_WATERMARK && !overloaded_cpu) {
+		if (debugging(0))
+			g_message("high average CPU load (%d%%), entering overloaded state",
+				avg);
+		gnet_prop_set_boolean_val(PROP_OVERLOADED_CPU, TRUE);
+	} else if (overloaded_cpu && avg < LOAD_LOW_WATERMARK) {
+		if (debugging(0))
+			g_message("average CPU load (%d%%) low, leaving overloaded state",
+				avg);
+		gnet_prop_set_boolean_val(PROP_OVERLOADED_CPU, FALSE);
+	}
+
+	return tm_time();		/* Exact, since tm_now_exact() called on entry */
+}
+
+/**
+ * Main timer routine, called once per second.
+ */
 static gboolean
 main_timer(gpointer p)
 {
-	time_t now = time(NULL);
+	time_t now;
 
 	(void) p;
 	if (signal_received) {
@@ -405,6 +632,8 @@ main_timer(gpointer p)
 		gtk_gnutella_exit(1);
 	}
 
+	now = check_cpu_usage();
+
 #ifdef MALLOC_STATS
 	if (signal_malloc) {
 		if (signal_malloc == 1)
@@ -417,7 +646,7 @@ main_timer(gpointer p)
 
 	bsched_timer();					/* Scheduling update */
 	host_timer();					/* Host connection */
-    hcache_timer();
+    hcache_timer(now);
 	node_timer(now);				/* Node timeouts */
 	http_timer(now);				/* HTTP request timeouts */
 	if (!exiting) {
@@ -440,7 +669,7 @@ main_timer(gpointer p)
 	 */
 
 	if (!exiting) {
-		main_gui_timer();
+		main_gui_timer(now);
 
 		/* Update for things that change slowly */
 		if (main_slow_update++ > SLOW_UPDATE_PERIOD) {
@@ -449,8 +678,15 @@ main_timer(gpointer p)
 		}
 	}
 
-	icon_timer();
-	bg_sched_timer();				/* Background tasks */
+	/*
+	 * The following are low-priority tasks, not called if we've
+	 * detected a high CPU load.
+	 */
+
+	if (!overloaded_cpu) {
+		icon_timer();
+		bg_sched_timer();			/* Background tasks */
+	}
 
 	return TRUE;
 }
@@ -469,59 +705,60 @@ scan_files_once(gpointer p)
 	return FALSE;
 }
 
+static const gchar * const log_domains[] = {
+	G_LOG_DOMAIN, "Gtk", "GLib", "Pango"
+};
+
 static void
-log_handler(const gchar *log_domain, GLogLevelFlags log_level,
+log_handler(const gchar *domain, GLogLevelFlags level,
 	const gchar *message, gpointer user_data)
 {
 	gint saved_errno = errno;
 	time_t now;
 	struct tm *ct;
-	const char *level;
+	const char *prefix;
 	gchar *safer;
 
-	(void) log_domain;
+	(void) domain;
 	(void) user_data;
-	now = time(NULL);
+	now = tm_time();
 	ct = localtime(&now);
 
-	switch (log_level) {
-	case G_LOG_LEVEL_CRITICAL:
-		level = "CRITICAL";
-		break;
-	case G_LOG_LEVEL_ERROR:
-		level = "ERROR";
-		break;
-	case G_LOG_LEVEL_WARNING:
-		level = "WARNING";
-		break;
-	case G_LOG_LEVEL_MESSAGE:
-		level = "MESSAGE";
-		break;
-	case G_LOG_LEVEL_INFO:
-		level = "INFO";
-		break;
-	case G_LOG_LEVEL_DEBUG:
-		level = "DEBUG";
-		break;
+	switch (level) {
+#define CASE(x) case CAT2(G_LOG_LEVEL_,x): prefix = STRINGIFY(x); break;
+
+	CASE(CRITICAL)
+	CASE(ERROR)
+	CASE(WARNING)
+	CASE(MESSAGE)
+	CASE(INFO)
+	CASE(DEBUG)
+#undef CASE
 	default:
-		level = "UNKNOWN";
+		prefix = "UNKNOWN";
 	}
 
-	safer = hex_escape(message, FALSE); /* non-strict escaping */
+	safer = control_escape(message);
 
-	fprintf(stderr, "%.2d/%.2d/%.2d %.2d:%.2d:%.2d (%s): %s\n",
-		ct->tm_year % 100, ct->tm_mon + 1, ct->tm_mday,
-		ct->tm_hour, ct->tm_min, ct->tm_sec, level, safer);
+	fprintf(stderr, "%02d-%02d-%02d %.2d:%.2d:%.2d (%s): %s\n",
+		(1900 + ct->tm_year) % 100, ct->tm_mon + 1, ct->tm_mday,
+		ct->tm_hour, ct->tm_min, ct->tm_sec, prefix, safer);
 
 	if (safer != message)
 		G_FREE_NULL(safer);
 
-#if 0
+#if 0 
 	/* Define to debug Glib or Gtk problems */
-	if (log_domain &&
-		(!strcmp(log_domain, "Gtk") || !strcmp(log_domain, "GLib"))
-	) {
-		raise(SIGTRAP);
+	if (domain) {
+		guint i;
+
+		for (i = 0; i < G_N_ELEMENTS(log_domains); i++) {
+			const gchar *dom = log_domains[i];
+			if (dom && 0 == strcmp(domain, dom)) {
+				raise(SIGTRAP);
+				break;
+			}
+		}
 	}
 #endif
 
@@ -531,34 +768,102 @@ log_handler(const gchar *log_domain, GLogLevelFlags log_level,
 static void
 log_init(void)
 {
-	const gchar *domains[] = { G_LOG_DOMAIN, "Gtk", "GLib" };
 	guint i;
 
-	for (i = 0; i < G_N_ELEMENTS(domains); i++) {
-		g_log_set_handler(domains[i],
+	for (i = 0; i < G_N_ELEMENTS(log_domains); i++) {
+		g_log_set_handler(log_domains[i],
 			G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING |
 			G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG,
 			log_handler, NULL);
 	}
 }
 
+/**
+ * Closes all file descriptors greater or equal to ``fd''.
+ */
+static void
+close_fds(gint fd)
+{
+	g_assert(fd >= 0);
+
+#ifdef F_CLOSEM
+	if (-1 == fcntl(fd, F_CLOSEM))
+#endif
+	{
+		gdouble start;
+		gint num_fds = compat_max_fd();
+
+		start = tm_cputime(NULL, NULL);
+		for (/* NOTHING */; fd < num_fds; fd++) {
+			close(fd);
+
+			/* Just in case we're trying to close a bazillion fds on a vax */
+			if (0 == (fd & 0xff) && tm_cputime(NULL, NULL) - start > 5) {
+				g_warning("Aborted closing file descriptors after "
+					"exceeding 5s CPU time (fd=%d)", fd);
+				break;
+			}
+		}
+	}
+}
+
 extern char **environ;
 
+#ifdef FAST_ASSERTIONS
+const char *assert_msg_;
+
+/**
+ * This a SIGSEGV signal handler used for "fast" assertions.
+ *
+ * NOTE: The code inside must be signal-safe. See also:
+ * http://www.opengroup.org/onlinepubs/009695399/functions/xsh_chap02_04.html
+ */
+static void
+assertion_failure(int signo)
+{
+	(void) signo;
+
+	/* Prevent looping, if the following crashes. */
+	set_signal(signo, SIG_DFL);
+	if (!assert_msg_)
+		assert_msg_ = SIGSEGV == signo ? "\nSegmentation fault\n" : "\nTrap\n";
+	write(STDERR_FILENO, assert_msg_, strlen(assert_msg_));
+
+	raise(signo);
+}
+#endif /* FAST_ASSERTIONS */
+
+static void
+assertion_init(void)
+{
+#ifdef FAST_ASSERTIONS
+#if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__))
+	set_signal(SIGTRAP, assertion_failure);
+#else
+	set_signal(SIGSEGV, assertion_failure);
+#endif	/* GCC/x86 */
+#endif	/* FAST_ASSERTIONS */
+}
+
 int
 main(int argc, char **argv)
 {
-	gint i;
+	assertion_init();
+	misc_init();
 
-	if (0 == getuid() || 0 == geteuid()) {
+	tm_now_exact(&start_time);
+
+	if (compat_is_superuser()) {
 		fprintf(stderr, "Never ever run this as root!\n");
 		exit(EXIT_FAILURE);
 	}
 
-	for (i = 3; i < 256; i++)
-		close(i);					/* Just in case */
+	close_fds(3); /* Just in case */
 
 	set_signal(SIGINT, SIG_IGN);	/* ignore SIGINT in adns (e.g. for gdb) */
+#ifdef SIGPIPE
 	set_signal(SIGPIPE, SIG_IGN);
+#endif
 
 #ifdef MALLOC_STATS
 	set_signal(SIGUSR1, sig_malloc);
@@ -568,6 +873,7 @@ main(int argc, char **argv)
 	gm_savemain(argc, argv, environ);	/* For gm_setproctitle() */
 
 	/* Our inits */
+	(void) tm_time_exact();
 	log_init();
 #ifndef OFFICIAL_BUILD
 	g_warning("%s \"%s\"",
@@ -576,6 +882,11 @@ main(int argc, char **argv)
 		PACKAGE_SOURCE_DIR);
 #endif
 
+	/* If one of the two below fails, the GLib installation is broken. */
+	STATIC_ASSERT(sizeof(size_t) == sizeof(gsize));
+	STATIC_ASSERT(sizeof(ssize_t) == sizeof(gssize));
+
+	inputevt_init();
 	tiger_init();
 	random_init();
 	locale_init();
@@ -586,6 +897,9 @@ main(int argc, char **argv)
 	socket_init();
 	gnet_stats_init();
 	iso3166_init();
+	dbus_util_init();
+	vendor_init();
+	vmsg_init();
 
 	main_gui_early_init(argc, argv);
 
@@ -663,6 +977,7 @@ main(int argc, char **argv)
 
 	/* Okay, here we go */
 
+	(void) tm_time_exact();
 	bsched_enable_all();
 	version_ancient_warn();
 
