@@ -1,5 +1,5 @@
 /*
- * $Id: gtk-missing.c,v 1.22 2005/06/29 14:24:28 daichik Exp $
+ * $Id: gtk-missing.c,v 1.34 2005/12/19 15:31:20 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -37,7 +37,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c,v 1.22 2005/06/29 14:24:28 daichik Exp $");
+RCSID("$Id: gtk-missing.c,v 1.34 2005/12/19 15:31:20 cbiere Exp $");
 
 #include "gtk-missing.h"
 #include "columns.h"
@@ -85,44 +85,9 @@ gtk_clist_set_column_name(GtkCList * clist, gint col, gchar * t)
 
     clist->column[col].title = g_strdup(t);
 }
-
-/**
- * Save visibility of columns in given property which must by a boolean array
- * property with at least as many elements as there are columns.
- */
-void
-gtk_clist_save_visibility(GtkCList *clist, property_t prop)
-{
-	gint i;
-	gboolean val;
-
-	g_assert(clist);
-
-    for (i = 0; i < clist->columns; i++) {
-		val = clist->column[i].visible;
-		gui_prop_set_boolean(prop, &val, i, 1);
-	}
-}
-
-/**
- * Restore visibility of columns from given property which must by a boolean
- * array property with at least as many elements as there are columns.
- */
-void
-gtk_clist_restore_visibility(GtkCList *clist, property_t prop)
-{
-	gint i;
-	gboolean val;
-
-	g_assert(clist);
-
-    for (i = 0; i < clist->columns; i++) {
-		gui_prop_get_boolean(prop, &val, i, 1);
-    	gtk_clist_set_column_visibility(clist, i, val);
-	}
-}
 #endif /* USE_GTK1 */
 
+#define GTK_ITERATION_MAX	100		/* Don't spend too much time in GUI */
 
 /**
  * Process all pending gtk events (id est draw now!).
@@ -135,67 +100,62 @@ gint
 gtk_main_flush(void)
 {
     gint val = FALSE;
+	gint i = 0;
 
-    while (gtk_events_pending()) {
+    while (gtk_events_pending() && i++ < GTK_ITERATION_MAX) {
         val = gtk_main_iteration_do(FALSE);
 		if (val)
 			break;
 	}
 
+	if (i > GTK_ITERATION_MAX && !val)
+		g_warning("gtk_main_flush: too much work");
+
     return val;
 }
 
-
-
 /**
  * Select the menu item which has given data attached to it.
  */
 void
-option_menu_select_item_by_data(GtkWidget *m, gpointer *d)
+option_menu_select_item_by_data(GtkOptionMenu *option_menu, gconstpointer data)
 {
-    GList *l;
-    gint n = 0;
-    GtkWidget *menu;
+    GList *iter;
+    gint i;
 
-    g_assert(m != NULL);
-    g_assert(GTK_IS_OPTION_MENU(m));
+    g_assert(option_menu);
+    g_assert(GTK_IS_OPTION_MENU(option_menu));
 
-    menu = GTK_WIDGET(gtk_option_menu_get_menu(GTK_OPTION_MENU(m)));
-
-    for (l = GTK_MENU_SHELL(menu)->children; l != NULL; l = g_list_next(l)) {
-        if (l->data != NULL) {
-            if (gtk_object_get_user_data((GtkObject *) l->data) == d) {
-                gtk_option_menu_set_history(GTK_OPTION_MENU(m), n);
-                break;
-            }
+    iter = GTK_MENU_SHELL(gtk_option_menu_get_menu(option_menu))->children;
+    for (i = 0; iter != NULL; iter = g_list_next(iter), i++) {
+        if (iter->data && gtk_object_get_user_data(iter->data) == data) {
+			gtk_option_menu_set_history(option_menu, i);
+			return;
         }
-
-        n ++;
     }
 
-    if (l == NULL)
-        g_warning("option_menu_select_item_by_data: no item with data %p",
-			cast_to_gconstpointer(d));
+	g_warning("option_menu_select_item_by_data: no item with data %p", data);
 }
 
 
 
 /**
- * Add a new menu item to the GtkMenu m, with the label l and
- * the data d.
+ * Add a new menu item to the GtkMenu "menu" with the label "label_text" and
+ * the data "data".
  */
 GtkWidget *
-menu_new_item_with_data(GtkMenu * m, gchar * l, gpointer d )
+menu_new_item_with_data(GtkMenu *menu, const gchar *label_text, gpointer data)
 {
-    GtkWidget *w;
+    GtkWidget *widget;
 
-    g_assert(l != NULL);
+    g_assert(menu);
+    g_assert(label_text);
 
-    w = gtk_menu_item_new_with_label(l);
-    gtk_object_set_user_data((GtkObject *)w, (gpointer)d);
-    gtk_widget_show(w);
-    gtk_menu_append(m, w);
-    return w;
+    widget = gtk_menu_item_new_with_label(label_text);
+    gtk_object_set_user_data(GTK_OBJECT(widget), data);
+    gtk_widget_show(widget);
+    gtk_menu_append(menu, widget);
+    return widget;
 }
 
 
@@ -204,19 +164,16 @@ menu_new_item_with_data(GtkMenu * m, gchar * l, gpointer d )
  * the GtkOptionMenu m.
  */
 gpointer
-option_menu_get_selected_data(GtkWidget *m)
+option_menu_get_selected_data(GtkOptionMenu *option_menu)
 {
     GtkWidget *menu;
 
-    g_assert(GTK_IS_OPTION_MENU(m));
+    g_assert(GTK_IS_OPTION_MENU(option_menu));
 
-    menu = gtk_menu_get_active
-        (GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(m))));
-
-    return (menu != NULL) ? gtk_object_get_user_data(GTK_OBJECT(menu)) : NULL;
+    menu = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(option_menu)));
+    return menu ? gtk_object_get_user_data(GTK_OBJECT(menu)) : NULL;
 }
 
-
 /**
  * Given a radio button it returns a pointer to the active radio
  * button in the group the given button is in.
@@ -287,22 +244,19 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
 }
 
 /**
- * Takes a NULL terminated array of strings which are supposed to
- * be widgets names found with the given top-level widget. Sets all
- * to the given sentitivity state.
+ * Takes an array of strings which are supposed to be widgets names found with
+ * the given top-level widget. Sets all to the given sentitivity state.
  */
 void
-gtk_mass_widget_set_sensitive(GtkWidget *toplevel, gchar *list[], gboolean b)
+gtk_mass_widget_set_sensitive(GtkWidget *toplevel,
+	const gchar * const list[], guint n, gboolean b)
 {
-    guint n;
-    GtkWidget *w;
+    guint i;
 
     g_assert(toplevel != NULL);
 
-    for (n = 0; list[n] != NULL; n ++) {
-        w = lookup_widget(toplevel, list[n]);
-        gtk_widget_set_sensitive(w, b);
-    }
+    for (i = 0; i < n; i++)
+        gtk_widget_set_sensitive(lookup_widget(toplevel, list[i]), b);
 }
 
 /**
@@ -379,7 +333,7 @@ GtkTreeIter *
 w_tree_iter_new(void)
 {
 	GtkTreeIter *iter;
-	
+
 	iter = walloc(sizeof *iter);
 	return iter;
 }
@@ -422,21 +376,22 @@ typedef struct collect_data_struct {
     GSList *results;
     GSList *to_unselect;
 	GCompareFunc cfn;
+	tree_selection_get_data_func gdf;	
 	GtkTreeView *tv;
 	const gchar *name; /* name of the treeview widget (for debugging) */
+	guint column;
 } collect_data_struct_t;
 
 static inline void
 tree_selection_collect_data_record(GtkTreeModel *model, GtkTreeIter *iter,
 		collect_data_struct_t *cdata, gboolean unselect)
 {
-	gpointer data = NULL;
+	gpointer data;
 
 	g_assert(NULL != cdata);
 	g_assert(NULL != iter);
 
-	gtk_tree_model_get(model, iter, c_sr_record, &data, (-1));
-	g_assert(NULL != data);
+	data = cdata->gdf(model, iter);
 
 	if (unselect) {
 		cdata->to_unselect = g_slist_prepend(cdata->to_unselect,
@@ -483,7 +438,7 @@ tree_selection_unselect_helper(gpointer data, gpointer user_data)
 {
 	GtkTreeSelection *s = user_data;
 	GtkTreeIter *iter = data;
-	
+
 	gtk_tree_selection_unselect_iter(s, iter);
 	w_tree_iter_free(iter);
 }
@@ -495,7 +450,8 @@ tree_selection_unselect_helper(gpointer data, gpointer user_data)
  * items from the result list. Using cfn will significantly increase runtime.
  */
 GSList *
-tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
+tree_selection_collect_data(GtkTreeSelection *selection,
+	tree_selection_get_data_func gdf, GCompareFunc cfn)
 {
 	collect_data_struct_t cdata;
 
@@ -504,6 +460,7 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
 	cdata.results = NULL;
 	cdata.to_unselect = NULL;
 	cdata.cfn = cfn;
+	cdata.gdf = gdf;
 	cdata.tv = gtk_tree_selection_get_tree_view(selection);
 	if (gui_debug >= 3) {
 		cdata.name = gtk_widget_get_name(
@@ -517,7 +474,7 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
      * Browse selected rows and gather data.
      */
 	gtk_tree_selection_selected_foreach(selection,
-		tree_selection_collect_data_helper, (gpointer) &cdata);
+		tree_selection_collect_data_helper, &cdata);
 
     /*
      * Now unselect the rows from which we got data.
@@ -533,82 +490,6 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
     return cdata.results;
 }
 
-void
-tree_view_save_widths(GtkTreeView *treeview, property_t prop)
-{
-	gint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		guint32 width;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-
-		width = gtk_tree_view_column_get_width(c);
-		if ((gint) width > 0)
-			gui_prop_set_guint32(prop, &width, i, 1);
-	}
-}
-
-void
-tree_view_restore_widths(GtkTreeView *treeview, property_t prop)
-{
-	gint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		guint32 width;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		gui_prop_get_guint32(prop, &width, i, 1);
-		g_object_set(G_OBJECT(c),
-			"fixed-width", MAX(1, (gint32) width),
-			(void *) 0);
-	}
-}
-
-void
-tree_view_save_visibility(GtkTreeView *treeview, property_t prop)
-{
-	guint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		gboolean val;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		val = gtk_tree_view_column_get_visible(c);
-		gui_prop_set_boolean(prop, &val, i, 1);
-	}
-}
-
-void
-tree_view_restore_visibility(GtkTreeView *treeview, property_t prop)
-{
-	guint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		gboolean val;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		gui_prop_get_boolean(prop, &val, i, 1);
-		gtk_tree_view_column_set_visible(c, val);
-	}
-}
-
 struct tree_view_motion {
 	tree_view_motion_callback cb;
 	GtkTreeView *tv;
@@ -675,13 +556,13 @@ on_tree_view_motion_notify(GtkWidget *widget,
 		}
 #undef EVENT_TYPE
 
-		g_message("%s: type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
-				__func__, type,
+		g_message("on_tree_view_motion_notify(): type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
+				type,
 				(gint) event->x, (gint) event->y, event->axes,
 				(gint) event->x_root, (gint) event->y_root);
 	}
 #endif /* 0 */
-	
+
 	tvm->x = event->x;
 	tvm->y = event->y;
 	tvm->ready = TRUE;
@@ -707,12 +588,21 @@ tree_view_motion_timeout(gpointer data)
 		if (path)
 			gtk_tree_path_free(path);
 	}
-	
+
 	return TRUE;
 }
 
+/**
+ * Registers the callback for the motion-notify-event. Only the last event
+ * seen during any interval will invoke the callback.
+ *
+ * @param tv The GtkTreeView.
+ * @param cb The callback.
+ * @param interval The interval in milliseconds.
+ */
 tree_view_motion_t *
-tree_view_motion_set_callback(GtkTreeView *tv, tree_view_motion_callback cb)
+tree_view_motion_set_callback(GtkTreeView *tv,
+	tree_view_motion_callback cb, guint interval)
 {
 	tree_view_motion_t *tvm;
 
@@ -722,7 +612,7 @@ tree_view_motion_set_callback(GtkTreeView *tv, tree_view_motion_callback cb)
 	tvm = g_malloc(sizeof *tvm);
 	tvm->tv = tv;
 	tvm->cb = cb;
-	tvm->timeout_id = g_timeout_add(400, tree_view_motion_timeout, tvm);
+	tvm->timeout_id = g_timeout_add(interval, tree_view_motion_timeout, tvm);
 	tvm->signal_id = g_signal_connect(GTK_OBJECT(tv),
 		"motion-notify-event", G_CALLBACK(on_tree_view_motion_notify), tvm);
 	return tvm;
@@ -772,40 +662,70 @@ gtk_editable_get_value_as_uint32(GtkEditable *editable)
  * choice number set as user_data.
  */
 void
-gtk_combo_init_choices(GtkCombo *combo, GtkSignalFunc func,
+widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 		prop_def_t *def, gpointer user_data)
 {
-    guint i;
     guint32 original;
 
     g_assert(def != NULL);
-    g_assert(combo != NULL);
+    g_assert(widget != NULL);
+    g_assert(GTK_IS_COMBO(widget) || GTK_IS_OPTION_MENU(widget));
     g_assert(def->type == PROP_TYPE_MULTICHOICE);
     g_assert(def->data.guint32.choices != NULL);
 
     original = *def->data.guint32.value;
 
-    for (i = 0; def->data.guint32.choices[i].title != NULL; i++) {
-        GtkWidget *list_item;
-        GList *l;
+	if (GTK_IS_COMBO(widget)) {
+		GtkCombo *combo = GTK_COMBO(widget);
+		const gchar *title;
+    	guint i;
+		
+		for (i = 0; (title = def->data.guint32.choices[i].title) != NULL; i++) {
+			GtkWidget *list_item;
+			GList *l;
 
-        list_item = gtk_list_item_new_with_label(
-						_(def->data.guint32.choices[i].title));
+			list_item = gtk_list_item_new_with_label(_(title));
 
-        gtk_object_set_user_data(GTK_OBJECT(list_item),
-            GINT_TO_POINTER(def->data.guint32.choices[i].value));
+			gtk_object_set_user_data(GTK_OBJECT(list_item),
+					GINT_TO_POINTER(def->data.guint32.choices[i].value));
 
-        gtk_widget_show(list_item);
+			gtk_widget_show(list_item);
 
-        gtk_signal_connect_after(GTK_OBJECT(list_item),
-			"select", func, user_data);
+			gtk_signal_connect_after(GTK_OBJECT(list_item),
+					"select", func, user_data);
 
-        l = g_list_prepend(NULL, (gpointer) list_item);
-        gtk_list_append_items(GTK_LIST(combo->list), l);
+			l = g_list_prepend(NULL, (gpointer) list_item);
+			gtk_list_append_items(GTK_LIST(combo->list), l);
 
-        if (def->data.guint32.choices[i].value == original)
-            gtk_list_select_child(GTK_LIST(combo->list), list_item);
-    }
+			if (def->data.guint32.choices[i].value == original)
+				gtk_list_select_child(GTK_LIST(combo->list), list_item);
+		}
+	} else if (GTK_IS_OPTION_MENU(widget)) {
+		GtkOptionMenu *option_menu = GTK_OPTION_MENU(widget);
+		GtkMenu *menu;
+		const gchar *title;
+		guint i;
+
+		menu = GTK_MENU(gtk_menu_new());
+		for (i = 0; (title = def->data.guint32.choices[i].title) != NULL; i++) {
+			GtkWidget *item;
+
+			item = gtk_menu_item_new_with_label(_(title));
+			gtk_widget_show(item);
+			gtk_object_set_user_data(GTK_OBJECT(item),
+				GUINT_TO_POINTER(def->data.guint32.choices[i].value));
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+#ifdef USE_GTK1
+			gtk_signal_connect_after(GTK_OBJECT(item), "activate",
+				func, user_data);
+#endif /* USE_GTK1 */
+#ifdef USE_GTK2
+			g_signal_connect_after(GTK_OBJECT(item), "activate",
+				G_CALLBACK(func), user_data);
+#endif /* USE_GTK2 */
+		}
+		gtk_option_menu_set_menu(option_menu, GTK_WIDGET(menu));
+	}
 }
 
 #ifdef USE_GTK1
@@ -1129,5 +1049,20 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 }
 #endif
 
+#ifdef USE_GTK1
+GtkWidget *
+gtk_paned_get_child1(GtkPaned *paned)
+{
+	g_return_val_if_fail(paned != NULL, NULL);
+	return paned->child1;
+}
+
+GtkWidget *
+gtk_paned_get_child2(GtkPaned *paned)
+{
+	g_return_val_if_fail(paned != NULL, NULL);
+	return paned->child2;
+}
+#endif
 
 /* vi: set ts=4 sw=4 cindent: */
