@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c,v 1.14 2005/04/27 15:36:20 cbiere Exp $
+ * $Id: downloads.c,v 1.30 2005/11/17 20:09:09 rmanfredi Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: downloads.c,v 1.14 2005/04/27 15:36:20 cbiere Exp $");
+RCSID("$Id: downloads.c,v 1.30 2005/11/17 20:09:09 rmanfredi Exp $");
 
 #include "gtk/downloads.h"
 #include "gtk/downloads_common.h"
@@ -40,11 +40,13 @@ RCSID("$Id: downloads.c,v 1.14 2005/04/27 15:36:20 cbiere Exp $");
 
 #include "lib/atoms.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
+#include "lib/utf8.h"
 #include "lib/override.h"	/* Must be the last header included */
 
 static gchar tmpstr[4096];
-static GHashTable *parents;			/* table of parent download iterators */
-static GHashTable *parents_queue;	/* table of parent queued dl iterators */
+static GHashTable *parents;			/**< table of parent download iterators */
+static GHashTable *parents_queue;	/**< table of parent queued dl iterators */
 
 /*
  * parents_gui_time
@@ -56,7 +58,7 @@ static GHashTable *parents_queue;	/* table of parent queued dl iterators */
  *
  *		--RAM, 03/01/2004.
  */
-static GHashTable *parents_gui_time;	/* Time at which parent was updated */
+static GHashTable *parents_gui_time;	/**< Time at which parent was updated */
 
 /*
  * parents_children
@@ -70,14 +72,38 @@ static GHashTable *parents_queue_children;
 
 static GtkCTree *ctree_downloads = NULL;
 static GtkCTree *ctree_downloads_queue = NULL;
+static GtkNotebook *notebook = NULL;
 
-#define IO_STALLED		60		/* If nothing exchanged after that many secs */
-#define DL_GUI_TREE_SPACE 5	 /* The space between a child node and a parent */
+static gboolean ctree_downloads_frozen = FALSE;
+static gboolean ctree_downloads_queue_frozen = FALSE;
+
+#define IO_STALLED			60	/**< If nothing exchanged after that many secs */
+#define DL_GUI_TREE_SPACE	5	/**< Space between a child node and a parent */
 
 /***
  *** Private functions
  ***/
 
+static inline const gchar *
+download_progress_to_string(const struct download *d)
+{
+	static gchar buf[32];
+
+	gm_snprintf(buf, sizeof buf, "%5.2f%%",
+		100.0 * guc_download_total_progress(d));
+	return buf;
+}
+
+static inline const gchar *
+source_progress_to_string(const struct download *d)
+{
+	static gchar buf[32];
+
+	gm_snprintf(buf, sizeof buf, "%5.2f%%",
+		100.0 * guc_download_source_progress(d));
+	return buf;
+}
+
 /**
  * Add the given tree node to the hashtable.
  * The key is an int ref on the fi_handle for a given download.
@@ -314,8 +340,6 @@ downloads_gui_any_status(struct download *d, download_status_t status)
 	gpointer key;
 	GtkCTreeNode *node, *parent;
 	GtkCTreeRow *row;
-    GtkCTree *ctree_downloads =
-		GTK_CTREE(lookup_widget(main_window, "ctree_downloads"));
 
 	if (!d->file_info)
 		return FALSE;
@@ -355,9 +379,6 @@ downloads_gui_all_aborted(struct download *d)
 	gboolean all_aborted = FALSE;
 	GtkCTreeNode *node, *parent;
 	GtkCTreeRow *row;
-    GtkCTree *ctree_downloads = GTK_CTREE
-            (lookup_widget(main_window, "ctree_downloads"));
-
 
 	if (NULL != d->file_info) {
 
@@ -405,8 +426,6 @@ downloads_gui_update_parent_status(struct download *d, time_t now,
 
 	GdkColor *color;
 	GtkCTreeNode *parent;
-    GtkCTree *ctree_downloads = GTK_CTREE
-            (lookup_widget(main_window, "ctree_downloads"));
 
 
     if (NULL != d->file_info) {
@@ -445,8 +464,6 @@ void gui_update_download_hostcount(struct download *d)
 {
 	gpointer key;
 	GtkCTreeNode *parent;
-    GtkCTree *ctree_downloads = GTK_CTREE
-            (lookup_widget(main_window, "ctree_downloads"));
 
     if (NULL != d->file_info) {
 
@@ -482,6 +499,7 @@ void downloads_gui_init(void)
 
 	clist = GTK_CLIST(ctree_downloads);
 	clist_queue = GTK_CLIST(ctree_downloads_queue);
+	notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
 
     gtk_clist_column_titles_passive(clist);
     gtk_clist_column_titles_passive(clist_queue);
@@ -489,7 +507,9 @@ void downloads_gui_init(void)
     gtk_clist_set_column_justification(
         clist_queue, c_queue_size, GTK_JUSTIFY_RIGHT);
     gtk_clist_set_column_justification(
-        clist, c_queue_size, GTK_JUSTIFY_RIGHT);
+        clist, c_dl_size, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(
+        clist, c_dl_progress, GTK_JUSTIFY_RIGHT);
 
 	parents = g_hash_table_new(NULL, NULL);
 	parents_queue = g_hash_table_new(NULL, NULL);
@@ -550,15 +570,15 @@ download_gui_add(struct download *d)
 
 	file_name = guc_file_info_readable_filename(d->file_info);
 
-	gm_snprintf(vendor, sizeof(vendor), "%s%s",
-		(d->server->attrs & DLS_A_BANNING) ? "*" : "",
-		download_vendor_str(d));
+	concat_strings(vendor, sizeof vendor,
+		(d->server->attrs & DLS_A_BANNING) ? "*" : "", download_vendor_str(d),
+		(void *) 0);
 
 	color = &(gtk_widget_get_style(GTK_WIDGET(ctree_downloads))
 				->fg[GTK_STATE_INSENSITIVE]);
 
 	titles[c_queue_filename] = file_name;
-    titles[c_queue_server] = vendor;
+    titles[c_queue_server] = lazy_utf8_to_locale(vendor);
 	titles[c_queue_status] = "";
 
 	if (d->file_info->file_size_known)
@@ -692,7 +712,7 @@ download_gui_add(struct download *d)
 	} else {					/* This is an active download */
 
 		titles[c_dl_filename] = file_name;
-		titles[c_dl_server] = vendor;
+		titles[c_dl_server] = lazy_utf8_to_locale(vendor);
 		titles[c_dl_status] = "";
 
 		if (d->file_info->file_size_known)
@@ -702,6 +722,7 @@ download_gui_add(struct download *d)
 		titles[c_dl_range] = "";
         titles[c_dl_host] = guc_download_get_hostname(d);
         titles[c_dl_loc] = guc_download_get_country(d);
+        titles[c_dl_progress] = source_progress_to_string(d);
 
 		if (NULL != d->file_info) {
 			key = GUINT_TO_POINTER(d->file_info->fi_handle);
@@ -739,6 +760,7 @@ download_gui_add(struct download *d)
 					titles_parent[c_dl_host] = host;
 			        titles_parent[c_dl_loc] = country;
         			titles_parent[c_dl_range] = range;
+        			titles_parent[c_dl_progress] = source_progress_to_string(d);
 
 					new_node = gtk_ctree_insert_node(ctree_downloads,
 						parent, NULL,
@@ -768,6 +790,8 @@ download_gui_add(struct download *d)
 						c_dl_range, "");
 					gtk_ctree_node_set_text(ctree_downloads, parent,
 						c_dl_loc, "");
+					gtk_ctree_node_set_text(ctree_downloads, parent,
+						c_dl_progress, download_progress_to_string(d));
 
 					gtk_ctree_node_set_row_data(ctree_downloads, parent,
 						DL_GUI_IS_HEADER);
@@ -803,7 +827,7 @@ download_gui_add(struct download *d)
 					NG_("%u host", "%u hosts", n), n);
 
 				gtk_ctree_node_set_text(ctree_downloads, parent,
-					c_queue_host, tmpstr);
+					c_dl_host, tmpstr);
 
 			} else {
 				/*  There are no other downloads with the same file_info
@@ -845,19 +869,21 @@ gui_update_download_server(struct download *d)
 	g_assert(d->server);
 	g_assert(download_vendor(d));
 
-	/*
-	 * Prefix vendor name with a '*' if they are considered as potentially
-	 * banning us and we activated anti-banning features.
-	 *		--RAM, 05/07/2003
-	 */
+	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, d);
+	if (NULL != node) {
+		/*
+		 * Prefix vendor name with a '*' if they are considered as potentially
+		 * banning us and we activated anti-banning features.
+		 *		--RAM, 05/07/2003
+		 */
 
-	(void) gm_snprintf(tmpstr, sizeof(tmpstr), "%s%s",
-		(d->server->attrs & DLS_A_BANNING) ? "*" : "",
-		download_vendor(d));
+		concat_strings(tmpstr, sizeof tmpstr,
+			(d->server->attrs & DLS_A_BANNING) ? "*" : "", download_vendor(d),
+			(void *) 0);
 
-	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, (gpointer) d);
-	if (NULL != node)
-		gtk_ctree_node_set_text(ctree_downloads, node, c_dl_server, tmpstr);
+		gtk_ctree_node_set_text(ctree_downloads, node, c_dl_server,
+			lazy_utf8_to_locale(tmpstr));
+	}
 }
 
 void
@@ -897,6 +923,27 @@ gui_update_download_range(struct download *d)
 		gtk_ctree_node_set_text(ctree_downloads, node, c_dl_range, tmpstr);
 }
 
+/*
+ * Update the size of the active download.
+ */
+void
+gui_update_download_size(struct download *d)
+{
+	GtkCTreeNode *node;
+
+	g_assert(d);
+	g_assert(!DOWNLOAD_IS_QUEUED(d));
+
+	if (!d->file_info->file_size_known)
+		return;
+
+	gm_snprintf(tmpstr, sizeof(tmpstr), "%s", short_size(d->size));
+
+	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, (gpointer) d);
+	if (NULL != node)
+		gtk_ctree_node_set_text(ctree_downloads, node, c_dl_size, tmpstr);
+}
+
 void
 gui_update_download_host(struct download *d)
 {
@@ -909,29 +956,31 @@ gui_update_download_host(struct download *d)
 	g_assert(d->status != GTA_DL_QUEUED);
 
 	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, (gpointer) d);
-	if (NULL != node)
+	if (NULL != node) {
 		gtk_ctree_node_set_text(ctree_downloads, node,
 			c_dl_host, guc_download_get_hostname(d));
+		gtk_ctree_node_set_text(ctree_downloads, node,
+			c_dl_loc, guc_download_get_country(d));
+	}
 }
 
 void
 gui_update_download(struct download *d, gboolean force)
 {
 	const gchar *a = NULL;
-	time_t now = time((time_t *) NULL);
+	time_t now = tm_time();
     GdkColor *color;
 	GtkCTreeNode *node, *parent;
 	struct download *drecord;
-	struct dl_file_info *fi;
+	fileinfo_t *fi;
 	gpointer key;
 	gint active_src, tot_src;
-	gfloat percent_done =0;
+	gdouble percent_done = 0;
 	guint32 s = 0;
+	gboolean copy_status_to_parent = FALSE;
 
 	gint rw;
     gint current_page;
-	static GtkNotebook *notebook = NULL;
-	static GtkNotebook *dl_notebook = NULL;
 	gboolean looking = TRUE;
 
     if (d->last_gui_update == now && !force)
@@ -948,23 +997,10 @@ gui_update_download(struct download *d, gboolean force)
 	 * states...
 	 */
 
-	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
-
-	if (dl_notebook == NULL)
-		dl_notebook =
-			GTK_NOTEBOOK(lookup_widget(main_window, "notebook_downloads"));
-
     current_page = gtk_notebook_get_current_page(notebook);
-    if (current_page != nb_main_page_downloads)
+    if (current_page != nb_main_page_dl_active)
         looking = FALSE;
 
-	if (looking) {
-		current_page = gtk_notebook_get_current_page(dl_notebook);
-		if (current_page != nb_downloads_page_downloads)
-			looking = FALSE;
-	}
-
 	if (!looking) {
 		switch (d->status) {
 		case GTA_DL_ACTIVE_QUEUED:
@@ -1006,7 +1042,7 @@ gui_update_download(struct download *d, gboolean force)
 
 				if (guc_get_parq_dl_queue_length(d) > 0) {
 					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-						" / %d", guc_get_parq_dl_queue_length(d));
+						"/%d", guc_get_parq_dl_queue_length(d));
 				}
 
 				if (guc_get_parq_dl_eta(d)  > 0) {
@@ -1065,7 +1101,8 @@ gui_update_download(struct download *d, gboolean force)
 							_("Sending push"));
 
 				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, _(" via %s"),
-						ip_port_to_gchar(cproxy_ip(cp), cproxy_port(cp)));
+						host_addr_port_to_string(cproxy_addr(cp),
+							cproxy_port(cp)));
 
 				if (!cp->done) {
 					switch (cp->state) {
@@ -1153,6 +1190,7 @@ gui_update_download(struct download *d, gboolean force)
 	case GTA_DL_VERIFY_WAIT:
 		g_assert(FILE_INFO_COMPLETE(fi));
 		g_strlcpy(tmpstr, _("Waiting for SHA1 checking..."), sizeof(tmpstr));
+		copy_status_to_parent = TRUE;		/* In active pane */
 		a = tmpstr;
 		break;
 
@@ -1160,6 +1198,7 @@ gui_update_download(struct download *d, gboolean force)
 		g_assert(FILE_INFO_COMPLETE(fi));
 		gm_snprintf(tmpstr, sizeof(tmpstr),
 			_("Computing SHA1 (%.02f%%)"), fi->cha1_hashed * 100.0 / fi->size);
+		copy_status_to_parent = TRUE;		/* In active pane */
 		a = tmpstr;
 		break;
 
@@ -1169,6 +1208,7 @@ gui_update_download(struct download *d, gboolean force)
 	case GTA_DL_DONE:
 		g_assert(FILE_INFO_COMPLETE(fi));
 		g_assert(fi->cha1_hashed <= fi->size);
+		copy_status_to_parent = TRUE;		/* In active pane */
 		{
 			gboolean sha1_ok = fi->cha1 &&
 				(fi->sha1 == NULL || sha1_eq(fi->sha1, fi->cha1));
@@ -1213,14 +1253,11 @@ gui_update_download(struct download *d, gboolean force)
 
 	case GTA_DL_RECEIVING:
 		if (d->pos - d->skip > 0) {
-			gfloat p = 0, pt = 0;
+			gdouble p = 0;
 			gint bps;
 			guint32 avg_bps;
 
-			if (d->size)
-                p = (d->pos - d->skip) * 100.0 / d->size;
-            if (download_filesize(d))
-                pt = download_filedone(d) * 100.0 / download_filesize(d);
+			p = guc_download_source_progress(d);
 
 			bps = bio_bps(d->bio);
 			avg_bps = bio_avg_bps(d->bio);
@@ -1239,9 +1276,6 @@ gui_update_download(struct download *d, gboolean force)
 
                 s = remain / avg_bps;
 
-				rw = gm_snprintf(tmpstr, sizeof(tmpstr),
-					"%.02f%% / %.02f%%", p, pt);
-
 				if (now - d->last_update > IO_STALLED)
 					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, " %s",
 						_("(stalled)"));
@@ -1250,7 +1284,7 @@ gui_update_download(struct download *d, gboolean force)
 						" (%s)", short_rate(bps));
 
 				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-					" [%d/%d] TR: %s", fi->recvcount, fi->lifecount,
+					_(" [%d/%d] TR: %s"), fi->recvcount, fi->lifecount,
 					s ? short_time(s) : "-");
 
 				if (fi->recv_last_rate) {
@@ -1265,7 +1299,7 @@ gui_update_download(struct download *d, gboolean force)
 					}
 				}
 			} else {
-				rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%.02f%% %s", p,
+				rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%4.02f%% %s", p,
 					(now - d->last_update > IO_STALLED) ? _("(stalled)") : "");
 			}
 
@@ -1275,7 +1309,7 @@ gui_update_download(struct download *d, gboolean force)
 
 			if (d->ranges != NULL) {
 				gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-					" <PFS %.02f%%>", d->ranges_size * 100.0 / fi->size);
+					" <PFS %4.02f%%>", d->ranges_size * 100.0 / fi->size);
 			}
 
 			a = tmpstr;
@@ -1297,9 +1331,9 @@ gui_update_download(struct download *d, gboolean force)
 		break;
 	case GTA_DL_SINKING:
 		{
-			gchar buf[32];
+			gchar buf[UINT64_DEC_BUFLEN];
 			
-			uint64_to_string_buf(buf, sizeof buf, d->sinkleft);
+			uint64_to_string_buf(d->sinkleft, buf, sizeof buf);
 			gm_snprintf(tmpstr, sizeof(tmpstr),
 				_("Sinking (%s bytes left)"), buf);
 		}
@@ -1352,13 +1386,9 @@ gui_update_download(struct download *d, gboolean force)
 						if ((GTA_DL_RECEIVING == d->status) &&
 							(d->pos - d->skip > 0)) {
 
-							percent_done = 0;
 							s = 0;
-
-	        			    if (download_filesize(d))
-		                		percent_done = ((download_filedone(d) * 100.0)
-									/ download_filesize(d));
-
+							percent_done = 100.0 *
+								guc_download_total_progress(d);
 							active_src = fi->recvcount;
 							tot_src = fi->lifecount;
 
@@ -1367,13 +1397,13 @@ gui_update_download(struct download *d, gboolean force)
 
 							if (s) {
 								gm_snprintf(tmpstr, sizeof(tmpstr),
-									"%.02f%%  (%s)  [%d/%d]  TR:  %s",
+									_("%4.02f%%  (%s)  [%d/%d]  TR:  %s"),
 									percent_done,
 									short_rate(fi->recv_last_rate),
 									active_src, tot_src, short_time(s));
 							} else {
 								gm_snprintf(tmpstr, sizeof(tmpstr),
-									"%.02f%%  (%s)  [%d/%d]  TR:  -",
+									_("%4.02f%%  (%s)  [%d/%d]  TR:  -"),
 									percent_done,
 									short_rate(fi->recv_last_rate),
 									active_src, tot_src);
@@ -1393,10 +1423,13 @@ gui_update_download(struct download *d, gboolean force)
 		/* Update status column */
 		if (NULL != node) {
 			gtk_ctree_node_set_text(ctree_downloads, node, c_dl_status, a);
+			gtk_ctree_node_set_text(ctree_downloads, node,
+				c_dl_progress, source_progress_to_string(d));
     	    if (DOWNLOAD_IS_IN_PUSH_MODE(d))
         	     gtk_ctree_node_set_foreground(ctree_downloads, node, color);
 		}
 
+
 		/*  Update header for downloads with multiple hosts */
 		if (NULL != d->file_info) {
 
@@ -1407,32 +1440,26 @@ gui_update_download(struct download *d, gboolean force)
 				parent != NULL &&
 				now != get_last_parent_gui_update(key)
 			) {
-				drecord = gtk_ctree_node_get_row_data(ctree_downloads,
-					parent);
+				drecord = gtk_ctree_node_get_row_data(ctree_downloads, parent);
 
 				if (DL_GUI_IS_HEADER == drecord) {
 					/* There is a header entry, we need to update it */
 
-					/* Download is done */
-					if (GTA_DL_DONE == d->status) {
+					gtk_ctree_node_set_text(ctree_downloads, parent,
+						c_dl_progress, download_progress_to_string(d));
+					
+					if (copy_status_to_parent) {
+						/* Download is done */
 
-						gm_snprintf(tmpstr, sizeof(tmpstr),
-							_("Complete"));
 						gtk_ctree_node_set_text(ctree_downloads, parent,
-							c_dl_status, tmpstr);
+							c_dl_status, a);
 						record_parent_gui_update(key, now);
 
 					} else {
 						if ((GTA_DL_RECEIVING == d->status) &&
 							(d->pos - d->skip > 0)) {
 
-							percent_done = 0;
 							s = 0;
-
-	        			    if (download_filesize(d))
-		                		percent_done = ((download_filedone(d) * 100.0)
-									/ download_filesize(d));
-
 							active_src = fi->recvcount;
 							tot_src = fi->lifecount;
 
@@ -1441,14 +1468,12 @@ gui_update_download(struct download *d, gboolean force)
 
 							if (s) {
 								gm_snprintf(tmpstr, sizeof(tmpstr),
-									"%.02f%%  (%s)  [%d/%d]  TR:  %s",
-									percent_done,
+									_("(%s)  [%d/%d]  TR:  %s"),
 									short_rate(fi->recv_last_rate),
 									active_src, tot_src, short_time(s));
 							} else {
 								gm_snprintf(tmpstr, sizeof(tmpstr),
-									"%.02f%%  (%s)  [%d/%d]  TR:  -",
-									percent_done,
+									_("(%s)  [%d/%d]  TR:  -"),
 									short_rate(fi->recv_last_rate),
 									active_src, tot_src);
 							}
@@ -1577,7 +1602,7 @@ download_gui_remove(struct download *d)
 	GtkCTreeNode *node, *parent;
 	GtkCTreeRow *parent_row;
 	struct download *drecord;
-	gchar *host, *range, *server, *status, *country;
+	gchar *host, *range, *server, *status, *country, *progress;
 	const gchar *filename;
 	gpointer key;
 	gint n;
@@ -1722,6 +1747,8 @@ download_gui_remove(struct download *d)
 							c_dl_host, &host);
 						gtk_ctree_node_get_text(ctree_downloads, node,
 							c_dl_server, &server);
+						gtk_ctree_node_get_text(ctree_downloads, node,
+							c_dl_progress, &progress);
 						gtk_ctree_node_get_text(ctree_downloads, node,
 							c_dl_status, &status);
 						gtk_ctree_node_get_text(ctree_downloads, node,
@@ -1737,6 +1764,8 @@ download_gui_remove(struct download *d)
 							c_dl_host, host);
 						gtk_ctree_node_set_text(ctree_downloads,  parent,
 							c_dl_server, server);
+						gtk_ctree_node_set_text(ctree_downloads,  parent,
+							c_dl_progress, progress);
 						gtk_ctree_node_set_text(ctree_downloads,  parent,
 							c_dl_status, status);
 						gtk_ctree_node_set_text(ctree_downloads,  parent,
@@ -1783,6 +1812,7 @@ void
 downloads_gui_expand_all(GtkCTree *ctree)
 {
 	gtk_ctree_expand_recursive(ctree, NULL);
+	downloads_update_active_pane();
 }
 
 
@@ -1793,6 +1823,90 @@ void
 downloads_gui_collapse_all(GtkCTree *ctree)
 {
 	gtk_ctree_collapse_recursive(ctree, NULL);
+	downloads_update_active_pane();
+}
+
+/**
+ * Update "active" pane if needed.
+ */
+void
+downloads_update_active_pane(void)
+{
+	GtkCList *clist = GTK_CLIST(ctree_downloads);
+
+	if (!ctree_downloads_frozen)
+		return;
+
+	gtk_clist_thaw(clist);
+	gtk_clist_freeze(clist);
+}
+
+/**
+ * Update "queue" pane if needed.
+ */
+void
+downloads_update_queue_pane(void)
+{
+	GtkCList *clist = GTK_CLIST(ctree_downloads_queue);
+
+	if (!ctree_downloads_queue_frozen)
+		return;
+
+	gtk_clist_thaw(clist);
+	gtk_clist_freeze(clist);
+}
+
+/**
+ * Periodically called to update downloads display.
+ */
+void
+downloads_gui_update_display(time_t unused_now)
+{
+	GtkCList *clist = NULL;
+	gboolean *frozen = NULL;
+    gint current_page;
+
+	(void) unused_now;
+
+    current_page = gtk_notebook_get_current_page(notebook);
+
+	/*
+	 * We make sure the trees are frozen, so that no GUI redrawing ever
+	 * takes place until we're called here and they are watching.
+	 */
+
+    if (current_page == nb_main_page_dl_active) {
+		frozen = &ctree_downloads_frozen;
+		clist = GTK_CLIST(ctree_downloads);
+	} else if (current_page == nb_main_page_dl_queue) {
+		frozen = &ctree_downloads_queue_frozen;
+		clist = GTK_CLIST(ctree_downloads_queue);
+	} else {
+		/*
+		 * They're not looking, no need to update the visuals!
+		 */
+
+		if (!ctree_downloads_frozen) {
+			clist = GTK_CLIST(ctree_downloads);
+			gtk_clist_freeze(clist);
+			ctree_downloads_frozen = TRUE;
+		}
+		if (!ctree_downloads_queue_frozen) {
+			clist = GTK_CLIST(ctree_downloads_queue);
+			gtk_clist_freeze(clist);
+			ctree_downloads_queue_frozen = TRUE;
+		}
+
+		return;
+	}
+
+	g_assert(frozen != NULL);
+
+	if (*frozen)
+		gtk_clist_thaw(clist);		/* Will update visuals */
+
+	gtk_clist_freeze(clist);
+	*frozen = TRUE;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
