@@ -1,5 +1,5 @@
 /*
- * $Id: search.c,v 1.31 2005/07/08 14:39:44 daichik Exp $
+ * $Id: search.c,v 1.60 2005/12/17 09:25:44 daichik Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -43,22 +43,26 @@
 #include "gtk/settings.h"
 #include "gtk/statusbar.h"
 #include "gtk/columns.h"
+#include "gtk/misc.h"
 #include "gtk/notebooks.h"
 #include "search_cb.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
+#include "if/core/sockets.h"
 
 #include "lib/atoms.h"
 #include "lib/base32.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
+#include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/urn.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: search.c,v 1.31 2005/07/08 14:39:44 daichik Exp $");
+RCSID("$Id: search.c,v 1.60 2005/12/17 09:25:44 daichik Exp $");
 
 #define MAX_TAG_SHOWN	60		/**< Show only first chars of tag */
 
@@ -66,7 +70,6 @@ static gchar tmpstr[4096];
 
 GList *searches = NULL;		/* List of search structs */
 search_t *search_selected = NULL;
-static GList *list_search_history = NULL;
 
 /**
  * Characteristics of data in search results columns, used for sorting.
@@ -89,6 +92,7 @@ static gint search_results_compare_func
     (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2);
 #endif
 static void set_search_color(struct search *sch);
+static void gui_search_create_ctree(GtkWidget ** sw, GtkCTree ** ctree);
 
 
 /*
@@ -103,11 +107,13 @@ static GtkWidget *default_scrolled_window = NULL;
 
 /**
  *	Add the given tree node to the hashtable.
+ *
  *  The key is an atomized sha1 of the search result.
  *
  *  @bug
- *	FIXME: The "key" is an atom of the record's SHA1, why don't we create that
- * 	atom here, as we free it in "remove_parent_with_sha1"?  Emile 02/15/2004
+ *	FIXME: The "key" is an atom of the record's SHA1, why don't we
+ *	create that atom here, as we free it in "remove_parent_with_sha1"?
+ *	Emile 02/15/2004
  */
 static inline void
 add_parent_with_sha1(GHashTable *ht, gpointer key,
@@ -148,15 +154,12 @@ remove_parent_with_sha1(GHashTable *ht, const gchar *sha1)
  *	@returns the tree node corresponding to the given key, an atomized
  *	sha1.
  */
-GtkCTreeNode *find_parent_with_sha1(GHashTable *ht, gpointer key)
+GtkCTreeNode *
+find_parent_with_sha1(GHashTable *ht, gpointer key)
 {
-	return(g_hash_table_lookup(ht, key));
+	return g_hash_table_lookup(ht, key);
 }
 
-
-/**
- *	search_gui_free_parent
- */
 gboolean
 search_gui_free_parent(gpointer key, gpointer unused_value, gpointer unused_x)
 {
@@ -166,10 +169,6 @@ search_gui_free_parent(gpointer key, gpointer unused_value, gpointer unused_x)
 	return TRUE;
 }
 
-
-/**
- *	search_gui_free_gui_record
- */
 void
 search_gui_free_gui_record(gpointer gui_rc)
 {
@@ -274,7 +273,8 @@ search_gui_close_search(search_t *sch)
      * the same calls may still need them!.
      *      --BLUE 26/05/2002
      */
- 	searches = g_list_remove(searches, (gpointer) sch);
+ 	searches = g_list_remove(searches, sch);
+	search_gui_option_menu_searches_update();
 
 	search_gui_clear_search(sch);
     search_gui_remove_search(sch);
@@ -291,25 +291,6 @@ search_gui_close_search(search_t *sch)
 	g_free(sch);
 }
 
-
-/**
- * Create a new search and start it. Use default reissue timeout.
- */
-gboolean search_gui_new_search(
-	const gchar *query, flag_t flags, search_t **search)
-{
-    guint32 timeout;
-    gint sort_col = SORT_NO_COL;
-	gint sort_order = SORT_NONE;
-
-    gnet_prop_get_guint32_val
-		(PROP_SEARCH_REISSUE_TIMEOUT, &timeout);
-
-    return search_gui_new_search_full(query, timeout,
-		sort_col, sort_order, flags | SEARCH_ENABLED, search);
-}
-
-
 /**
  * Create a new search and start it.
  *
@@ -319,41 +300,61 @@ gboolean search_gui_new_search(
  */
 gboolean
 search_gui_new_search_full(const gchar *querystr,
-	guint32 reissue_timeout, gint sort_col,
-	gint sort_order, flag_t flags, search_t **search)
+	time_t create_time, guint lifetime, guint32 reissue_timeout,
+	gint sort_col, gint sort_order, flag_t flags, search_t **search)
 {
-    search_t *sch;
-	gnet_search_t sch_id;
-    GList *rules;
-    const gchar *titles[c_sl_num];
-    gint row;
-    const gchar *query, *error;
-    GtkWidget *combo_searches = lookup_widget(main_window, "combo_searches");
     GtkWidget *clist_search = lookup_widget(main_window, "clist_search");
     GtkWidget *notebook_search_results =
         lookup_widget(main_window, "notebook_search_results");
     GtkWidget *button_search_close =
         lookup_widget(main_window, "button_search_close");
     GtkWidget *entry_search = lookup_widget(main_window, "entry_search");
+    const gchar *titles[c_sl_num];
+    const gchar *error, *query;
+    search_t *sch;
+	gnet_search_t sch_id;
+    GList *rules;
+    gint row;
 
-	query = search_gui_parse_query(querystr, &rules, &error);
-	if (!query) {
-		statusbar_gui_message(5, "%s", error);
-		return FALSE;
+	if (0 == (flags & (SEARCH_F_PASSIVE | SEARCH_F_BROWSE))) {
+		query = search_gui_parse_query(querystr, &rules, &error);
+		if (!query) {
+			statusbar_gui_message(5, "%s", error);
+			return FALSE;
+		}
+	} else {
+		query = querystr;
+		rules = NULL;
 	}
-    sch_id = guc_search_new(query, reissue_timeout, flags);
-	if (-1 == (gint) sch_id) {
+	
+    sch_id = guc_search_new(query, create_time, lifetime,
+				reissue_timeout, flags);
+	if ((gnet_search_t) -1 == sch_id) {
 		statusbar_gui_warning(5, "%s", _("Failed to create the search"));
 		return FALSE;
 	}
 
-	sch = g_new0(search_t, 1);
-	sch->sort_col = sort_col;
-	sch->sort_order = sort_order;
+	sch = g_malloc0(sizeof *sch);
+
+	if (sort_col >= 0 && (guint) sort_col < SEARCH_RESULTS_VISIBLE_COLUMNS)
+		sch->sort_col = sort_col;
+	else
+		sch->sort_col = SORT_NO_COL;
+
+	switch (sort_order) {
+	case SORT_ASC:
+	case SORT_DESC:
+		sch->sort_order = sort_order;
+		break;
+	default:
+		sch->sort_order = SORT_NONE;
+	}
+
 	sch->query = atom_str_get(query);
-	sch->enabled = (flags & SEARCH_ENABLED) ? TRUE : FALSE;
+	sch->enabled = (flags & SEARCH_F_ENABLED) ? TRUE : FALSE;
+	sch->browse = (flags & SEARCH_F_BROWSE) ? TRUE : FALSE;
     sch->search_handle = sch_id;
-    sch->passive = (flags & SEARCH_PASSIVE) ? TRUE : FALSE;
+    sch->passive = (flags & SEARCH_F_PASSIVE) ? TRUE : FALSE;
 	sch->dups = g_hash_table_new(search_gui_hash_func,
 					search_gui_hash_key_compare);
 
@@ -363,22 +364,7 @@ search_gui_new_search_full(const gchar *querystr,
 	g_list_free(rules);
 	rules = NULL;
 
-	/* Create the list item */
-
-	if (utf8_is_valid_string(query, 0)) {
-		const gchar *q;
-		
-		q = utf8_to_locale(query, 0);
-		if (NULL != q && *q != '\0')
-			query = q;
-	}
-
-	sch->list_item = gtk_list_item_new_with_label(query);
-	gtk_widget_show(sch->list_item);
-	gtk_list_prepend_items(GTK_LIST(GTK_COMBO(combo_searches)->list),
-		g_list_prepend(NULL, sch->list_item));
-
-    titles[c_sl_name] = query;
+    titles[c_sl_name] = lazy_utf8_to_ui_string(sch->query);
     titles[c_sl_hit] = "0";
     titles[c_sl_new] = "0";
     row = gtk_clist_append(GTK_CLIST(clist_search), (gchar **) titles);
@@ -390,8 +376,7 @@ search_gui_new_search_full(const gchar *querystr,
 		/* We have to create a new ctree for this search */
 		gui_search_create_ctree(&sch->scrolled_window, &sch->ctree);
 
-		gtk_object_set_user_data((GtkObject *) sch->scrolled_window,
-								 (gpointer) sch);
+		gtk_object_set_user_data((GtkObject *) sch->scrolled_window, sch);
 
 		gtk_notebook_append_page(GTK_NOTEBOOK(notebook_search_results),
 								 sch->scrolled_window, NULL);
@@ -407,8 +392,7 @@ search_gui_new_search_full(const gchar *querystr,
 			g_warning
 				("new_search(): No current search but no default ctree !?\n");
 
-		gtk_object_set_user_data((GtkObject *) sch->scrolled_window,
-								 (gpointer) sch);
+		gtk_object_set_user_data((GtkObject *) sch->scrolled_window, sch);
 	}
 
 	gui_search_update_tab_label(sch);
@@ -424,27 +408,36 @@ search_gui_new_search_full(const gchar *querystr,
             w, _("(no search)"));
     }
 
-	gtk_signal_connect(GTK_OBJECT(sch->list_item), "select",
-					   GTK_SIGNAL_FUNC(on_search_selected),
-					   (gpointer) sch);
-
 	search_gui_sort_column(sch, sort_col);
-	search_gui_set_current_search(sch);
-
-	gtk_widget_set_sensitive(combo_searches, TRUE);
+	
 	gtk_widget_set_sensitive(button_search_close, TRUE);
     gtk_widget_set_sensitive(
         lookup_widget(main_window, "button_search_expand_all"), TRUE);
     gtk_widget_set_sensitive(
         lookup_widget(main_window, "button_search_collapse_all"), TRUE);
 
-    gtk_entry_set_text(GTK_ENTRY(entry_search),"");
+    gtk_entry_set_text(GTK_ENTRY(entry_search), "");
 
-	searches = g_list_append(searches, (gpointer) sch);
+	searches = g_list_append(searches, sch);
+	search_gui_option_menu_searches_update();
+	
+	if (search_gui_update_expiry(sch))
+		sch->enabled = FALSE;
 
 	if (sch->enabled)
 		guc_search_start(sch->search_handle);
 
+	/*
+	 * Make new search the current search, unless it's a browse-host search:
+	 * we need to initiate the download and only if everything is OK will
+	 * we move to the newly created search.
+	 */
+	
+	if (sch->browse)
+		gui_search_force_update_tab_label(sch);
+	else
+		search_gui_set_current_search(sch);
+
 	set_search_color(sch);
 
 	if (search)
@@ -460,8 +453,9 @@ search_gui_new_search_full(const gchar *querystr,
  * If the value in sort_col for r1 is "greater than" r2 returns +1
  * 0 if they're equal, and -1 if r1 is "less than" r2
  */
-static gint search_gui_compare_records(
-	gint sort_col, const gui_record_t *g1, const gui_record_t *g2)
+static gint
+search_gui_compare_records(gint sort_col,
+	const gui_record_t *g1, const gui_record_t *g2)
 {
 	record_t *r1 = g1->shared_record;
 	record_t *r2 = g2->shared_record;
@@ -492,6 +486,10 @@ static gint search_gui_compare_records(
             result = strcmp(r1->ext, r2->ext);
             break;
 
+        case c_sr_charset:
+            result = strcmp(r1->charset, r2->charset);
+            break;
+
         case c_sr_size:
 			/*
 			 * Sort by size, then by identical SHA1.
@@ -506,8 +504,7 @@ static gint search_gui_compare_records(
             break;
 
         case c_sr_info:
-			result = strncmp(cast_to_gpointer(rs1->vendor),
-						cast_to_gpointer(rs2->vendor), sizeof rs1->vendor);
+			result = CMP(ntohl(rs1->vcode.be32), ntohl(rs2->vcode.be32));
 			if (result)
 				break;
             result = CMP(rs1->status, rs2->status);
@@ -529,9 +526,6 @@ static gint search_gui_compare_records(
 
         case c_sr_meta:
 			break;				/* XXX Can't sort, metadata not in record! */
-
-        default:
-            g_assert_not_reached();
         }
     }
 
@@ -661,7 +655,7 @@ void
 search_gui_quick_sort_array_swap(GArray *array, gint i1, gint i2)
 {
 /*
- *  This is the old version by Emile. I the one below seems to work fine
+ *  This is the old version by Emile. The one below seems to work fine
  *  and should be faster because it does not remove/insert. Just in case
  *  my version doesn't work I'll leave the old one in.
  *    -- Richard, 6/2/2004
@@ -677,11 +671,11 @@ search_gui_quick_sort_array_swap(GArray *array, gint i1, gint i2)
     GtkCTreeNode *buf;
 
     buf = g_array_index(array, GtkCTreeNode *, i1);
-    g_array_index(array, GtkCTreeNode *, i1) = g_array_index(array, GtkCTreeNode *, i2);
+    g_array_index(array, GtkCTreeNode *, i1) =
+		g_array_index(array, GtkCTreeNode *, i2);
     g_array_index(array, GtkCTreeNode *, i2) = buf;
 }
 
-
 /**
  * Performs a recursive quick sort on the given array between indicies
  * beg and end.
@@ -1055,11 +1049,11 @@ search_gui_sort_column(search_t *search, gint column)
     /* create new arrow */
     switch (search->sort_order) {
     case SORT_ASC:
-        search->arrow = create_pixmap(main_window, "arrow_up.xpm");
+        search->arrow = create_pixmap(main_window, "arrow_down.xpm");
 		ascending = TRUE;
         break;
     case SORT_DESC:
-        search->arrow = create_pixmap(main_window, "arrow_down.xpm");
+        search->arrow = create_pixmap(main_window, "arrow_up.xpm");
 		ascending = FALSE;
         break;
     case SORT_NONE:
@@ -1144,9 +1138,15 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 	}
 
 	if (rc->alt_locs != NULL) {
+		guint32 count = (guint32) rc->alt_locs->hvcnt;
 		if (info->len)
 			g_string_append(info, ", ");
 		g_string_append(info, "alt");
+		if (count > 1) {
+			g_string_append_c(info, '(');
+			g_string_append(info, uint32_to_string(count));
+			g_string_append_c(info, ')');
+		}
 	}
 
 	if (NULL != rc->info)
@@ -1155,11 +1155,20 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 
 	g_string_free(info, TRUE);
 
-	/* Setup text for node.  Note only parent nodes will have # and size shown*/
-	titles[c_sr_filename] = (NULL != rc->name) ?  rc->name : empty;
-	titles[c_sr_ext] = (NULL != rc->ext) ? rc->ext : empty;
+	/* Setup text for node. Note only parent nodes will have # and size shown */
+	{
+		const gchar *ext;
+		
+		ext = search_gui_get_filename_extension(rc->utf8_name);
+		rc->ext = atom_str_get(ext ? lazy_utf8_to_ui_string(ext) : "");
+
+		titles[c_sr_filename] = lazy_utf8_to_ui_string(rc->utf8_name);
+	}
+
+	titles[c_sr_ext] = rc->ext;
+	titles[c_sr_charset] = rc->charset;
 	titles[c_sr_meta] = empty;
-	titles[c_sr_info] = (NULL != rc->info) ?  rc->info : empty;
+	titles[c_sr_info] = rc->info ? rc->info : empty;
 	titles[c_sr_size] = empty;
 	titles[c_sr_count] = empty;
 	titles[c_sr_loc] = iso3166_country_cc(rs->country);
@@ -1185,7 +1194,7 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 			/* A parent exists with that sha1, add as child to that parent */
 			node = gtk_ctree_insert_node(ctree, parent, NULL,
 						(gchar **) titles, /* override const */
-						5, NULL, NULL, NULL, NULL, 0, 0);
+						G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
 
 			/* Update the "#" column of the parent, +1 for parent */
 			count = gtk_ctree_count_node_children(ctree, parent);
@@ -1206,7 +1215,7 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 			/* Add node as a parent */
 			node = gtk_ctree_insert_node(ctree, parent = NULL, NULL,
 						(gchar **) titles, /* override const */
-						5, NULL, NULL, NULL, NULL, 0, 0);
+						G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
 			add_parent_with_sha1(sch->parents, key, node);
 
 			/* Update count in the records (use for column sorting) */
@@ -1219,7 +1228,7 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 
 		node = gtk_ctree_insert_node(ctree, parent = NULL, NULL,
 					(gchar **) titles, /* override */
-					5, NULL, NULL, NULL, NULL, 0, 0);
+					G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
 		/* Update count in the records (use for column sorting) */
 		gui_rc->num_children = 0;
 		is_parent = TRUE;
@@ -1328,9 +1337,6 @@ search_gui_add_record(search_t *sch, record_t *rc, GString *vinfo,
 }
 
 
-/**
- *	search_gui_set_clear_button_sensitive
- */
 void
 search_gui_set_clear_button_sensitive(gboolean flag)
 {
@@ -1483,9 +1489,7 @@ search_gui_remove_result(GtkCTree *ctree, GtkCTreeNode *node)
  * items in the selection within `selected'.
  */
 static guint
-download_selection_of_ctree(
-	GtkCTree *ctree,
-	guint *selected)
+download_selection_of_ctree(GtkCTree *ctree, guint *selected)
 {
 	struct results_set *rs;
 	gui_record_t *grc;
@@ -1496,9 +1500,9 @@ download_selection_of_ctree(
 	gboolean resort = FALSE;
 	guint created = 0;
 	guint count = 0;
+	guint32 flags;
 	GtkCTreeNode *node;
 	GtkCTreeRow *row;
-	gchar *filename;
 	search_t *current_search = search_gui_get_current_search();
 
     gnet_prop_get_boolean_val(PROP_SEARCH_REMOVE_DOWNLOADED,
@@ -1529,19 +1533,16 @@ download_selection_of_ctree(
         }
 
 		rs = rc->results_set;
-		need_push = (rs->status & ST_FIREWALL) != 0;
+		need_push = 0 != (rs->status & ST_FIREWALL);
+		flags = (rs->status & ST_TLS) ? CONNECT_F_TLS : 0;
 
-		filename = gm_sanitize_filename(rc->name, FALSE, FALSE);
-		if (guc_download_new(filename, rc->size, rc->index,
-				rs->ip, rs->port, rs->guid, rs->hostname,
-				rc->sha1, rs->stamp, need_push, NULL, rs->proxies)
+		if (guc_download_new(rc->name, rc->size, rc->index,
+				rs->addr, rs->port, rs->guid, rs->hostname,
+				rc->sha1, rs->stamp, need_push, NULL, rs->proxies, flags)
 		) {
 			created++;
 		}
 
-		if (filename != rc->name)
-			G_FREE_NULL(filename);
-
 		if (rs->proxies != NULL)
 			search_gui_free_proxies(rs);
 
@@ -1626,8 +1627,7 @@ download_selection_of_ctree(
  * @returns the amount of discarded results.
  */
 static guint
-discard_selection_of_ctree(
-	GtkCTree *ctree)
+discard_selection_of_ctree(GtkCTree *ctree)
 {
 	gui_record_t *grc;
 	record_t *rc;
@@ -1726,24 +1726,18 @@ discard_selection_of_ctree(
 }
 
 /**
- *	search_gui_download_files
- *
  *	Download selected files
  */
-void search_gui_download_files(void)
+void
+search_gui_download_files(void)
 {
     search_t *current_search = search_gui_get_current_search();
 
 	if (current_search) {
-        GtkWidget *notebook_main;
-        GtkWidget *ctree_menu;
         guint selected;
         guint created;
 		gchar buf[1024];
 
-        notebook_main = lookup_widget(main_window, "notebook_main");
-        ctree_menu = lookup_widget(main_window, "ctree_menu");
-
 		created = download_selection_of_ctree(
 			GTK_CTREE(current_search->ctree), &selected);
 
@@ -1762,22 +1756,16 @@ void search_gui_download_files(void)
 }
 
 /**
- *	search_gui_discard_files
- *
  *	Discard selected files
  */
-void search_gui_discard_files(void)
+void
+search_gui_discard_files(void)
 {
     search_t *current_search = search_gui_get_current_search();
 
 	if (current_search) {
-        GtkWidget *notebook_main;
-        GtkWidget *ctree_menu;
         guint discarded;
 
-        notebook_main = lookup_widget(main_window, "notebook_main");
-        ctree_menu = lookup_widget(main_window, "ctree_menu");
-
 		discarded = discard_selection_of_ctree(
 			GTK_CTREE(current_search->ctree));
 
@@ -1833,7 +1821,8 @@ search_gui_search_results_col_widths_changed(property_t prop)
  * This is not in settings_gui because the current search should not be
  * known outside this file.
  */
-gboolean search_gui_search_results_col_visible_changed(property_t prop)
+gboolean
+search_gui_search_results_col_visible_changed(property_t prop)
 {
 	gint i;
     gboolean *val;
@@ -1879,16 +1868,11 @@ static gint search_results_compare_func
  *** Public functions
  ***/
 
-
-/*
- *	search_gui_init
- */
-void search_gui_init(void)
+void
+search_gui_init(void)
 {
     GtkNotebook *notebook_search_results = GTK_NOTEBOOK
         (lookup_widget(main_window, "notebook_search_results"));
-    GtkCombo *combo_searches = GTK_COMBO
-        (lookup_widget(main_window, "combo_searches"));
 
 	GtkCTree *ctree;
 	search_t *current_search;
@@ -1903,9 +1887,6 @@ void search_gui_init(void)
   	gtk_notebook_set_tab_label_text
         (notebook_search_results, default_scrolled_window, _("(no search)"));
 
-	gtk_signal_connect(GTK_OBJECT(combo_searches->popwin),
-					   "hide", GTK_SIGNAL_FUNC(on_search_popdown_switch),
-					   NULL);
 	gtk_signal_connect(GTK_OBJECT(notebook_search_results), "switch_page",
 					   GTK_SIGNAL_FUNC(on_search_notebook_switch), NULL);
 
@@ -1925,10 +1906,8 @@ void search_gui_init(void)
 }
 
 
-/*
- *	search_gui_shutdown
- */
-void search_gui_shutdown(void)
+void
+search_gui_shutdown(void)
 {
 	GtkCTree *ctree;
     search_t *current_search = search_gui_get_current_search();
@@ -1946,45 +1925,30 @@ void search_gui_shutdown(void)
         search_gui_close_search((search_t *) searches->data);
 
 	search_gui_common_shutdown();
-
-    g_list_free(list_search_history);
-    list_search_history = NULL;
 }
 
-
-/*
- *	search_gui_get_searches
- */
-const GList *search_gui_get_searches(void)
+const GList *
+search_gui_get_searches(void)
 {
 	return (const GList *) searches;
 }
 
 /**
- * search_gui_remove_search:
- *
  * Remove the search from the gui and update all widgets accordingly.
  */
-void search_gui_remove_search(search_t * sch)
+void
+search_gui_remove_search(search_t * sch)
 {
     gint row;
-    GList *glist;
     gboolean sensitive;
     search_t *current_search;
     GtkCList *clist_search = GTK_CLIST
         (lookup_widget(main_window, "clist_search"));
     GtkNotebook *notebook_search_results = GTK_NOTEBOOK
         (lookup_widget(main_window, "notebook_search_results"));
-    GtkCombo *combo_searches = GTK_COMBO
-         (lookup_widget(main_window, "combo_searches"));
 
     g_assert(sch != NULL);
 
-   	glist = g_list_prepend(NULL, sch->list_item);
-	gtk_list_remove_items(GTK_LIST(combo_searches->list), glist);
-    g_list_free(glist);
-	glist = NULL;
-
     row = gtk_clist_find_row_from_data(clist_search, sch);
     gtk_clist_remove(clist_search, row);
 
@@ -2014,20 +1978,15 @@ void search_gui_remove_search(search_t * sch)
 		search_gui_forget_current_search();
 
 		search_gui_update_items(NULL);
+    	search_gui_update_expiry(NULL);
 
-		gtk_entry_set_text
-            (GTK_ENTRY(lookup_widget(main_window, "combo_entry_searches")), "");
+        gtk_notebook_set_tab_label_text(notebook_search_results,
+			default_scrolled_window, _("(no search)"));
 
-        gtk_notebook_set_tab_label_text
-            (notebook_search_results, default_scrolled_window, _("(no search)"));
-
-		gtk_widget_set_sensitive
-            (lookup_widget(main_window, "button_search_clear"), FALSE);
+		gtk_widget_set_sensitive(lookup_widget(main_window,
+			"button_search_clear"), FALSE);
 	}
 
-	gtk_widget_set_sensitive(
-        GTK_WIDGET(combo_searches),
-        searches != NULL);
 	gtk_widget_set_sensitive(
         lookup_widget(main_window, "button_search_close"),
         searches != NULL);
@@ -2050,25 +2009,18 @@ void search_gui_remove_search(search_t * sch)
 }
 
 
-/*
- *	search_gui_set_current_search
- *
- *
- *
- */
-void search_gui_set_current_search(search_t *sch)
+void
+search_gui_set_current_search(search_t *sch)
 {
-    search_t *old_sch = search_gui_get_current_search();
-    GtkCTree *ctree;
-    GtkCTreeNode * node;
-    GtkWidget *spinbutton_reissue_timeout;
-    GtkCList *clist_search;
-	gint i;
     static gboolean locked = FALSE;
-    gboolean passive;
+    search_t *old_sch = search_gui_get_current_search();
+    GtkWidget *spinbutton_reissue_timeout;
+   	GtkCList *clist;
     gboolean frozen;
+    gboolean active;
     guint32 reissue_timeout;
 	search_t *current_search = old_sch;
+	gint i;
 
 	g_assert(sch != NULL);
 
@@ -2080,16 +2032,16 @@ void search_gui_set_current_search(search_t *sch)
 	if (old_sch)
 		gui_search_force_update_tab_label(old_sch);
 
-    passive = guc_search_is_passive(sch->search_handle);
+    active = guc_search_is_active(sch->search_handle);
     frozen = guc_search_is_frozen(sch->search_handle);
-    reissue_timeout = guc_search_get_reissue_timeout
-		(sch->search_handle);
+    reissue_timeout = guc_search_get_reissue_timeout(sch->search_handle);
 
     /*
      * We now propagate the column visibility from the current_search
      * to the new current_search.
      */
     if (current_search != NULL) {
+    	GtkCTree *ctree;
 
         ctree = GTK_CTREE(current_search->ctree);
 
@@ -2104,22 +2056,24 @@ void search_gui_set_current_search(search_t *sch)
 	search_gui_current_search(sch);
 	sch->unseen_items = 0;
 
-    spinbutton_reissue_timeout= lookup_widget
+    spinbutton_reissue_timeout = lookup_widget
         (main_window, "spinbutton_search_reissue_timeout");
-    clist_search = GTK_CLIST
-            (lookup_widget(main_window, "clist_search"));
+   	clist = GTK_CLIST(lookup_widget(main_window, "clist_search"));
 
     if (sch != NULL) {
+    	gint row;	
+		
         gui_search_force_update_tab_label(sch);
         search_gui_update_items(sch);
 
-        gtk_clist_select_row(
-            clist_search,
-            gtk_clist_find_row_from_data(clist_search, sch),
-            0);
+		row = gtk_clist_find_row_from_data(clist, sch);
+        gtk_clist_select_row(clist, row, 0);
+		if (!GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(clist)))
+			gtk_clist_moveto(clist, row, 0, 0.0, 0.0);
+
         gtk_spin_button_set_value
             (GTK_SPIN_BUTTON(spinbutton_reissue_timeout), reissue_timeout);
-        gtk_widget_set_sensitive(spinbutton_reissue_timeout, !passive);
+        gtk_widget_set_sensitive(spinbutton_reissue_timeout, active);
         gtk_widget_set_sensitive(
             lookup_widget(main_window, "button_search_download"),
             GTK_CLIST(sch->ctree)->selection != NULL);
@@ -2127,20 +2081,16 @@ void search_gui_set_current_search(search_t *sch)
             lookup_widget(main_window, "button_search_clear"),
             sch->items != 0);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_restart"), !passive);
+            lookup_widget(popup_search, "popup_search_restart"), active);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_duplicate"), !passive);
+            lookup_widget(popup_search, "popup_search_duplicate"), active);
         gtk_widget_set_sensitive(
             lookup_widget(popup_search, "popup_search_stop"), !frozen);
         gtk_widget_set_sensitive(
             lookup_widget(popup_search, "popup_search_resume"),frozen);
 
-        /*
-         * Combo "Active searches"
-         */
-        gtk_list_item_select(GTK_LIST_ITEM(sch->list_item));
     } else {
-        gtk_clist_unselect_all(clist_search);
+        gtk_clist_unselect_all(clist);
         gtk_widget_set_sensitive(spinbutton_reissue_timeout, FALSE);
         gtk_widget_set_sensitive(
             lookup_widget(main_window, "button_search_download"), FALSE);
@@ -2172,28 +2122,33 @@ void search_gui_set_current_search(search_t *sch)
      * Tree menu
      */
     {
-        GtkCTree *ctree_menu = GTK_CTREE
-            (lookup_widget(main_window, "ctree_menu"));
+        GtkCTree *ctree;
+    	GtkCTreeNode *node;
+	   
+		ctree = GTK_CTREE(lookup_widget(main_window, "ctree_menu"));
 
-        node = gtk_ctree_find_by_row_data(
-            ctree_menu,
-            gtk_ctree_node_nth(ctree_menu,0),
-            GINT_TO_POINTER(nb_main_page_search));
+        node = gtk_ctree_find_by_row_data(ctree, gtk_ctree_node_nth(ctree, 0),
+					GINT_TO_POINTER(nb_main_page_search));
 
-        if (node != NULL)
-            gtk_ctree_select(ctree_menu,node);
+        if (node != NULL) {
+            gtk_ctree_select(ctree, node);
+			if (!GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(ctree)))
+				gtk_ctree_node_moveto(ctree, node, 0, 0.0, 0.0);
+		}
     }
 
+	if (search_gui_update_expiry(sch))
+		gui_search_set_enabled(sch, FALSE);
+
     locked = FALSE;
 }
 
 
 /**
- *	gui_search_create_ctree
- *
  *	Create a new GtkCTree for search results
  */
-void gui_search_create_ctree(GtkWidget ** sw, GtkCTree ** ctree)
+static void
+gui_search_create_ctree(GtkWidget ** sw, GtkCTree ** ctree)
 {
 	GtkWidget *label;
     GtkWidget *hbox;
@@ -2240,6 +2195,14 @@ void gui_search_create_ctree(GtkWidget ** sw, GtkCTree ** ctree)
     gtk_widget_show_all(hbox);
     gtk_clist_set_column_name(GTK_CLIST(*ctree), c_sr_ext, _("Extension"));
 
+	label = gtk_label_new(_("Encoding"));
+    gtk_misc_set_alignment(GTK_MISC(label),0,0.5);
+    hbox = gtk_hbox_new(FALSE, 4);
+    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+	gtk_clist_set_column_widget(GTK_CLIST(*ctree), c_sr_charset, hbox);
+    gtk_widget_show_all(hbox);
+    gtk_clist_set_column_name(GTK_CLIST(*ctree), c_sr_charset, _("Encoding"));
+
 	label = gtk_label_new(_("Size"));
     gtk_misc_set_alignment(GTK_MISC(label),0,0.5);
     hbox = gtk_hbox_new(FALSE, 4);
@@ -2310,30 +2273,28 @@ void gui_search_create_ctree(GtkWidget ** sw, GtkCTree ** ctree)
  *	Like search_update_tab_label but always update the label
  *
  */
-void gui_search_force_update_tab_label(struct search *sch)
+void
+gui_search_force_update_tab_label(struct search *sch)
 {
     GtkNotebook *notebook_search_results = GTK_NOTEBOOK
         (lookup_widget(main_window, "notebook_search_results"));
     GtkCList *clist_search = GTK_CLIST
         (lookup_widget(main_window, "clist_search"));
 	search_t *current_search = search_gui_get_current_search();
-	const gchar *query = sch->query;
     gint row;
-	
-	if (utf8_is_valid_string(query, 0)) {
-		const gchar *q;
-		
-		q = utf8_to_locale(query, 0);
-		if (NULL != q && *q != '\0')
-			query = q;
-	}
 
-	if (sch == current_search || sch->unseen_items == 0)
-		gm_snprintf(tmpstr, sizeof(tmpstr), "%s\n(%d)", query,
-				   sch->items);
-	else
-		gm_snprintf(tmpstr, sizeof(tmpstr), "%s\n(%d, %d)", query,
-				   sch->items, sch->unseen_items);
+	{
+		const gchar *s;
+		
+		s = lazy_utf8_to_ui_string(sch->query);
+		if (sch == current_search || sch->unseen_items == 0)
+			gm_snprintf(tmpstr, sizeof(tmpstr), "%s\n(%d)",
+				s, sch->items);
+		else
+			gm_snprintf(tmpstr, sizeof(tmpstr), "%s\n(%d, %d)",
+				s, sch->items, sch->unseen_items);
+	}
+	
 	sch->last_update_items = sch->items;
 	gtk_notebook_set_tab_label_text
         (notebook_search_results, sch->scrolled_window, tmpstr);
@@ -2353,8 +2314,7 @@ void gui_search_force_update_tab_label(struct search *sch)
         gtk_clist_set_background(clist_search, row, NULL);
     }
 
-	sch->last_update_time = time(NULL);
-
+	sch->last_update_time = tm_time();
 }
 
 
@@ -2368,7 +2328,7 @@ gui_search_update_tab_label(struct search *sch)
 	if (sch->items == sch->last_update_items)
 		return TRUE;
 
-	if (time(NULL) - sch->last_update_time < TAB_UPDATE_TIME)
+	if (tm_time() - sch->last_update_time < TAB_UPDATE_TIME)
 		return TRUE;
 
 	gui_search_force_update_tab_label(sch);
@@ -2410,47 +2370,6 @@ gui_search_get_colors(
 }
 
 
-/**
- * Adds a search string to the search history combo. Makes
- * sure we do not get more than 10 entries in the history.
- * Also makes sure we don't get duplicate history entries.
- * If a string is already in history and it's added again,
- * it's moved to the beginning of the history list.
- */
-void
-gui_search_history_add(gchar *s)
-{
-    GList *new_hist = NULL;
-    GList *cur_hist = list_search_history;
-    guint n = 0;
-
-    g_return_if_fail(s);
-
-    while (cur_hist != NULL) {
-        if (n < 9 && 0 != g_ascii_strcasecmp(s,cur_hist->data)) {
-            /* copy up to the first 9 items */
-            new_hist = g_list_append(new_hist, cur_hist->data);
-            n ++;
-        } else {
-            /* and free the rest */
-            g_free(cur_hist->data);
-        }
-        cur_hist = cur_hist->next;
-    }
-    /* put the new item on top */
-    new_hist = g_list_prepend(new_hist, g_strdup(s));
-
-    /* set new history */
-    gtk_combo_set_popdown_strings(
-        GTK_COMBO(lookup_widget(main_window, "combo_search")),
-        new_hist);
-
-    /* free old list structure */
-    g_list_free(list_search_history);
-
-    list_search_history = new_hist;
-}
-
 
 /**
  * Set proper search color in list depending on whether it is enabled.
