@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c,v 1.32 2005/05/04 01:29:25 cbiere Exp $
+ * $Id: downloads.c,v 1.59 2005/12/17 10:39:23 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: downloads.c,v 1.32 2005/05/04 01:29:25 cbiere Exp $");
+RCSID("$Id: downloads.c,v 1.59 2005/12/17 10:39:23 cbiere Exp $");
 
 #if !GTK_CHECK_VERSION(2,5,0)
 #include "pbarcellrenderer.h"
@@ -39,6 +39,7 @@ RCSID("$Id: downloads.c,v 1.32 2005/05/04 01:29:25 cbiere Exp $");
 #include "gtk/columns.h"
 #include "gtk/notebooks.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/misc.h"
 
 #include "if/core/bsched.h"
 #include "if/core/http.h"
@@ -47,18 +48,19 @@ RCSID("$Id: downloads.c,v 1.32 2005/05/04 01:29:25 cbiere Exp $");
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
-#include "lib/utf8.h"
 #include "lib/glib-missing.h"
+#include "lib/utf8.h"
+#include "lib/tm.h"
+#include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-static gchar tmpstr[4096];
-static GHashTable *parents;			/* table of parent download iterators */
-static GHashTable *parents_queue;	/* table of parent queued dl iterators */
-static GHashTable *ht_dl_iters;		/* table of iters to find downloads */
+static GHashTable *parents;			/**< table of parent download iterators */
+static GHashTable *parents_queue;	/**< table of parent queued dl iterators */
+static GHashTable *ht_dl_iters;		/**< table of iters to find downloads */
 static GtkTreeView *treeview_downloads;
 static GtkTreeView *treeview_downloads_queue;
 
-#define IO_STALLED		60		/* If nothing exchanged after that many secs */
+#define IO_STALLED		60		/**< If nothing exchanged after that many secs */
 
 
 #if 0
@@ -274,7 +276,7 @@ static gboolean
 downloads_gui_update_parent_status(download_t *d, const gchar *new_status)
 {
 	GtkTreeIter *parent;
-	GtkTreeStore *model;
+	GtkTreeStore *store;
 
 	if (!d->file_info)
 		return FALSE;
@@ -283,8 +285,8 @@ downloads_gui_update_parent_status(download_t *d, const gchar *new_status)
 	if (!parent)
 		return FALSE;
 
-	model = (GtkTreeStore *) gtk_tree_view_get_model(treeview_downloads);
-	gtk_tree_store_set(model, parent, c_dl_status, new_status, (-1));
+	store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview_downloads));
+	gtk_tree_store_set(store, parent, c_dl_status, new_status, (-1));
 	return TRUE;
 }
 
@@ -347,8 +349,9 @@ add_column(GtkTreeView *treeview, GtkType column_type, const gchar *name,
 
 
 /**
- *	Add one column to the treeview
- *	Note: Usable only for active downloads treeview.
+ *	Add one column to the treeview.
+ *
+ *	@note Usable only for active downloads treeview.
  */
 static void
 add_active_downloads_column(GtkTreeView *treeview,
@@ -369,8 +372,9 @@ add_active_downloads_column(GtkTreeView *treeview,
 }
 
 /**
- *	Add one column to the treeview
- *	Note: Usable only for downloads queue treeview.
+ *	Add one column to the treeview.
+ *
+ *	@note Usable only for downloads queue treeview.
  */
 static void
 add_queue_downloads_column(GtkTreeView *treeview,
@@ -393,7 +397,7 @@ static gint
 compare_size_func(GtkTreeModel *model,
 	GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
 {
-   	download_t *rec[2] = { NULL, NULL };
+   	download_t *rec[2];
    	GtkTreeIter *iter[2];
 	GtkTreeIter child;
 	guint i;
@@ -403,24 +407,70 @@ compare_size_func(GtkTreeModel *model,
 
 	iter[0] = a;
 	iter[1] = b;
-	
+
+	for (i = 0; i < G_N_ELEMENTS(rec); i++) {
+		rec[i] = NULL;
+    	gtk_tree_model_get(model, iter[i], column, &rec[i], (-1));
+
+		if (DL_GUI_IS_HEADER == rec[i]) {
+			gtk_tree_model_iter_nth_child(model, &child, iter[i], 0);
+    		gtk_tree_model_get(model, &child, column, &rec[i], (-1));
+		}
+		
+		if (!rec[i]) {
+			/* This might get triggered by download_gui_add() before
+			 * the child contains valid data. */
+			return 0;
+		}
+	}
+
+	return CMP(rec[1]->file_size, rec[0]->file_size);
+}
+
+static gint
+compare_range_func(GtkTreeModel *model,
+	GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
+{
+   	download_t *rec[2] = { NULL, NULL };
+   	GtkTreeIter *iter[2];
+	GtkTreeIter child;
+	filesize_t r1, r2;
+	guint i;
+	gint column = GPOINTER_TO_INT(user_data);
+
+	g_assert(column == c_queue_record || column == c_dl_record);
+
+	iter[0] = a;
+	iter[1] = b;
+
 	for (i = 0; i < G_N_ELEMENTS(rec); i++) {
     	gtk_tree_model_get(model, iter[i], column, &rec[i], (-1));
 		if (DL_GUI_IS_HEADER == rec[i]) {
 			gtk_tree_model_iter_nth_child(model, &child, iter[i], 0);
     		gtk_tree_model_get(model, &child, column, &rec[i], (-1));
 		}
+
+		if (!rec[i]) {
+			/* This might get triggered by download_gui_add() before
+			 * the child contains valid data. */
+			return 0;
+		}
 	}
 
-	g_return_val_if_fail(rec[0] && rec[1], 0);
-	return CMP(rec[1]->file_size, rec[0]->file_size);
+	r1 = rec[0]->skip;
+	r2 = rec[1]->skip;
+	if (r1 == r2) {
+		return CMP(rec[0]->size, rec[1]->size);
+	} else {
+		return CMP(r1, r2);
+	}
 }
 
 /**
- *	Add all columns to the treeview
- * 	Set titles, alignment, width, etc. here
+ *	Add all columns to the treeview.
+ * 	Set titles, alignment, width, etc. here.
  *
- *	Note: Usable only for active downloads treeview.
+ *	@note Usable only for active downloads treeview.
  */
 static void
 add_active_downloads_columns(GtkTreeView *treeview)
@@ -435,7 +485,7 @@ add_active_downloads_columns(GtkTreeView *treeview)
 		{ N_("Size"),	  c_dl_size,	 1.0, compare_size_func },
 		{ N_("Host"),	  c_dl_host,	 0.0, NULL },
 		{ N_("Loc"),	  c_dl_loc,	     0.0, NULL },
-		{ N_("Range"),	  c_dl_range,	 0.0, NULL },
+		{ N_("Range"),	  c_dl_range,	 0.0, compare_range_func },
 		{ N_("Server"),	  c_dl_server,	 0.0, NULL },
 		{ N_("Progress"), c_dl_progress, 0.0, NULL },
 		{ N_("Status"),	  c_dl_status,   0.0, NULL }
@@ -461,10 +511,10 @@ add_active_downloads_columns(GtkTreeView *treeview)
 
 
 /**
- *	Add all columns to the treeview
- * 	Set titles, alignment, width, etc. here
+ *	Add all columns to the treeview.
+ * 	Set titles, alignment, width, etc. here.
  *
- *	Note: Usable only for downloads queue treeview.
+ *	@note Usable only for downloads queue treeview.
  */
 static void
 add_queue_downloads_columns(GtkTreeView *treeview)
@@ -505,7 +555,7 @@ create_downloads_model(void)
 	static GType columns[c_dl_num];
 	GtkTreeStore *store;
 	guint i;
-	
+
 	STATIC_ASSERT(c_dl_num == G_N_ELEMENTS(columns));
 #define SET(c, x) case (c): columns[i] = (x); break
 	for (i = 0; i < G_N_ELEMENTS(columns); i++) {
@@ -556,7 +606,7 @@ create_queue_model(void)
 		}
 	}
 #undef SET
-	
+
 	store = gtk_tree_store_newv(G_N_ELEMENTS(columns), columns);
 	return GTK_TREE_MODEL(store);
 }
@@ -567,7 +617,7 @@ create_queue_model(void)
 
 
 /**
- *	Initalize the download gui
+ *	Initalize the download gui.
  *
  *	Important things in here:  initialization of hash tables, adding columns
  * 	to the treeviews, creating treeview model (what the columns mean and their
@@ -578,7 +628,7 @@ downloads_gui_init(void)
 {
 	GtkTreeSelection *selection;
 	GtkTreeView	*treeview;
-	
+
 	parents = g_hash_table_new(NULL, NULL);
 	parents_queue = g_hash_table_new(NULL, NULL);
 	ht_dl_iters = g_hash_table_new(NULL, NULL);
@@ -652,7 +702,7 @@ downloads_gui_init(void)
 }
 
 /**
- * Shutdown procedures
+ * Shutdown procedures.
  */
 void
 downloads_gui_shutdown(void)
@@ -686,12 +736,14 @@ download_gui_add(download_t *d)
 {
 	static const gchar unknown_size_str[] = "no size";
 	const gchar *vendor;
+	gchar *vendor_buf = NULL;
+	size_t vendor_size = 0;
 	GHashTable *ht;
 	GtkTreeView *treeview;
 	GtkTreeIter *parent;
 	GtkTreeIter *child;
 	GtkTreeStore *model;
-	gint host_count;
+	gint host_column;
 
 	g_return_if_fail(d);
 	g_return_if_fail(d->file_info);
@@ -700,21 +752,20 @@ download_gui_add(download_t *d)
 
 	vendor = download_vendor_str(d);
 	if (d->server->attrs & DLS_A_BANNING) {
-		static gchar buf[256]; /* MUST be static to survive this frame */
-
-		buf[0] = '*';
-		g_strlcpy(&buf[1], vendor, sizeof buf - 1);	/* Mind the -1 */
-		vendor = buf;
+		vendor_size = w_concat_strings(&vendor_buf, "*", vendor, (void *) 0);
+		vendor = vendor_buf;
 	}
 
 	if (DOWNLOAD_IS_QUEUED(d)) {
 		/* This is a queued download */
 		treeview = treeview_downloads_queue;
 		ht = parents_queue;
+		host_column = c_queue_host;
 	} else {
 		/* This is an active download */
 		treeview = treeview_downloads;
 		ht = parents;
+		host_column = c_dl_host;
 	}
 
 	model = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
@@ -724,6 +775,7 @@ download_gui_add(download_t *d)
 
 	if (DOWNLOAD_IS_QUEUED(d)) {
 		const gchar *d_file_name, *d_file_size;
+		gchar *to_free = NULL;
 
 		/* This is a queued download */
 
@@ -732,9 +784,9 @@ download_gui_add(download_t *d)
 		 *  download d into a new child node.
 		 */
 		if (!parent) {
-			host_count = 1;
 			d_file_name = guc_file_info_readable_filename(d->file_info);
-			d_file_name = lazy_locale_to_utf8(deconstify_gchar(d_file_name), 0);
+			to_free = filename_to_utf8_normalized(d_file_name, UNI_NORM_GUI);
+			d_file_name = to_free;
 
 			if (d->file_info->file_size_known)
 				d_file_size = short_size(d->file_info->size);
@@ -750,11 +802,7 @@ download_gui_add(download_t *d)
 				c_queue_record, &drecord,
 				(-1));
 
-			if (DL_GUI_IS_HEADER == drecord) {
-				host_count = gtk_tree_model_iter_n_children(
-					(GtkTreeModel *) model, parent);
-				g_assert(host_count > 0);
-			} else {
+			if (DL_GUI_IS_HEADER != drecord) {
 				gchar *filename, *host, *country, *size, *server, *status;
 				GtkTreeIter *iter;
 
@@ -772,8 +820,11 @@ download_gui_add(download_t *d)
 				/* No header entry so we will create one */
 				iter = tree_iter_new();
 				gtk_tree_store_append(model, iter, parent);
-				host_count = 2;
 
+				/* XXX: This will trigger compare_size_func() but the
+				 *		child contains no valid data yet.
+				 */
+				
 				/* Copy the old parents info into the new node */
 				gtk_tree_store_set(model, iter,
 					c_queue_filename, "\"",
@@ -821,21 +872,22 @@ download_gui_add(download_t *d)
 		  	c_queue_host, guc_download_get_hostname(d),
 		  	c_queue_loc, guc_download_get_country(d),
 	      	c_queue_size, d_file_size,
-	      	c_queue_server, lazy_locale_to_utf8(vendor, 0),
+	      	c_queue_server, vendor,
 	      	c_queue_status, NULL,
 		  	c_queue_record, d,
    	      	(-1));
-
+		G_FREE_NULL(to_free);
 	} else {
 		const gchar *d_file_name, *d_file_size;
+		gchar *to_free = NULL;
 		gint progress;
 
 		/* This is an active download */
 
 		if (!parent) {
-			host_count = 1;
 			d_file_name = guc_file_info_readable_filename(d->file_info);
-			d_file_name = lazy_locale_to_utf8(deconstify_gchar(d_file_name), 0);
+			to_free = filename_to_utf8_normalized(d_file_name, UNI_NORM_GUI);
+			d_file_name = to_free;
 
 			if (d->file_info->file_size_known)
 				d_file_size = short_size(d->file_info->size);
@@ -851,21 +903,16 @@ download_gui_add(download_t *d)
 				c_dl_record, &drecord,
 				(-1));
 
-			if (DL_GUI_IS_HEADER == drecord) {
-				host_count = gtk_tree_model_iter_n_children(
-								(GtkTreeModel *) model, parent);
-				g_assert(host_count > 0);
-			} else {
+			if (DL_GUI_IS_HEADER != drecord) {
 				gchar *filename, *host, *size, *range, *status,
 					*server, *country;
-				gfloat percent_done;
 				GtkTreeIter *iter;
+				gchar tmp[1024];
 
 				g_assert(find_download(drecord) != NULL);
 
 				/* No header entry so we will create one */
 				iter = tree_iter_new();
-				host_count = 2;
 
 				/* Copy the old parents info into a new node */
 				gtk_tree_store_append(model, iter, parent);
@@ -881,6 +928,10 @@ download_gui_add(download_t *d)
 					c_dl_status, &status,
 					(-1));
 
+				/* XXX: This will trigger compare_size_func() but the
+				 *		child contains no valid data yet.
+				 */
+
 				gtk_tree_store_set(model, iter,
 			      	c_dl_filename, "\"",
 			  		c_dl_host, host,
@@ -893,11 +944,8 @@ download_gui_add(download_t *d)
 	      			c_dl_record, drecord,
 		        	(-1));
 
-				percent_done = 100.0 * guc_download_total_progress(d);
-				progress = percent_done;
-				gm_snprintf(tmpstr, sizeof(tmpstr),
-					"%.02f%% [%d/%d]  TR:  -",
-					percent_done,
+				progress = 100.0 * guc_download_total_progress(d);
+				gm_snprintf(tmp, sizeof tmp, _("[%u/%u]  TR:  -"),
 					d->file_info->recvcount, d->file_info->lifecount);
 
 				/* Clear the old info */
@@ -909,7 +957,7 @@ download_gui_add(download_t *d)
 					c_dl_range, NULL,
 					c_dl_server, NULL,
 					c_dl_progress, CLAMP(progress, 0, 100),
-					c_dl_status, tmpstr,
+					c_dl_status, tmp,
 					c_dl_record, DL_GUI_IS_HEADER,
 					(-1));
 
@@ -941,11 +989,12 @@ download_gui_add(download_t *d)
 			c_dl_loc, guc_download_get_country(d),
 			c_dl_size, d_file_size,
 			c_dl_range, NULL,
-			c_dl_server, lazy_locale_to_utf8(vendor, 0),
+			c_dl_server, vendor,
 			c_dl_progress, CLAMP(progress, 0, 100),
 			c_dl_status, NULL,
 			c_dl_record, d,
 			(-1));
+		G_FREE_NULL(to_free);
 	}
 
 	REGRESSION(
@@ -963,14 +1012,16 @@ download_gui_add(download_t *d)
 		 * hash table */
 		add_parent_with_fi_handle(ht, d->file_info->fi_handle, child);
 	} else {
-		g_assert(host_count > 0);
-		gm_snprintf(tmpstr, sizeof(tmpstr),
-			NG_("%u host", "%u hosts", host_count), host_count);
-		gtk_tree_store_set(model, parent, c_dl_host, tmpstr, (-1));
+		guint hosts = d->file_info->lifecount;
+		gchar tmp[1024];
+
+		gm_snprintf(tmp, sizeof tmp, NG_("%u host", "%u hosts", hosts), hosts);
+		gtk_tree_store_set(model, parent, host_column, tmp, (-1));
 	}
 
 	/* Download was added to either the active or queued downloads treeview */
 	d->visible = TRUE;
+	WFREE_NULL(vendor_buf, vendor_size);
 }
 
 
@@ -1026,11 +1077,12 @@ download_gui_remove(download_t *d)
 		g_assert(iter == parent);
 		remove_parent_with_fi_handle(ht, d->file_info->fi_handle);
 	} else if (n > 2) {
-		guint count = n - 1;
+		guint hosts = d->file_info->lifecount;
+		gchar tmp[1024];
+
 		g_assert(iter != parent);
-		gm_snprintf(tmpstr, sizeof(tmpstr),
-			NG_("%u host", "%u hosts", count), count);
-		gtk_tree_store_set(store, parent, host_column, tmpstr, (-1));
+		gm_snprintf(tmp, sizeof tmp, NG_("%u host", "%u hosts", hosts), hosts);
+		gtk_tree_store_set(store, parent, host_column, tmp, (-1));
 	} else if (2 == n) {
 		GtkTreeIter *child_iter;
 		gint i;
@@ -1104,7 +1156,7 @@ download_gui_remove(download_t *d)
 				c_dl_range, range,
 				c_dl_server, server,
 				c_dl_progress, CLAMP(progress, 0, 100),
-				c_dl_status, tmpstr,
+				c_dl_status, status,
 				c_dl_record, drecord,
 				(-1));
 
@@ -1151,11 +1203,11 @@ download_gui_remove(download_t *d)
 /**
  *	Updates the given column of the given treeview
  */
-void
+static void
 gui_update_download_column(download_t *d, GtkTreeView *tree_view,
 	gint column, const gchar *value)
 {
-	GtkTreeStore *model = (GtkTreeStore *) gtk_tree_view_get_model(tree_view);
+	GtkTreeStore *model = cast_to_gpointer(gtk_tree_view_get_model(tree_view));
 	GtkTreeIter *iter;
 
 	g_assert(d);
@@ -1170,7 +1222,7 @@ gui_update_download_column(download_t *d, GtkTreeView *tree_view,
 		return;
 	}
 
-	gtk_tree_store_set(model, iter, column, lazy_locale_to_utf8(value, 0), (-1));
+	gtk_tree_store_set(model, iter, column, value, (-1));
 }
 
 
@@ -1182,6 +1234,8 @@ void
 gui_update_download_server(download_t *d)
 {
 	const gchar *server;
+	gchar *buf = NULL;
+	size_t size = 0;
 
 	g_assert(d);
 	g_assert(!DOWNLOAD_IS_QUEUED(d));
@@ -1195,13 +1249,11 @@ gui_update_download_server(download_t *d)
 	 */
 	server = download_vendor(d);
 	if (d->server->attrs & DLS_A_BANNING) {
-		static gchar buf[256]; /* MUST be static to survive this frame */
-
-		buf[0] = '*';
-		g_strlcpy(&buf[1], server, sizeof buf - 1); /* Mind the -1 */
+		size = w_concat_strings(&buf, "*", server, (void *) 0);
 		server = buf;
 	}
 	gui_update_download_column(d, treeview_downloads, c_dl_server, server);
+	WFREE_NULL(buf, size);
 }
 
 
@@ -1212,8 +1264,8 @@ void
 gui_update_download_range(download_t *d)
 {
 	filesize_t len;
-	size_t rw;
 	const gchar *and_more = "";
+	gchar buf[256];
 
 	g_assert(d);
 	g_assert(!DOWNLOAD_IS_QUEUED(d));
@@ -1229,16 +1281,31 @@ gui_update_download_range(download_t *d)
 
 	len += d->overlap_size;
 
-	rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s%s",
-			compact_size(len), and_more);
-
+	concat_strings(buf, sizeof buf,
+		compact_size(len), and_more, d->skip ? " @ " : "", (void *) 0);
 	if (d->skip)
-		gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, " @ %s",
-			compact_size(d->skip));
+		g_strlcat(buf, compact_size(d->skip), sizeof buf);
 
-	gui_update_download_column(d, treeview_downloads, c_dl_range, tmpstr);
+	gui_update_download_column(d, treeview_downloads, c_dl_range, buf);
 }
 
+/**
+ *	Update the size column of the active downloads treeview
+ */
+void
+gui_update_download_size(download_t *d)
+{
+	gchar buf[256];
+
+	g_assert(d);
+	g_assert(!DOWNLOAD_IS_QUEUED(d));
+
+	if (!d->file_info->file_size_known)
+		return;
+
+	concat_strings(buf, sizeof buf, short_size(d->size), (void *) 0);
+	gui_update_download_column(d, treeview_downloads, c_dl_size, buf);
+}
 
 /**
  *	Update the host column of the active downloads treeview
@@ -1251,6 +1318,8 @@ gui_update_download_host(download_t *d)
 
 	gui_update_download_column(d, treeview_downloads,
 		c_dl_host, guc_download_get_hostname(d));
+	gui_update_download_column(d, treeview_downloads,
+		c_dl_loc, guc_download_get_country(d));
 }
 
 
@@ -1262,19 +1331,20 @@ void
 gui_update_download(download_t *d, gboolean force)
 {
 	static GtkNotebook *notebook = NULL;
-	const gchar *a = NULL;
-	time_t now = time((time_t *) NULL);
-	struct dl_file_info *fi;
+	time_t now = tm_time();
+	fileinfo_t *fi;
 	download_t *drecord;
-	gint rw;
 	GtkTreeView *treeview;
 	GtkTreeIter *parent;
-	GtkTreeStore *model;
+	GtkTreeStore *store;
 	GHashTable *ht;
     gint current_page;
 	gboolean looking = TRUE;
 	gboolean has_header = FALSE;
 	gboolean is_queued;
+	gchar status_buf[4096];
+	const gchar *status_ptr = status_buf;
+	size_t rw;
 
     if (d->last_gui_update == now && !force)
 		return;
@@ -1295,17 +1365,19 @@ gui_update_download(download_t *d, gboolean force)
 		ht = parents;
 	}
 
-	model = (GtkTreeStore *) gtk_tree_view_get_model(treeview);
+	status_buf[0] = '\0';
+	store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
 	parent = find_parent_with_fi_handle(ht, d->file_info->fi_handle);
 	g_assert(parent != NULL);
 
-	gtk_tree_model_get(GTK_TREE_MODEL(model), parent,
+	gtk_tree_model_get(GTK_TREE_MODEL(store), parent,
 		is_queued ? c_queue_record : c_dl_record, &drecord,
 		(-1));
 
 	if (DL_GUI_IS_HEADER == drecord) {
 		/* There is a header entry, we need to update it */
 		const gchar *status = NULL;
+		gchar tmp[1024];
 		gint progress = 0;
 
 		has_header = TRUE;
@@ -1315,26 +1387,24 @@ gui_update_download(download_t *d, gboolean force)
 			progress = 100;
 			status = _("Complete");
 		} else /* if (GTA_DL_RECEIVING == d->status && d->pos > d->skip)*/ {
-			gfloat percent_done = guc_download_total_progress(d);
 
 			if (fi->recvcount && fi->recv_last_rate) {
 				guint s = (fi->size - fi->done) / fi->recv_last_rate;
 
-				gm_snprintf(tmpstr, sizeof(tmpstr),
-					"%.02f%%  (%s)  [%d/%d]  TR:  %s",
-					percent_done * 100.0, short_rate(fi->recv_last_rate),
+				gm_snprintf(tmp, sizeof tmp, _("(%s)  [%d/%d]  TR:  %s"),
+					short_rate(fi->recv_last_rate),
 					fi->recvcount, fi->lifecount, short_time(s));
 			} else {
-				gm_snprintf(tmpstr, sizeof(tmpstr), "%.02f%% [%d/%d]",
-				percent_done * 100.0, fi->recvcount, fi->lifecount);
+				gm_snprintf(tmp, sizeof tmp, "[%d/%d]",
+					fi->recvcount, fi->lifecount);
 			}
 
-			progress = percent_done * 100;
-			status = tmpstr;
+			progress = guc_download_total_progress(d) * 100;
+			status = tmp;
 		}
 
 		if (status && !is_queued) {
-			gtk_tree_store_set(model, parent,
+			gtk_tree_store_set(store, parent,
 				c_dl_status, status,
 				c_dl_progress, CLAMP(progress, 0, 100),
       			(-1));
@@ -1353,7 +1423,7 @@ gui_update_download(download_t *d, gboolean force)
 		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
 
     current_page = gtk_notebook_get_current_page(notebook);
-    if (current_page != nb_main_page_downloads)
+    if (current_page != nb_main_page_dl_active)
         looking = FALSE;
 
 	if (!looking) {
@@ -1382,42 +1452,40 @@ gui_update_download(download_t *d, gboolean force)
 	case GTA_DL_ACTIVE_QUEUED:	/* JA, 31 jan 2003 Active queueing */
 		{
 			gint elapsed = delta_time(now, d->last_update);
-			rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s", _("Queued"));
+			rw = gm_snprintf(status_buf, sizeof status_buf, "%s", _("Queued"));
 
+			
 			if (guc_get_parq_dl_position(d) > 0) {
 
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-					_(" (slot %d"),		/* ) */
-					guc_get_parq_dl_position(d));
+				rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+					_(" (slot %u"), (guint) guc_get_parq_dl_position(d));
 
 				if (guc_get_parq_dl_queue_length(d) > 0) {
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-						" / %d", guc_get_parq_dl_queue_length(d));
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+						"/%u", (guint) guc_get_parq_dl_queue_length(d));
 				}
 
-				if (guc_get_parq_dl_eta(d)  > 0) {
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				if (guc_get_parq_dl_eta(d) > 0) {
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 						_(", ETA: %s"),
-						short_time((guc_get_parq_dl_eta(d)
-							- elapsed)));
+						short_time(guc_get_parq_dl_eta(d) - elapsed));
 				}
 
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, /* ( */ ")");
+				rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw, ")");
 			}
 
-			rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-					_(" retry in %ds"),
-					(gint) (guc_get_parq_dl_retry_delay(d)
-						- elapsed));
+			rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+					_(" retry in %us"),
+					(guint) (guc_get_parq_dl_retry_delay(d) - elapsed));
 		}
-		a = tmpstr;
+		status_ptr = status_buf;
 		break;
 	case GTA_DL_QUEUED:
-		a = d->remove_msg ? d->remove_msg : "";
+		status_ptr = d->remove_msg ? d->remove_msg : "";
 		break;
 
 	case GTA_DL_CONNECTING:
-		a = _("Connecting...");
+		status_ptr = _("Connecting..."); 
 		break;
 
 	case GTA_DL_PUSH_SENT:
@@ -1428,40 +1496,41 @@ gui_update_download(download_t *d, gboolean force)
 
 				if (cp->done) {
 					if (cp->sent)
-						rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s",
+						rw = gm_snprintf(status_buf, sizeof status_buf, "%s",
 								cp->directly ?
 									_("Push sent directly") : _("Push sent"));
 					else
-						rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s",
+						rw = gm_snprintf(status_buf, sizeof status_buf, "%s",
 								_("Failed to send push"));
 				} else
-					rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s",
+					rw = gm_snprintf(status_buf, sizeof status_buf, "%s",
 							_("Sending push"));
 
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, " via %s",
-						ip_port_to_gchar(cproxy_ip(cp), cproxy_port(cp)));
+				rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+						_(" via %s"),
+						host_addr_port_to_string(cproxy_addr(cp),
+						cproxy_port(cp)));
 
 				if (!cp->done) {
+					const gchar *s;
 					switch (cp->state) {
-					case HTTP_AS_CONNECTING:	a = _("Connecting"); break;
-					case HTTP_AS_REQ_SENDING:	a = _("Sending request"); break;
-					case HTTP_AS_REQ_SENT:		a = _("Request sent"); break;
-					case HTTP_AS_HEADERS:		a = _("Reading headers"); break;
-					default:					a = "..."; break;
+					case HTTP_AS_CONNECTING:	s = _("Connecting"); break;
+					case HTTP_AS_REQ_SENDING:	s = _("Sending request"); break;
+					case HTTP_AS_REQ_SENT:		s = _("Request sent"); break;
+					case HTTP_AS_HEADERS:		s = _("Reading headers"); break;
+					default:					s = "..."; break;
 					}
 
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-						": %s", a);
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+							": %s", s);
 				}
-
-				a = tmpstr;
 			} else {
 				switch (d->status) {
 				case GTA_DL_PUSH_SENT:
-					a = _("Push sent");
+					status_ptr = _("Push sent");
 					break;
 				case GTA_DL_FALLBACK:
-					a = _("Falling back to push");
+					status_ptr = _("Falling back to push");
 					break;
 				default:
 					break;
@@ -1472,23 +1541,24 @@ gui_update_download(download_t *d, gboolean force)
 
 	case GTA_DL_REQ_SENDING:
 		if (d->req != NULL) {
-			gm_snprintf(tmpstr, sizeof(tmpstr), _("Sending request (%d%%)"),
-				(gint) guc_download_get_http_req_percent(d));
-			a = tmpstr;
-		} else
-			a = _("Sending request");
+			gm_snprintf(status_buf, sizeof status_buf,
+				_("Sending request (%u%%)"),
+				(guint) guc_download_get_http_req_percent(d));
+		} else {
+			status_ptr = _("Sending request");
+		}
 		break;
 
 	case GTA_DL_REQ_SENT:
-		a = _("Request sent");
+		status_ptr = _("Request sent");
 		break;
 
 	case GTA_DL_HEADERS:
-		a = _("Receiving headers");
+		status_ptr = _("Receiving headers");
 		break;
 
 	case GTA_DL_ABORTED:
-		a = d->unavailable ? _("Aborted (Server down)") : _("Aborted");
+		status_ptr = d->unavailable ? _("Aborted (Server down)") : _("Aborted");
 
 		/* If this download is aborted, it's possible all the downloads in this
 	     * parent node (if there is one) are aborted too. If so, update parent*/
@@ -1501,29 +1571,27 @@ gui_update_download(download_t *d, gboolean force)
 		if (d->last_update != d->start_date) {
 			guint32 t = d->last_update - d->start_date;
 
-			gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s) %s",
+			gm_snprintf(status_buf, sizeof status_buf, "%s (%s) %s",
 				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
 				short_rate((d->range_end - d->skip + d->overlap_size) / t),
 				short_time(t));
 		} else {
-			gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s)",
+			gm_snprintf(status_buf, sizeof status_buf, "%s (%s)",
 				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
 				_("< 1s"));
 		}
-		a = tmpstr;
 		break;
 
 	case GTA_DL_VERIFY_WAIT:
 		g_assert(FILE_INFO_COMPLETE(fi));
-		g_strlcpy(tmpstr, _("Waiting for SHA1 checking..."), sizeof(tmpstr));
-		a = tmpstr;
+		utf8_strlcpy(status_buf, _("Waiting for SHA1 checking..."),
+			sizeof status_buf);
 		break;
 
 	case GTA_DL_VERIFYING:
 		g_assert(FILE_INFO_COMPLETE(fi));
-		gm_snprintf(tmpstr, sizeof(tmpstr),
+		gm_snprintf(status_buf, sizeof status_buf,
 			_("Computing SHA1 (%.02f%%)"), fi->cha1_hashed * 100.0 / fi->size);
-		a = tmpstr;
 		break;
 
 	case GTA_DL_VERIFIED:
@@ -1536,14 +1604,14 @@ gui_update_download(download_t *d, gboolean force)
 			gboolean sha1_ok = fi->cha1 &&
 				(fi->sha1 == NULL || sha1_eq(fi->sha1, fi->cha1));
 
-			rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s %s",
+			rw = gm_snprintf(status_buf, sizeof status_buf, "%s %s",
 				fi->sha1 == NULL ? _("SHA1 figure") : _("SHA1 check"),
 				fi->cha1 == NULL ?	_("ERROR") :
 				sha1_ok ?			_("OK") :
 									_("FAILED"));
 			if (fi->cha1 && fi->cha1_hashed) {
 				guint elapsed = fi->cha1_elapsed;
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 					" (%s) %s",
 					short_rate(fi->cha1_hashed / (elapsed ? elapsed : 1)),
 					short_time(fi->cha1_elapsed));
@@ -1551,16 +1619,16 @@ gui_update_download(download_t *d, gboolean force)
 
 			switch (d->status) {
 			case GTA_DL_MOVE_WAIT:
-				g_strlcpy(&tmpstr[rw], _("; Waiting for moving..."),
-					sizeof(tmpstr)-rw);
+				utf8_strlcpy(&status_buf[rw], _("; Waiting for moving..."),
+					sizeof status_buf - rw);
 				break;
 			case GTA_DL_MOVING:
-				gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 					_("; Moving (%.02f%%)"), fi->copied * 100.0 / fi->size);
 				break;
 			case GTA_DL_DONE:
 				if (fi->copy_elapsed) {
-					gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 						_("; Moved (%s) %s"),
 						short_rate(fi->copied / fi->copy_elapsed),
 						short_time(fi->copy_elapsed));
@@ -1570,16 +1638,14 @@ gui_update_download(download_t *d, gboolean force)
 				break;
 			}
 		}
-		a = tmpstr;
 		break;
 
 	case GTA_DL_RECEIVING:
 		if (d->pos > d->skip) {
 			gint bps;
 			guint32 avg_bps;
-			gfloat progress_source, progress_total;
+			gfloat progress_source;
 
-			progress_total = guc_download_total_progress(d);
 			progress_source = guc_download_source_progress(d);
 
 			bps = bio_bps(d->bio);
@@ -1601,40 +1667,42 @@ gui_update_download(download_t *d, gboolean force)
 
                 s = remain / avg_bps;
 
-				rw = gm_snprintf(tmpstr, sizeof(tmpstr),
-					"%.02f%% / %.02f%% ",
-					progress_source * 100.0, progress_total * 100.0);
+				if (progress_source > 1.0) {
+					rw = gm_snprintf(status_buf, sizeof status_buf, "%.02f%% ",
+						progress_source * 100.0);
+				}
 
 				if (delta_time(now, d->last_update) > IO_STALLED)
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr) - rw,
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 							"%s ", _("(stalled)"));
 				else
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr) - rw,
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 						"(%s) ", short_rate(bps));
 
 				if (!has_header) {
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 						"[%d/%d]", fi->recvcount, fi->lifecount);
 				}
 
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-					" TR: %s", s ? short_time(s) : "-");
+				rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
+					_(" TR: %s"), s ? short_time(s) : "-");
 
 				if (!has_header && fi->recv_last_rate) {
 					s = (fi->size - fi->done) / fi->recv_last_rate;
 
-					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					rw += gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 						" / %s", short_time(s));
 
 					if (fi->recvcount > 1) {
 						bs = fi->recv_last_rate / 1024.0;
 
-						rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-							" (%s)", short_rate(bps));
+						rw += gm_snprintf(&status_buf[rw],
+								sizeof status_buf - rw,
+								" (%s)", short_rate(bps));
 					}
 				}
 			} else if (delta_time(now, d->last_update) > IO_STALLED) {
-				rw = gm_snprintf(tmpstr, sizeof(tmpstr),
+				rw = gm_snprintf(status_buf, sizeof status_buf,
 						" %s", _("(stalled)"));
 			}
 
@@ -1642,40 +1710,38 @@ gui_update_download(download_t *d, gboolean force)
 			 * If source is a partial source, show it.
 			 */
 			if (d->ranges != NULL) {
-				gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				gm_snprintf(&status_buf[rw], sizeof status_buf - rw,
 					" <PFS %.02f%%>", d->ranges_size * 100.0 / fi->size);
 			}
 
-			a = tmpstr;
-		} else
-			a = _("Connected");
+		} else {
+			status_ptr = _("Connected");
+		}
 		break;
 
 	case GTA_DL_ERROR:
-		a = d->remove_msg ? d->remove_msg : _("Unknown error");
+		status_ptr = d->remove_msg ? d->remove_msg : _("Unknown error");
 		break;
 
 	case GTA_DL_TIMEOUT_WAIT:
 		{
 			gint when = d->timeout_delay - delta_time(now, d->last_update);
-			gm_snprintf(tmpstr, sizeof(tmpstr), _("Retry in %ds"),
-				MAX(0, when));
+			gm_snprintf(status_buf, sizeof status_buf, _("Retry in %us"),
+				(guint) MAX(0, when));
 		}
-		a = tmpstr;
 		break;
 	case GTA_DL_SINKING:
 		{
-			gchar bytes[32];
+			gchar bytes[UINT64_DEC_BUFLEN];
 
-			uint64_to_string_buf(bytes, sizeof bytes, d->sinkleft);
-			gm_snprintf(tmpstr, sizeof(tmpstr),
+			uint64_to_string_buf(d->sinkleft, bytes, sizeof bytes);
+			gm_snprintf(status_buf, sizeof status_buf,
 				_("Sinking (%s bytes left)"), bytes);
 		}
-		a = tmpstr;
 		break;
 	default:
-		gm_snprintf(tmpstr, sizeof(tmpstr), _("Unknown status %u"), d->status);
-		a = tmpstr;
+		gm_snprintf(status_buf, sizeof status_buf,
+			_("Unknown status %u"), d->status);
 	}
 
 	if (d->status != GTA_DL_TIMEOUT_WAIT)
@@ -1709,8 +1775,11 @@ gui_update_download(download_t *d, gboolean force)
 			progress = 100 * guc_download_source_progress(d);
 			break;
 		}
-		gtk_tree_store_set(model, iter,
-			c_dl_status, (a && a[0] != '\0') ? a : NULL,
+		
+		utf8_enforce(status_buf, sizeof status_buf,
+			status_ptr ? status_ptr : "");
+		gtk_tree_store_set(store, iter,
+			c_dl_status, status_buf[0] != '\0' ? status_buf : NULL,
 			c_dl_progress, CLAMP(progress, 0, 100),
 			(-1));
 	}
@@ -1720,6 +1789,7 @@ gui_update_download(download_t *d, gboolean force)
 
     if (DOWNLOAD_IS_QUEUED(d)) {
 		GtkTreeIter *iter;
+		GtkTreeModel *model;
 
 		iter = find_download(d);
 		if (!iter)
@@ -1729,10 +1799,11 @@ gui_update_download(download_t *d, gboolean force)
 					d->file_info->fi_handle);
 		g_return_if_fail(parent != NULL);
 
-		model =	(GtkTreeStore *) gtk_tree_view_get_model(
-									treeview_downloads_queue);
-		gtk_tree_store_set(model, iter,
-			c_queue_status, (a && a[0] != '\0') ? a : NULL,
+		utf8_enforce(status_buf, sizeof status_buf,
+			status_ptr ? status_ptr : "");
+		model = gtk_tree_view_get_model(treeview_downloads_queue);
+		gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+			c_queue_status, status_buf[0] != '\0' ? status_buf : NULL,
 			(-1));
 		return;
 	}
@@ -1892,4 +1963,15 @@ downloads_gui_collapse_all(GtkTreeView *tree_view)
 	gtk_tree_view_collapse_all(tree_view);
 }
 
+/**
+ * Periodically called to update downloads display.
+ */
+void
+downloads_gui_update_display(time_t unused_now)
+{
+	(void) unused_now;
+
+	/* Nothing needed for GTK2 */
+}
+
 /* vi: set ts=4 sw=4 cindent: */
