@@ -1,10 +1,8 @@
 /* 
- * $Id: search_cb.c,v 1.34 2005/04/27 15:36:21 cbiere Exp $
+ * $Id: search_cb.c,v 1.63 2006/01/10 10:12:02 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
- * GUI filtering functions.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,20 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * GUI filtering functions.
+ *
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #include "gtk/gui.h"
 
-RCSID("$Id: search_cb.c,v 1.34 2005/04/27 15:36:21 cbiere Exp $");
+RCSID("$Id: search_cb.c,v 1.63 2006/01/10 10:12:02 cbiere Exp $");
 
 #include <gdk/gdkkeysyms.h>
 
@@ -35,6 +44,7 @@ RCSID("$Id: search_cb.c,v 1.34 2005/04/27 15:36:21 cbiere Exp $");
 
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/search.h"
+#include "gtk/statusbar.h"
 #include "gtk/misc.h"
 #include "gtk/columns.h"
 #include "gtk/notebooks.h"
@@ -42,6 +52,7 @@ RCSID("$Id: search_cb.c,v 1.34 2005/04/27 15:36:21 cbiere Exp $");
 #include "gtk/settings.h"
 
 #include "if/gui_property.h"
+#include "if/gui_property_priv.h"
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
@@ -62,23 +73,40 @@ static search_t *search_selected = NULL;
 static void
 refresh_popups(void)
 {
-	static const char * const popup_names[] = {
-		"popup_search_download",
-		"popup_search_drop_name",
-		"popup_search_drop_sha1",
-		"popup_search_drop_host",
-		"popup_search_drop_name_global",
-		"popup_search_drop_host_global",
-		"popup_search_autodownload_name",
-		"popup_search_autodownload_sha1",
-		"popup_search_new_from_selected",
+	/*
+	 * The following popup items are set insensitive if nothing is currently
+	 * selected (actually if the cursor is unset).
+	 */
+	static const struct {
+		const gchar *name;
+	} menu[] = {
+		{	"popup_search_download" },
+		{	"popup_search_drop" },
+		{	"popup_search_drop_global" },
+		{	"popup_search_autodownload" },
+		{	"popup_search_new_from_selected" },
+		{	"popup_search_metadata" },
+		{	"popup_search_browse_host" },
 	};
 	search_t *search = search_gui_get_current_search();
-	gboolean sensitive = NULL != search;
+	gboolean sensitive;
 	guint i;
 
-	for (i = 0; i < G_N_ELEMENTS(popup_names); i++)
-		gtk_widget_set_sensitive(lookup_widget(popup_search, popup_names[i]),
+	if (!search) {
+		sensitive = FALSE;
+	} else {
+		GtkTreePath *path;
+		
+		gtk_tree_view_get_cursor(GTK_TREE_VIEW(search->tree_view), &path, NULL);
+		sensitive = NULL != path;	
+	}
+	
+	gtk_widget_set_sensitive(
+		lookup_widget(main_window, "button_search_download"),
+		sensitive);
+
+	for (i = 0; i < G_N_ELEMENTS(menu); i++)
+		gtk_widget_set_sensitive(lookup_widget(popup_search, menu[i].name),
 			sensitive);
 
     if (search) {
@@ -87,7 +115,8 @@ refresh_popups(void)
 			!guc_search_is_frozen(search->search_handle));
 		gtk_widget_set_sensitive(
 			lookup_widget(popup_search_list, "popup_search_resume"),
-			guc_search_is_frozen(search->search_handle));
+			guc_search_is_frozen(search->search_handle)
+				&& !search_gui_is_expired(search));
 		if (search->passive)
 			gtk_widget_set_sensitive(
 				lookup_widget(popup_search_list, "popup_search_restart"),
@@ -108,11 +137,58 @@ void
 on_combo_entry_searches_activate(GtkEditable *unused_editable,
 	gpointer unused_udata)
 {
-    /* FIXME */
 	(void) unused_editable;
 	(void) unused_udata;
 }
 
+void
+on_entry_search_activate(GtkEditable *unused_editable,
+	gpointer unused_udata)
+{
+    /*
+     * Delegate to: on_button_search_clicked.
+     *      --BLUE, 30/04/2002
+     */
+
+	(void) unused_editable;
+	(void) unused_udata;
+
+	search_gui_new_search_entered();
+}
+
+/**
+ *	When a search string is entered, activate the search button
+ */
+void
+on_entry_search_changed(GtkEditable *editable, gpointer unused_udata)
+{
+	gchar *s = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
+	gchar *normalized;
+	gboolean changed;
+
+	(void) unused_udata;
+
+	/* Gimmick: Normalize the input on the fly because Gtk+ currently
+	 * renders them differently (for example decomposed) if they're are
+	 * not in Normalization Form Canonic (NFC)
+	 */
+	normalized = utf8_normalize(s, UNI_NORM_GUI);
+	changed = normalized != s && 0 != strcmp(s, normalized);
+	
+	if (changed)
+		gtk_entry_set_text(
+			GTK_ENTRY(lookup_widget(main_window, "entry_search")), normalized);
+
+	if (normalized != s)
+		G_FREE_NULL(normalized);
+
+	if (!changed) {
+		g_strstrip(s);
+		gtk_widget_set_sensitive(lookup_widget(main_window, "button_search"),
+			s[0] != '\0');
+	}
+	G_FREE_NULL(s);
+}
 
 void
 on_search_popdown_switch(GtkWidget *unused_w, gpointer unused_data)
@@ -147,9 +223,10 @@ on_search_notebook_focus_tab(GtkNotebook *notebook, GtkNotebookTab unused_tab,
 
 	(void) unused_tab;
 	(void) unused_udata;
+
 	page_num = gtk_notebook_get_current_page(notebook);
 	widget = gtk_notebook_get_nth_page(notebook, page_num);
-	sch = (search_t *) gtk_object_get_user_data(GTK_OBJECT(widget));
+	sch = gtk_object_get_user_data(GTK_OBJECT(widget));
 
 	g_return_if_fail(sch);
 
@@ -189,94 +266,9 @@ on_search_selected(GtkItem *unused_item, gpointer data)
 void
 on_button_search_clicked(GtkButton *unused_button, gpointer unused_udata)
 {
-	GtkWidget *widget;
-	gchar *e;
-	
 	(void) unused_button;
 	(void) unused_udata;
-
-	/*
-	 * Even though we might not be on_the_net() yet, record the search.
-	 * There is a callback mechanism when a new node is connected, which
-	 * will launch the search there if it has not been sent already.
-	 *              --patch from Mark Schreiber, 10/01/2002
-	 */
-
-	widget = lookup_widget(main_window, "entry_search");
-	e = STRTRACK(gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1));
-    g_strstrip(e);
-    if ('\0' != *e) {
-        filter_t *default_filter;
-        search_t *search;
-        gboolean res;
-
-		/*
-		 * It's important gui_search_history_add is called before
-		 * new_search, otherwise the search entry will not be
-		 * cleared.
-		 *      --BLUE, 04/05/2002
-		 */
-		gui_search_history_add(e);
-
-
-		/*
-		 * We have to capture the selection here already, because
-		 * new_search will trigger a rebuild of the menu as a
-		 * side effect.
-		 */
-		default_filter = (filter_t *)option_menu_get_selected_data
-				(lookup_widget(main_window, "optionmenu_search_filter"));
-
-		res = search_gui_new_search(e, 0, &search);
-
-		/*
-		 * If we should set a default filter, we do that.
-		 */
-		if (res && (default_filter != NULL)) {
-			rule_t *rule = filter_new_jump_rule(
-				default_filter, RULE_FLAG_ACTIVE);
-
-			/*
-			 * Since we don't want to distrub the shadows and
-			 * do a "force commit" without the user having pressed
-			 * the "ok" button in the dialog, we add the rule
-			 * manually.
-			 */
-			search->filter->ruleset =
-                g_list_append(search->filter->ruleset, rule);
-			rule->target->refcount ++;
-		}
-
-		if (!res)
-			gdk_beep();
-	}
-
-	gtk_widget_grab_focus(widget);
-	G_FREE_NULL(e);
-}
-
-void
-on_entry_search_activate(GtkEditable *unused_editable, gpointer user_data)
-{
-    /*
-     * Delegate to: on_button_search_clicked.
-     *      --BLUE, 30/04/2002
-     */
-
-	(void) unused_editable;
-	on_button_search_clicked(NULL, user_data);
-}
-
-void
-on_entry_search_changed(GtkEditable * editable, gpointer unused_udata)
-{
-	gchar *e = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
-
-	(void) unused_udata;
-	g_strstrip(e);
-	gtk_widget_set_sensitive(
-		lookup_widget(main_window, "button_search"), '\0' != *e);
-	G_FREE_NULL(e);
+	search_gui_new_search_entered();
 }
 
 void
@@ -414,7 +406,7 @@ on_button_search_filter_clicked(GtkButton *unused_button, gpointer unused_udata)
 }
 
 void
-on_button_search_download_selected_clicked(GtkButton *unused_button,
+on_button_search_download_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
 {
 	(void) unused_button;
@@ -423,54 +415,98 @@ on_button_search_download_selected_clicked(GtkButton *unused_button,
 }
 
 gboolean
-on_tree_view_search_results_click_column(GtkTreeViewColumn *unused_column,
-	gpointer unused_udata)
+on_tree_view_search_results_click_column(GtkTreeViewColumn *column,
+	gpointer udata)
 {
-	/* FIXME:
-	 * 			+--> sort descending -> sort ascending -> unsorted -+
+	GtkTreeSortable *model;
+	search_t *sch = udata;
+	GtkSortType order;
+	gint sort_col;
+
+	/* The default treeview is empty */
+	if (!sch)
+		return FALSE;
+
+	/* N.B.: column->tree_view is not really documented but it works. */
+	model = GTK_TREE_SORTABLE(
+				gtk_tree_view_get_model(GTK_TREE_VIEW(column->tree_view)));
+
+	/*
+	 * Here we enforce a tri-state sorting. Normally, Gtk+ would only
+	 * switch between ascending and descending but never switch back
+	 * to the unsorted state.
+	 *
+	 * 			+--> sort ascending -> sort descending -> unsorted -+
      *      	|                                                   |
      *      	+-----------------------<---------------------------+
      */
-	(void) unused_column;
-	(void) unused_udata;
-	return FALSE;
-}
 
-static const record_t *
-search_get_record_at_path(GtkTreeView *tv, GtkTreePath *path)
-{
-	const GList *l = search_gui_get_searches();
-	search_t *sch = NULL;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	const record_t *rc;
+	/*
+	 * "order" is set to the current sort-order, not the previous one
+	 * i.e., Gtk+ has already changed the order
+	 */
+	g_object_get(G_OBJECT(column), "sort-order", &order, (void *) 0);
 
-	for (/* NOTHING */; NULL != l; l = g_list_next(l)) {
-		if (tv == GTK_TREE_VIEW(((search_t *) l->data)->tree_view)) {
-			sch = (search_t *) l->data;
-			break;
-		}
+	gtk_tree_sortable_get_sort_column_id(model, &sort_col, NULL);
+
+	/* If the user switched to another sort column, reset the sort order. */
+	if (sch->sort_col != sort_col)
+		sch->sort_order = SORT_NONE;
+
+	sch->sort_col = sort_col;
+
+	/* The search has to keep state about the sort order itself because
+	 * Gtk+ knows only ASCENDING/DESCENDING but not NONE (unsorted). */
+	switch (sch->sort_order) {
+	case SORT_NONE:
+	case SORT_NO_COL:
+		sch->sort_order = SORT_ASC;
+		break;
+	case SORT_ASC:
+		sch->sort_order = SORT_DESC;
+		break;
+	case SORT_DESC:
+		sch->sort_order = SORT_NONE;
+		break;
 	}
-	g_return_val_if_fail(NULL != sch, NULL);
-	
-	model = GTK_TREE_MODEL(sch->model);
-	gtk_tree_model_get_iter(model, &iter, path);
-	gtk_tree_model_get(model, &iter, c_sr_record, &rc, (-1));
 
-	return rc;
+	if (SORT_NONE == sch->sort_order) {
+		/*
+		 * Reset the sorting and let the arrow disappear from the
+		 * header. Gtk+ actually seems to change the order of the
+		 * rows back to the original order (i.e., chronological).
+		 */
+		gtk_tree_view_column_set_sort_indicator(column, FALSE);
+#if GTK_CHECK_VERSION(2,6,0)
+		gtk_tree_sortable_set_sort_column_id(model,
+			GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
+#endif /* Gtk+ >= 2.6.0 */
+	} else {
+		/*
+		 * Enforce the order as decided from the search state. Gtk+
+		 * might disagree but it'll do as told.
+		 */
+		gtk_tree_sortable_set_sort_column_id(model, sort_col,
+			SORT_ASC == sch->sort_order
+				? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
+	}
+	/* Make the column stays clickable. */
+	gtk_tree_view_column_set_clickable(column, TRUE);
+
+	return FALSE;
 }
 
 static const gchar *
 search_get_vendor_from_record(const record_t *rc)
 {
-	gchar *s;
+	const gchar *s;
 
-	g_assert(rc != NULL);	
+	g_assert(rc != NULL);
 
-	s = lookup_vendor_name(rc->results_set->vendor);
+	s = lookup_vendor_name(rc->results_set->vcode);
 	if (s == NULL)
 		return _("Unknown");
-	
+
 	if (rc->results_set->version) {
 		static gchar buf[128];
 
@@ -498,17 +534,19 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 			return;
 		}
 
-		rc = search_get_record_at_path(tv, path);
+		rc = search_gui_get_record_at_path(tv, path);
 	}
 
 	if (!rc) {
 		GtkWidget *w;
-		
+
 		gtk_tooltips_set_tip(settings_gui_tooltips(), GTK_WIDGET(tv),
 			_("Move the cursor over a row to see details."), NULL);
 		w = settings_gui_tooltips()->tip_window;
+#if 0
 		if (w)
 			gtk_widget_hide(w);
+#endif
 	} else {
 		gchar text[1024];
 
@@ -520,7 +558,8 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 			"%s %s\n"
 			"%s %s",
 			_("Peer:"),
-			ip_port_to_gchar(rc->results_set->ip, rc->results_set->port),
+			host_addr_port_to_string(rc->results_set->addr,
+				rc->results_set->port),
 			_("Country:"),
 			iso3166_country_name(rc->results_set->country),
 			iso3166_country_cc(rc->results_set->country),
@@ -532,116 +571,49 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 			guid_hex_str(rc->results_set->guid),
 			_("Size:"),
 			short_size(rc->size));
-		
+
 		gtk_tooltips_set_tip(settings_gui_tooltips(), GTK_WIDGET(tv),
 			text, NULL);
 	}
 }
 
-/**
- * Adds some indendation to XML-like text. The input text is assumed to be
- * "flat" and well-formed. If these assumptions fail, the output might look
- * worse than the input.
- *
- * @param s the string to format.
- * @return a newly allocated string.
- */
-static gchar *
-xml_indent(const gchar *s)
-{
-	GString *gs;
-	const gchar *p, *q;
-	guint i, depth = 0;
-
-	gs = g_string_new("");
-
-	q = s;
-	for (;;) {
-
-		q = skip_ascii_spaces(q);
-
-		/* Find the start of the tag */
-		p = strchr(q, '<');
-		if (!p)
-			p = strchr(q, '\0');
-
-		/* Append the text between the previous and the current tag, if any */
-		if (p != q)
-			gs = g_string_append_len(gs, q, p - q);
-		if ('\0' == *p)
-			break;
-		
-		/* Find the end of the tag */
-		q = strchr(p, '>');
-		if (!q)
-			q = strchr(p, '\0');
-
-		if (p[1] != '/') {
-			/* Something like <start> */
-			
-			for (i = 0; i < depth; i++)
-				gs = g_string_append_c(gs, '\t');
-			gs = g_string_append_len(gs, p, (q - p) + 1);
-
-			/* Check for tags like <tag/> */
-			if ('/' != *(q - 1)) {
-				depth++;
-			}
-		} else {
-			/* Something like </end> */
-
-			if (depth > 0) {
-				depth--;
-			}
-			
-			for (i = 0; i < depth; i++)
-				gs = g_string_append_c(gs, '\t');
-			gs = g_string_append_len(gs, p, (q - p) + 1);
-		}
-		gs = g_string_append(gs, "\n");
-
-		if ('>' == *q)
-			q++;
-	}
-	
-	return g_string_free(gs, FALSE);
-}
-
 static void
 search_update_details(GtkTreeView *tv, GtkTreePath *path)
 {
-	GtkTextBuffer *txt;
 	const record_t *rc = NULL;
-	gchar bytes[32];
+	gchar bytes[UINT64_DEC_BUFLEN];
 	gchar *xml_txt;
 
 	g_assert(tv != NULL);
 	g_assert(path != NULL);
 
-	rc = search_get_record_at_path(tv, path);
+	rc = search_gui_get_record_at_path(tv, path);
 	g_return_if_fail(rc != NULL);
-
+	
 	gtk_entry_set_text(
-			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_filename")),
-			lazy_locale_to_utf8(rc->name, 0));
-
+		GTK_ENTRY(lookup_widget(main_window, "entry_result_info_filename")),
+		rc->name
+			? lazy_unknown_to_utf8_normalized(rc->name, UNI_NORM_GUI, NULL)
+			: "");
+	
 	gtk_entry_printf(
-			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_sha1")),
-			"%s%s",
-			rc->sha1 ? "urn:sha1:" : _("<none>"),
-			rc->sha1 ? sha1_base32(rc->sha1) : "");
+		GTK_ENTRY(lookup_widget(main_window, "entry_result_info_sha1")),
+		"%s%s",
+		rc->sha1 ? "urn:sha1:" : _("<no SHA1 known>"),
+		rc->sha1 ? sha1_base32(rc->sha1) : "");
 
 	if (rc->results_set->hostname)
 		gtk_entry_set_text(GTK_ENTRY(
 					lookup_widget(main_window, "entry_result_info_source")),
-				hostname_port_to_gchar(
+				hostname_port_to_string(
 					rc->results_set->hostname, rc->results_set->port));
 	else
 		gtk_entry_set_text(GTK_ENTRY(
 					lookup_widget(main_window, "entry_result_info_source")),
-				ip_port_to_gchar(rc->results_set->ip, rc->results_set->port));
+				host_addr_port_to_string(rc->results_set->addr,
+					rc->results_set->port));
 
-	uint64_to_string_buf(bytes, sizeof bytes, rc->size);
+	uint64_to_string_buf(rc->size, bytes, sizeof bytes);
 	gtk_entry_printf(
 			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_size")),
 			_("%s (%s bytes)"), short_size(rc->size), bytes);
@@ -663,15 +635,55 @@ search_update_details(GtkTreeView *tv, GtkTreePath *path)
 			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_index")),
 			"%lu", (gulong) rc->index);
 
-	gtk_entry_set_text(
-			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_tag")),
-			rc->tag ? lazy_locale_to_utf8(rc->tag, 0) : _("<none>"));
+	{
+		gchar *s;
 
-	txt = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(main_window,
-					"textview_result_info_xml"))); 
-	xml_txt = rc->xml ? xml_indent(rc->xml) : NULL;
-	gtk_text_buffer_set_text(txt, xml_txt ? xml_txt : _("<none>"), -1);
-	G_FREE_NULL(xml_txt);
+		s = rc->tag
+				? unknown_to_utf8_normalized(rc->tag, UNI_NORM_GUI, NULL)
+				: NULL;
+		gtk_entry_set_text(
+			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_tag")),
+			s ? s : "");
+		if (rc->tag != s)
+			G_FREE_NULL(s);
+	}
+
+	/* Display XML data from the result if any */
+	{
+		GtkTextBuffer *txt;
+		
+		txt = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(main_window,
+					"textview_result_info_xml")));
+	
+		/*
+		 * Character set detection usually fails here because XML
+		 * is mostly ASCII so that the thresholds are not reached.
+		 */
+		if (rc->xml) {
+			gchar *s = unknown_to_utf8_normalized(rc->xml, UNI_NORM_GUI, NULL);
+			xml_txt = search_xml_indent(s);
+			if (rc->xml != s)
+				G_FREE_NULL(s);
+		} else {
+			xml_txt = NULL;
+		}
+		gtk_text_buffer_set_text(txt, EMPTY_STRING(xml_txt), -1);
+		G_FREE_NULL(xml_txt);
+	}
+
+#if 0
+	/**
+	 * Dump the raw filename to enhance the character set detection.
+	 * The following helps to figure out what the actual encoding is:
+	 *
+	 *	for cs in $(iconv -l); do
+	 *	  printf "${cs}: ${filename}" | iconv -f "${cs}" -t UTF-8 2>/dev/null
+	 *	done
+	 */
+	g_message("filename[] = \"%s\"",
+		rc->name ? lazy_string_to_printf_escape(rc->name) : "(null)");
+	(void) lazy_string_to_printf_escape("");	/* release buffer if any */
+#endif
 }
 
 
@@ -711,10 +723,10 @@ on_button_search_passive_clicked(GtkButton *unused_button,
      * new_search will trigger a rebuild of the menu as a
      * side effect.
      */
-    default_filter = (filter_t *) option_menu_get_selected_data(
-		lookup_widget(main_window, "optionmenu_search_filter"));
+    default_filter = option_menu_get_selected_data(GTK_OPTION_MENU(
+					lookup_widget(main_window, "optionmenu_search_filter")));
 
-	search_gui_new_search(_("Passive"), SEARCH_PASSIVE, &search);
+	search_gui_new_search(_("Passive"), SEARCH_F_PASSIVE, &search);
 
     /*
      * If we should set a default filter, we do that.
@@ -763,7 +775,8 @@ on_popup_search_drop_name_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_name_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_name_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule, search->filter);
     g_slist_free(sl);
 }
@@ -782,7 +795,8 @@ on_popup_search_drop_sha1_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_sha1_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule, search->filter);
     g_slist_free(sl);
 }
@@ -801,7 +815,8 @@ on_popup_search_drop_host_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_host_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_host_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule, search->filter);
     g_slist_free(sl);
 }
@@ -819,7 +834,8 @@ void on_popup_search_drop_name_global_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_name_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_name_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule,
         filter_get_global_pre());
     g_slist_free(sl);
@@ -839,7 +855,8 @@ on_popup_search_drop_sha1_global_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_sha1_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule,
         filter_get_global_pre());
     g_slist_free(sl);
@@ -859,7 +876,8 @@ on_popup_search_drop_host_global_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_host_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_host_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule,
         filter_get_global_pre());
     g_slist_free(sl);
@@ -879,7 +897,8 @@ on_popup_search_autodownload_name_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_name_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_name_eq);
     g_slist_foreach(sl, (GFunc) filter_add_download_name_rule, search->filter);
     g_slist_free(sl);
 }
@@ -898,7 +917,8 @@ on_popup_search_autodownload_sha1_activate(GtkMenuItem *unused_menuitem,
 	search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
+    sl = tree_selection_collect_data(selection,
+			search_gui_get_record, gui_record_sha1_eq);
     g_slist_foreach(sl, (GFunc) filter_add_download_sha1_rule, search->filter);
     g_slist_free(sl);
 }
@@ -917,9 +937,9 @@ on_popup_search_new_from_selected_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-    sl = tree_selection_collect_data(selection, gui_record_sha1_or_name_eq);
-    g_slist_foreach(
-        sl, (GFunc) search_gui_add_targetted_search, search->filter);
+    sl = tree_selection_collect_data(selection, search_gui_get_record,
+			gui_record_sha1_or_name_eq);
+    g_slist_foreach(sl, search_gui_add_targetted_search, search->filter);
     g_slist_free(sl);
 }
 
@@ -952,9 +972,9 @@ on_popup_search_duplicate_activate(GtkMenuItem *unused_menuitem,
     /* FIXME: should call search_duplicate which has to be written. */
     /* FIXME: should properly duplicate passive searches. */
 
-	search_gui_new_search_full(search->query,
+	search_gui_new_search_full(search->query, tm_time(), search_lifetime,
 		timeout, search->sort_col, search->sort_order,
-		search->enabled ? SEARCH_ENABLED : 0, NULL);
+		search->enabled ? SEARCH_F_ENABLED : 0, NULL);
 }
 
 void
@@ -982,7 +1002,10 @@ on_popup_search_resume_activate(GtkMenuItem *unused_menuitem,
 
     search = search_gui_get_current_search();
     g_return_if_fail(NULL != search);
-	gui_search_set_enabled(search, TRUE);
+	if (!search_gui_is_expired(search)) {
+		gui_search_set_enabled(search, TRUE);
+		search_gui_update_expiry(search);
+	}
 }
 
 void
@@ -997,6 +1020,7 @@ on_popup_search_stop_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_return_if_fail(NULL != search);
 	gui_search_set_enabled(search, FALSE);
+	search_gui_update_expiry(search);
 }
 
 void
@@ -1041,15 +1065,12 @@ on_popup_search_collapse_all_activate (GtkMenuItem *unused_menuitem,
 
 
 /**
- * Queue a bitzi query
+ * Queue a bitzi query.
  */
 void
 on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata)
 {
-	search_t *search;
-	GtkTreeSelection *selection;
-	GSList *sl, *sl_records;
 	guint32 bitzi_debug;
 
 	(void) unused_menuitem;
@@ -1059,42 +1080,46 @@ on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
 	if (bitzi_debug)
 		g_message("on_search_meta_data_active: called");
 
-	/* collect the list of files selected */
-
-	search = search_gui_get_current_search();
-	g_assert(search != NULL);
-
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
-	sl_records = tree_selection_collect_data(selection, gui_record_sha1_eq);
-
-	/* Queue up our requests */
-	if (bitzi_debug)
-		g_message("on_search_meta_data: %d items", g_slist_length(sl_records));
-
-	for (sl = sl_records; sl; sl = g_slist_next(sl)) {
-		record_t *rec;
-
-		rec = sl->data;
-		if (rec->sha1) {
-			GtkTreeIter *parent;
-
-			/* set the feedback */
-			parent = find_parent_with_sha1(search->parents, rec->sha1);
-			g_assert(parent != NULL);
-			gtk_tree_store_set(GTK_TREE_STORE(search->model), parent,
-				c_sr_meta, _("Query queued..."),
-				(-1));
-
-			/* then send the query... */
-	    	guc_query_bitzi_by_urn(rec->sha1);
-
-		}
-    }
-
-	g_slist_free(sl_records);
-
+	search_gui_request_bitzi_data();
 }
 
+static void
+search_gui_browse_selected_helper(gpointer data, gpointer unused_udata)
+{
+	const record_t *rc = data;
+
+	(void) unused_udata;
+	
+	search_gui_new_browse_host(
+		rc->results_set->hostname,
+		rc->results_set->addr,
+		rc->results_set->port,
+		rc->results_set->guid,
+		0 != (rc->results_set->status & ST_FIREWALL),
+		rc->results_set->proxies);
+}
+
+/**
+ * Request host browsing for the selected host.
+ */
+void
+search_gui_browse_selected(void)
+{
+	search_t *search;
+    GSList *sl;
+   
+	search = search_gui_get_current_search();
+	if (!search)
+		return;
+
+    sl = tree_selection_collect_data(
+			gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view)),
+			search_gui_get_record,
+			gui_record_host_eq);
+    g_slist_foreach(sl, search_gui_browse_selected_helper, NULL);
+    g_slist_free(sl);
+
+}
 
 void
 search_callbacks_shutdown(void)
