@@ -1,5 +1,5 @@
 /*
- * $Id: search_common.c,v 1.31 2005/05/04 20:29:21 cbiere Exp $
+ * $Id: search_common.c,v 1.73 2006/01/10 10:23:09 cbiere Exp $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -24,25 +24,33 @@
  */
 
 /**
+ * @ingroup gtk
  * @file
  *
  * Common GUI search routines.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
  */
 
 #include "gui.h"
 
-RCSID("$Id: search_common.c,v 1.31 2005/05/04 20:29:21 cbiere Exp $");
+RCSID("$Id: search_common.c,v 1.73 2006/01/10 10:23:09 cbiere Exp $");
 
 #include "search.h"
 #include "settings.h"
+#include "gtk-missing.h"
 
 #include "search_xml.h"
 #include <libxml/parser.h>
 
+#include "gtk/statusbar.h"
+
 #include "if/gui_property_priv.h"
 #include "if/gnet_property.h"
 #include "if/core/downloads.h"
 #include "if/core/guid.h"
+#include "if/core/sockets.h"
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
@@ -51,24 +59,25 @@ RCSID("$Id: search_common.c,v 1.31 2005/05/04 20:29:21 cbiere Exp $");
 #include "lib/file.h"
 #include "lib/glib-missing.h"
 #include "lib/vendors.h"
+#include "lib/tm.h"
 #include "lib/urn.h"
 #include "lib/utf8.h"
 #include "lib/zalloc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
-static search_t *current_search  = NULL; /* The search currently displayed */
+static search_t *current_search  = NULL; /**< The search currently displayed */
 
-static zone_t *rs_zone;		/* Allocation of results_set */
-static zone_t *rc_zone;		/* Allocation of record */
+static zone_t *rs_zone;		/**< Allocation of results_set */
+static zone_t *rc_zone;		/**< Allocation of record */
 
-static const gchar search_file[] = "searches"; /* "old" file to searches */
-
-static gchar tmpstr[1024];
+static const gchar search_file[] = "searches"; /**< "old" file to searches */
 
 static GSList *accumulated_rs = NULL;
+static GList *list_search_history = NULL;
 
 static GtkLabel *label_items_found = NULL;
+static GtkLabel *label_search_expiry = NULL;
 
 /**
  * Human readable translation of servent trailer open flags.
@@ -79,15 +88,94 @@ static struct {
 	const gchar *status;
 } open_flags[] = {
 	{ ST_BUSY,			N_("busy") },
-	{ ST_UPLOADED,		N_("stable") },		/* Allows uploads -> stable */
+	{ ST_UPLOADED,		N_("stable") },		/**< Allows uploads -> stable */
 	{ ST_FIREWALL,		N_("push") },
 	{ ST_PUSH_PROXY,	N_("proxy") },
-	{ ST_BOGUS,			N_("bogus") },		/* Bogus IP address */
+	{ ST_BOGUS,			N_("bogus") },		/**< Bogus IP address */
 };
 
 search_t *search_gui_get_current_search(void)	{ return current_search; }
 void search_gui_forget_current_search(void)		{ current_search = NULL; }
-void search_gui_current_search(search_t *sch)	{ current_search = sch; }
+
+static void
+on_option_menu_menu_item_activate(GtkMenuItem *unused_item, gpointer udata)
+{
+	GtkOptionMenu *option_menu;
+
+	(void) unused_item;
+
+	option_menu = GTK_OPTION_MENU(udata);
+	search_gui_set_current_search(option_menu_get_selected_data(option_menu));
+}
+
+void
+search_gui_option_menu_searches_update(void)
+{
+	GtkOptionMenu *option_menu;
+	GtkMenu *menu;
+	const GList *iter;
+
+	option_menu = GTK_OPTION_MENU(lookup_widget(main_window,
+						"option_menu_searches"));	
+	menu = GTK_MENU(gtk_menu_new());
+
+	iter = g_list_last(deconstify_gpointer(search_gui_get_searches()));
+	for (/* NOTHING */; iter != NULL; iter = g_list_previous(iter)) {
+		GtkWidget *item;
+		search_t *s = iter->data;
+
+		item = gtk_menu_item_new_with_label(lazy_utf8_to_ui_string(s->query));
+		gtk_widget_show(item);
+		gtk_object_set_user_data(GTK_OBJECT(item), s);
+		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
+#ifdef USE_GTK1
+		gtk_signal_connect(GTK_OBJECT(item), "activate",
+			on_option_menu_menu_item_activate, option_menu);
+#endif /* USE_GTK1 */
+#ifdef USE_GTK2
+		g_signal_connect(GTK_OBJECT(item), "activate",
+			G_CALLBACK(on_option_menu_menu_item_activate), option_menu);
+#endif /* USE_GTK2 */
+
+	}
+	gtk_option_menu_set_menu(option_menu, GTK_WIDGET(menu));
+}
+
+void
+search_gui_option_menu_searches_select(const search_t *sch)
+{
+	option_menu_select_item_by_data(
+		GTK_OPTION_MENU(lookup_widget(main_window, "option_menu_searches")),
+		sch);
+}
+
+void
+search_gui_current_search(search_t *sch)
+{
+	search_gui_option_menu_searches_select(sch);
+   	current_search = sch;
+}
+
+/**
+ * Create a new search and start it. Use default reissue timeout.
+ */
+gboolean
+search_gui_new_search(const gchar *query, flag_t flags, search_t **search)
+{
+    guint32 timeout;
+	gboolean ret;
+
+    gnet_prop_get_guint32_val(PROP_SEARCH_REISSUE_TIMEOUT, &timeout);
+
+	if (!(SEARCH_F_PASSIVE & flags))
+		query = lazy_ui_string_to_utf8(query);
+
+    ret = search_gui_new_search_full(query, tm_time(), search_lifetime, timeout,
+			search_sort_default_column, search_sort_default_order,
+			flags | SEARCH_F_ENABLED, search);
+
+	return ret;
+}
 
 
 /**
@@ -102,25 +190,58 @@ search_gui_free_alt_locs(record_t *rc)
 	g_assert(rc->refcount >= 0 && rc->refcount < INT_MAX);
 	g_assert(alt != NULL);
 
-	wfree(alt->hvec, alt->hvcnt * sizeof(*alt->hvec));
-	wfree(alt, sizeof(*alt));
+	{
+		gint i;
+
+		for (i = 0; i < alt->hvcnt; i++)
+			g_assert(host_addr_initialized(alt->hvec[i].addr));
+	}
+
+	wfree(alt->hvec, alt->hvcnt * sizeof *alt->hvec);
+	wfree(alt, sizeof *alt);
 
 	rc->alt_locs = NULL;
 }
 
+/**
+ * Clone the proxies list given by the core.
+ */
+gnet_host_vec_t *
+search_gui_proxies_clone(gnet_host_vec_t *v)
+{
+	gnet_host_vec_t *new;
+
+	if (v == NULL)
+		return NULL;
+
+	new = walloc(sizeof *new);
+	new->hvec = walloc(v->hvcnt * sizeof(*v->hvec));
+	new->hvcnt = v->hvcnt;
+
+	memcpy(new->hvec, v->hvec, v->hvcnt * sizeof(*v->hvec));
+
+	return new;
+}
+
+/**
+ * Free the cloned vector of host.
+ */
+void
+search_gui_host_vec_free(gnet_host_vec_t *v)
+{
+	g_assert(v != NULL);
+
+	wfree(v->hvec, v->hvcnt * sizeof(*v->hvec));
+	wfree(v, sizeof *v);
+}
+
 /**
  * Free the push proxies held within a result set.
  */
 void
 search_gui_free_proxies(results_set_t *rs)
 {
-	gnet_host_vec_t *v = rs->proxies;
-
-	g_assert(v != NULL);
-
-	wfree(v->hvec, v->hvcnt * sizeof(*v->hvec));
-	wfree(v, sizeof(*v));
-
+	search_gui_host_vec_free(rs->proxies);
 	rs->proxies = NULL;
 }
 
@@ -142,6 +263,7 @@ search_gui_free_record(record_t *rc)
 	g_assert(NULL == rc->results_set);
 
 	atom_str_free(rc->name);
+	atom_str_free(rc->utf8_name);
 	if (rc->ext != NULL)
         atom_str_free(rc->ext);
 	if (rc->tag != NULL)
@@ -222,7 +344,7 @@ search_gui_clean_r_set(results_set_t *rs)
      * Remove empty searches from record set.
      */
 	for (sl = sl_remove; sl != NULL; sl = g_slist_next(sl)) {
-		record_t *rc = (record_t *) sl->data;
+		record_t *rc = sl->data;
 
 		rc->results_set = NULL;
 		search_gui_free_record(rc);
@@ -276,8 +398,8 @@ search_gui_free_r_set(results_set_t *rs)
 
 	g_assert(rs->num_recs == g_slist_length(rs->records));
 	for (sl = rs->records; sl != NULL; sl = g_slist_next(sl)) {
-		record_t *rc = (record_t *) sl->data;
-		
+		record_t *rc = sl->data;
+
 		g_assert(rc->magic == RECORD_MAGIC);
 		g_assert(rc->results_set == rs);
 		rc->results_set = NULL;
@@ -319,7 +441,7 @@ search_gui_dispose_results(results_set_t *rs)
 	 */
 
 	for (l = search_gui_get_searches(); NULL != l; l = g_list_next(l)) {
-		search_t *sch = (search_t *) l->data;
+		search_t *sch = l->data;
 
 		if (NULL != sch->r_sets && hash_list_contains(sch->r_sets, rs)) {
 			refs++;			/* Found one more reference to this search */
@@ -372,7 +494,7 @@ search_gui_unref_record(record_t *rc)
 	g_assert(rs->num_recs > 0);
 	rs->num_recs--;
 
-	g_assert((rs->num_recs > 0) ^ (rs->records == NULL));
+	g_assert((rs->num_recs > 0) ^ (!rs->records));
 
 	/*
 	 * We can't free the results_set structure right now if it does not
@@ -414,7 +536,7 @@ guint
 search_gui_hash_func(gconstpointer p)
 {
 	const record_t *rc = p;
-	
+
 	g_assert(rc->magic == RECORD_MAGIC);
 	g_assert(rc->refcount >= 0 && rc->refcount < INT_MAX);
 
@@ -424,7 +546,7 @@ search_gui_hash_func(gconstpointer p)
 		GPOINTER_TO_UINT(rc->results_set->guid) ^	/* atom! */
 		(NULL != rc->sha1 ? 0 : g_str_hash(rc->name)) ^
 		rc->size ^
-		rc->results_set->ip ^
+		host_addr_hash(rc->results_set->addr) ^
 		rc->results_set->port;
 }
 
@@ -432,10 +554,10 @@ gint
 search_gui_hash_key_compare(gconstpointer a, gconstpointer b)
 {
 	const record_t *rc1 = a, *rc2 = b;
-	
+
 	/* Must compare same fields as search_hash_func() --RAM */
 	return rc1->size == rc2->size
-		&& rc1->results_set->ip == rc2->results_set->ip
+		&& host_addr_equal(rc1->results_set->addr, rc2->results_set->addr)
 		&& rc1->results_set->port == rc2->results_set->port
 		&& rc1->results_set->guid == rc2->results_set->guid	/* atom! */
 		&& (rc1->sha1 != NULL /* atom! */
@@ -458,7 +580,7 @@ search_gui_remove_r_set(search_t *sch, results_set_t *rs)
  * If we do, make sure that the index is still accurate,
  * otherwise inform the interested parties about the change.
  *
- * Returns true if the record is a duplicate.
+ * @returns true if the record is a duplicate.
  */
 gboolean
 search_gui_result_is_dup(search_t *sch, record_t *rc)
@@ -495,7 +617,7 @@ search_gui_result_is_dup(search_t *sch, record_t *rc)
 				old.rc->index, rc->index, guid_hex_str(rc->results_set->guid),
 				rc->name);
 		guc_download_index_changed(
-			rc->results_set->ip,		/* This is for optimizing lookups */
+			rc->results_set->addr,		/* This is for optimizing lookups */
 			rc->results_set->port,
 			rc->results_set->guid,		/* This is for formal identification */
 			old.rc->index,
@@ -507,7 +629,7 @@ search_gui_result_is_dup(search_t *sch, record_t *rc)
 }
 
 /**
- * Returns a pointer to gui_search_t from gui_searches which has
+ * @returns a pointer to gui_search_t from gui_searches which has
  * sh as search_handle. If none is found, return NULL.
  */
 search_t *
@@ -529,38 +651,50 @@ search_gui_find(gnet_search_t sh)
     return NULL;
 }
 
+/**
+ * Extract the filename extensions - if any - from the given UTF-8
+ * encoded filename and convert it to lowercase. If the extension
+ * exceeds a certain length, it is assumed that it's no extension
+ * but just a non-specific dot inside a filename.
+ *
+ * @return NULL if there's no filename extension, otherwise a pointer
+ *         to a static string holding the lowercased extension.
+ */
+gchar *
+search_gui_get_filename_extension(const gchar *filename_utf8)
+{
+	const gchar *p = strrchr(filename_utf8, '.');
+	static gchar ext[32];
+
+	if (!p || utf8_strlower(ext, &p[1], sizeof ext) >= sizeof ext) {
+		/* If the guessed extension is really this long, assume the
+		 * part after the dot isn't an extension at all. */
+		return NULL;
+	}
+	return ext;
+}
+
 /**
  * Create a new GUI record within `rs' from a Gnutella record.
  */
 record_t *
 search_gui_create_record(results_set_t *rs, gnet_record_t *r)
 {
-	const gchar *filename;
     record_t *rc;
 
     g_assert(r != NULL);
     g_assert(rs != NULL);
 
-    rc = (record_t *) zalloc(rc_zone);
+    rc = zalloc(rc_zone);
 
 	rc->magic = RECORD_MAGIC;
     rc->results_set = rs;
     rc->refcount = 0;
 
-    filename = r->name;
-#ifdef USE_GTK2
-    /* Gtk2 extracts this in search_gui2.c because of UTF8 issues. */
-    rc->ext  = NULL;
-#else
-	if (!is_ascii_string(filename)) {
-		if (utf8_is_valid_string(filename, 0))
-    		filename = utf8_to_locale(filename, 0);
-		if (!filename)
-			filename = r->name;
-	}
-    rc->ext  = atom_str_get(search_gui_extract_ext(filename));
-#endif
-	rc->name = atom_str_get(filename);
+    rc->ext = NULL;
+	rc->name = atom_str_get(r->name);
+	rc->utf8_name = atom_str_get(lazy_unknown_to_utf8_normalized(r->name,
+						UNI_NORM_GUI, &rc->charset));
     rc->size = r->size;
     rc->index = r->index;
     rc->sha1 = r->sha1 != NULL ? atom_sha1_get(r->sha1) : NULL;
@@ -570,16 +704,17 @@ search_gui_create_record(results_set_t *rs, gnet_record_t *r)
    	rc->flags = r->flags;
 	rc->alt_locs = NULL;
 
-	if (r->alt_locs != NULL) {
-		gnet_host_vec_t *a = r->alt_locs;				/* Original from core */
-		gnet_host_vec_t *alt = walloc(sizeof(*alt));	/* GUI copy */
-		gint hlen = a->hvcnt * sizeof(*a->hvec);
+	if (NULL != r->alt_locs) {
+		{
+			gint i;
 
-		alt->hvec = walloc(hlen);
-		alt->hvcnt = a->hvcnt;
-		memcpy(a->hvec, alt->hvec, hlen);
+			for (i = 0; i < r->alt_locs->hvcnt; i++)
+				g_assert(host_addr_initialized(r->alt_locs->hvec[i].addr));
+		}
 
-		rc->alt_locs = alt;
+		rc->alt_locs = wcopy(r->alt_locs, sizeof *r->alt_locs);
+		rc->alt_locs->hvec = wcopy(r->alt_locs->hvec,
+								r->alt_locs->hvcnt * sizeof *r->alt_locs->hvec);
 	}
 
     return rc;
@@ -595,35 +730,36 @@ search_gui_create_results_set(GSList *schl, const gnet_results_set_t *r_set)
     GSList *sl;
 	gint ignored = 0;
 
-    rs = (results_set_t *) zalloc(rs_zone);
+    rs = zalloc(rs_zone);
 
     rs->refcount = 0;
     rs->schl = g_slist_copy(schl);
 
     rs->guid = atom_guid_get(r_set->guid);
-    rs->ip = r_set->ip;
+    rs->addr = r_set->addr;
     rs->port = r_set->port;
     rs->status = r_set->status;
     rs->speed = r_set->speed;
     rs->stamp = r_set->stamp;
-    memcpy(rs->vendor, r_set->vendor, sizeof(rs->vendor));
+    rs->vcode = r_set->vcode;
 	rs->version = r_set->version ? atom_str_get(r_set->version) : NULL;
 	rs->hostname = r_set->hostname ? atom_str_get(r_set->hostname) : NULL;
 	rs->country = r_set->country;
-	rs->udp_ip = r_set->udp_ip;
+	rs->udp_addr = r_set->udp_addr;
 
     rs->num_recs = 0;
     rs->records = NULL;
-	rs->proxies = NULL;
+	rs->proxies = search_gui_proxies_clone(r_set->proxies);
 
     for (sl = r_set->records; sl != NULL; sl = g_slist_next(sl)) {
-        record_t *rc;
-		gnet_record_t *grc = (gnet_record_t *) sl->data;
+		gnet_record_t *grc = sl->data;
 
 		if (!(grc->flags & SR_DONT_SHOW)) {
+        	record_t *rc;
+		   
 			rc = search_gui_create_record(rs, grc);
 			rs->records = g_slist_prepend(rs->records, rc);
-			rs->num_recs ++;
+			rs->num_recs++;
 		} else
 			ignored++;
     }
@@ -633,6 +769,21 @@ search_gui_create_results_set(GSList *schl, const gnet_results_set_t *r_set)
     return rs;
 }
 
+void
+on_search_entry_activate(GtkWidget *unused_widget, gpointer unused_udata)
+{
+	(void) unused_widget;
+	(void) unused_udata;
+	search_gui_new_search_entered();
+}
+
+void
+on_option_menu_search_changed(GtkOptionMenu *option_menu, gpointer unused_udata)
+{
+	(void) unused_udata;
+	search_gui_set_current_search(option_menu_get_selected_data(option_menu));
+}
+
 /**
  * Initialize common structures.
  */
@@ -644,6 +795,8 @@ search_gui_common_init(void)
 
 	label_items_found = GTK_LABEL(
 		lookup_widget(main_window, "label_items_found"));
+	label_search_expiry = GTK_LABEL(
+		lookup_widget(main_window, "label_search_expiry"));
 }
 
 /**
@@ -656,6 +809,9 @@ search_gui_common_shutdown(void)
 	zdestroy(rc_zone);
 
 	rs_zone = rc_zone = NULL;
+
+    g_list_free(list_search_history);
+    list_search_history = NULL;
 }
 
 /**
@@ -665,28 +821,46 @@ search_gui_common_shutdown(void)
 void
 search_gui_check_alt_locs(results_set_t *rs, record_t *rc)
 {
-	gint i;
 	gnet_host_vec_t *alt = rc->alt_locs;
+	gint i;
 
 	g_assert(rc->magic == RECORD_MAGIC);
 	g_assert(rc->refcount >= 0 && rc->refcount < INT_MAX);
 	g_assert(alt != NULL);
 	g_assert(rs->proxies == NULL);	/* Since we downloaded record already */
 
-	for (i = alt->hvcnt - 1; i >= 0; i--) {
+	for (i = 0; i < alt->hvcnt; i++) {
 		gnet_host_t *h = &alt->hvec[i];
 
-		if (h->port == 0 || !ip_is_valid(h->ip))
+		g_assert(host_addr_initialized(h->addr));
+		if (h->port == 0 || !host_addr_is_routable(h->addr))
 			continue;
 
 		guc_download_auto_new(rc->name, rc->size, URN_INDEX,
-			h->ip, h->port, blank_guid, rs->hostname,
-			rc->sha1, rs->stamp, FALSE, TRUE, NULL, NULL);
+			h->addr, h->port, blank_guid, rs->hostname,
+			rc->sha1, rs->stamp, FALSE, TRUE, NULL, NULL, 0);
 	}
 
 	search_gui_free_alt_locs(rc);
 }
 
+/**
+ * Makes the sort column and order of the current search the default settings.
+ */
+void
+search_gui_set_sort_defaults(void)
+{
+	const search_t *sch;
+	
+	sch = current_search;
+	if (sch) {
+		gui_prop_set_guint32_val(PROP_SEARCH_SORT_DEFAULT_COLUMN,
+			sch->sort_col);
+		gui_prop_set_guint32_val(PROP_SEARCH_SORT_DEFAULT_ORDER,
+			sch->sort_order);
+	}
+}
+
 /**
  * Persist searches to disk.
  */
@@ -720,48 +894,6 @@ search_gui_store_searches(void)
 	G_FREE_NULL(path);
 }
 
-/**
- * Retrieve search list and restart searches.
- * The searches are normally retrieved from ~/.gtk-gnutella/searches.
- */
-static gboolean
-search_retrieve_old(void)
-{
-	FILE *in;
-	gint line;				/* File line number */
-	file_path_t fp;
-
-	file_path_set(&fp, settings_gui_config_dir(), search_file);
-	in = file_config_open_read("old searches (gtkg pre v0.90)", &fp, 1);
-	if (!in)
-		return FALSE;
-
-	/*
-	 * Retrieval of each searches.
-	 */
-
-	line = 0;
-
-	while (fgets(tmpstr, sizeof(tmpstr) - 1, in)) {	/* Room for trailing NUL */
-		line++;
-
-		if (tmpstr[0] == '#')
-			continue;				/* Skip comments */
-
-		if (tmpstr[0] == '\n')
-			continue;				/* Allow arbitrary blank lines */
-
-		(void) str_chomp(tmpstr, 0);	/* The search string */
-
-		search_gui_new_search(tmpstr, 0, NULL);
-		tmpstr[0] = '\0';
-	}
-
-	fclose(in);
-
-    return TRUE;
-}
-
 /**
  * Retrieve searches from disk.
  */
@@ -770,13 +902,7 @@ search_gui_retrieve_searches(void)
 {
 	LIBXML_TEST_VERSION
 
-    if (!search_retrieve_xml()) {
-		if (search_retrieve_old()) {
-        	g_warning(_("Found old searches file. Loaded it.\n"
-            	"On exit the searches will be saved in the new XML format\n"
-            	"You may remove \"searches.orig\"."));
-    	}
-	}
+    search_retrieve_xml();
 }
 
 /**
@@ -789,7 +915,7 @@ search_matched(search_t *sch, results_set_t *rs)
    	gboolean need_push;			/* Would need a push to get this file? */
 	gboolean skip_records;		/* Shall we skip those records? */
 	GString *vinfo = g_string_sized_new(40);
-	gchar *vendor;
+	const gchar *vendor;
     GdkColor *download_color;
     GdkColor *ignore_color;
     GdkColor *mark_color;
@@ -797,14 +923,16 @@ search_matched(search_t *sch, results_set_t *rs)
     gboolean send_pushes;
     gboolean is_firewalled;
 	guint i;
-	guint32 results_kept = 0;
+	guint32 flags = 0, results_kept = 0;
+	guint32 max_results;
 
     g_assert(sch != NULL);
     g_assert(rs != NULL);
 
 	gui_search_get_colors(sch, &mark_color, &ignore_color, &download_color);
 
-    vendor = lookup_vendor_name(rs->vendor);
+    vendor = lookup_vendor_name(rs->vcode);
+	max_results = sch->browse ? browse_host_max_results : search_max_results;
 
    	if (vendor) {
 		g_string_append(vinfo, vendor);
@@ -832,23 +960,28 @@ search_matched(search_t *sch, results_set_t *rs)
 		sch->udp_qhits++;
 		if (vinfo->len)
 			g_string_append(vinfo, ", ");
-		if (rs->ip == rs->udp_ip)
-			g_string_append(vinfo, "udp");
+		if (host_addr_equal(rs->addr, rs->udp_addr))
+			g_string_append(vinfo, _("udp"));
 		else {
-			g_string_append(vinfo, "udp ");
-			g_string_append(vinfo, ip_to_gchar(rs->udp_ip));
+			g_string_append(vinfo, _("udp "));
+			g_string_append(vinfo, host_addr_to_string(rs->udp_addr));
 		}
 	} else
 		sch->tcp_qhits++;
 
+	if (rs->status & ST_TLS)
+		g_string_append(vinfo, vinfo->len ? ", TLS" : "TLS");
+	if (rs->status & ST_BH)
+		g_string_append(vinfo, vinfo->len ? _(", browsable") : _("browsable"));
+	flags = (rs->status & ST_TLS) ? CONNECT_F_TLS : 0;
+
 	/*
 	 * If we're firewalled, or they don't want to send pushes, then don't
 	 * bother displaying results if they need a push request to succeed.
 	 *		--RAM, 10/03/2002
 	 */
-    gnet_prop_get_boolean
-		(PROP_SEND_PUSHES, &send_pushes, 0, 1);
-    gnet_prop_get_boolean(PROP_IS_FIREWALLED, &is_firewalled, 0, 1);
+    gnet_prop_get_boolean_val(PROP_SEND_PUSHES, &send_pushes);
+    gnet_prop_get_boolean_val(PROP_IS_FIREWALLED, &is_firewalled);
 
 	need_push = (rs->status & ST_FIREWALL) != 0;
 	skip_records = (!send_pushes || is_firewalled) && need_push;
@@ -857,17 +990,18 @@ search_matched(search_t *sch, results_set_t *rs)
 		printf("search_matched: [%s] got hit with %d record%s (from %s) "
 			"need_push=%d, skipping=%d\n",
 			sch->query, rs->num_recs, rs->num_recs == 1 ? "" : "s",
-			ip_port_to_gchar(rs->ip, rs->port), need_push, skip_records);
+			host_addr_port_to_string(rs->addr, rs->port),
+			need_push, skip_records);
 
   	for (l = rs->records; l && !skip_records; l = l->next) {
-		record_t *rc = (record_t *) l->data;
+		record_t *rc = l->data;
         filter_result_t *flt_result;
         gboolean downloaded = FALSE;
 		gboolean is_dup;
 
 		g_assert(rc->magic == RECORD_MAGIC);
 		g_assert(rc->refcount >= 0);
-		
+
         if (gui_debug > 7)
             printf("search_matched: [%s] considering %s (%s)\n",
 				sch->query, rc->name, vinfo->str);
@@ -885,11 +1019,11 @@ search_matched(search_t *sch, results_set_t *rs)
 		 * we detect a change.
 		 */
 
-		
+
 		g_assert(rc->refcount >= 0);
 		is_dup = search_gui_result_is_dup(sch, rc);
 		g_assert(rc->refcount >= 0);
-		
+
 		if (is_dup) {
 			sch->duplicates++;
 			continue;
@@ -927,8 +1061,8 @@ search_matched(search_t *sch, results_set_t *rs)
 				FILTER_PROP_STATE_DO)
 		) {
             guc_download_auto_new(rc->name, rc->size, rc->index,
-				rs->ip, rs->port, rs->guid, rs->hostname, rc->sha1, rs->stamp,
-				need_push, TRUE, NULL, rs->proxies);
+				rs->addr, rs->port, rs->guid, rs->hostname, rc->sha1,
+				rs->stamp, need_push, TRUE, NULL, rs->proxies, flags);
 
 			if (rs->proxies != NULL)
 				search_gui_free_proxies(rs);
@@ -955,7 +1089,7 @@ search_matched(search_t *sch, results_set_t *rs)
                 FILTER_PROP_STATE_DONT &&
             flt_result->props[FILTER_PROP_DISPLAY].user_data == 0) &&
 			(int) results_kept++ >= 0 && /* Count as kept even if max results */
-            sch->items < search_max_results
+            sch->items < max_results
 		) {
             GdkColor *fg_color = NULL;
             gboolean mark;
@@ -1024,7 +1158,7 @@ search_matched(search_t *sch, results_set_t *rs)
 	 * to make some room to allow the search to continue.
 	 */
 
-	if (sch->items >= search_max_results && !sch->passive)
+	if (sch->items >= max_results && !sch->passive)
 		gui_search_set_enabled(sch, FALSE);
 
 	/*
@@ -1037,7 +1171,7 @@ search_matched(search_t *sch, results_set_t *rs)
 	else
 		sch->unseen_items += sch->items - old_items;
 
-	if (time(NULL) - sch->last_update_time < TAB_UPDATE_TIME)
+	if (delta_time(tm_time(), sch->last_update_time) > TAB_UPDATE_TIME)
 		gui_search_update_tab_label(sch);
 
   	g_string_free(vinfo, TRUE);
@@ -1047,24 +1181,74 @@ search_matched(search_t *sch, results_set_t *rs)
  * Update the label string showing search stats.
  */
 void
-search_gui_update_items(struct search *sch)
+search_gui_update_items(const struct search *sch)
 {
     if (sch) {
-        const gchar *str = sch->passive ? _("(passive search) ") : "";
-
-		gm_snprintf(tmpstr, sizeof(tmpstr), _("%s%s%u %s "
+		gtk_label_printf(label_items_found, _("%u %s "
 			"(%u skipped, %u ignored, %u hidden, %u auto-d/l, %u %s)"
 			" Hits: %u (%u TCP, %u UDP)"),
-			sch->enabled ? "" : _("[stopped] "),
-			str,
 			sch->items, NG_("item", "items", sch->items),
 			sch->skipped, sch->ignored, sch->hidden, sch->auto_downloaded,
 			sch->duplicates, NG_("dupe", "dupes", sch->duplicates),
 			sch->tcp_qhits + sch->udp_qhits, sch->tcp_qhits, sch->udp_qhits);
-    } else
-        g_strlcpy(tmpstr, _("No search"), sizeof(tmpstr));
+    } else {
+       gtk_label_printf(label_items_found, "%s", _("No search"));
+	}
+}
 
-	gtk_label_set(label_items_found, tmpstr);
+gboolean
+search_gui_is_expired(const struct search *sch)
+{
+	gboolean expired = FALSE;
+	
+	if (sch && !sch->passive)
+		expired = guc_search_is_expired(sch->search_handle);
+
+	return expired;
+}
+
+gboolean
+search_gui_update_expiry(const struct search *sch)
+{
+	gboolean expired = FALSE;
+
+    if (sch) {
+		if (sch->passive) {
+   			gtk_label_printf(label_search_expiry, "%s", _("Passive search"));
+		} else if (sch->enabled) {
+			expired = search_gui_is_expired(sch);
+			
+			if (expired) {
+        		gtk_label_printf(label_search_expiry, "%s", _("Expired"));
+			} else {
+				guint lt;
+
+				lt = 3600 * guc_search_get_lifetime(sch->search_handle);
+				if (lt) {
+					time_t ct, start;
+					gint d;
+					
+					gnet_prop_get_timestamp_val(PROP_START_STAMP, &start);
+					ct = guc_search_get_create_time(sch->search_handle);
+					
+					d = delta_time(tm_time(), ct);
+					d = MAX(0, d);
+					d = (guint) d < lt ? lt - d : 0;
+					gtk_label_printf(label_search_expiry,
+						_("Expires in %s"), short_time(d));
+				} else {
+        			gtk_label_printf(label_search_expiry, "%s",
+						_("Expires with this session"));
+				}
+			}
+		} else {
+        	gtk_label_printf(label_search_expiry, "%s", _("[stopped]"));
+		}
+    } else {
+        gtk_label_printf(label_search_expiry, "%s", _("No search"));
+	}
+
+	return expired;
 }
 
 /***
@@ -1107,7 +1291,7 @@ search_gui_flush(time_t now)
     GSList *frozen = NULL;
 
 	gui_prop_get_guint32_val(PROP_SEARCH_ACCUMULATION_PERIOD, &period);
-    if (now && (difftime(now, last) < period))
+    if (last && difftime(now, last) < period)
         return;
 
     last = now;
@@ -1126,7 +1310,7 @@ search_gui_flush(time_t now)
     }
 
     for (curs = accumulated_rs; curs != NULL; curs = g_slist_next(curs)) {
-        results_set_t *rs = (results_set_t *) curs->data;
+        results_set_t *rs = curs->data;
         GSList *schl = g_slist_copy(rs->schl);
 
         /*
@@ -1221,25 +1405,26 @@ search_gui_flush(time_t now)
  * @author Andrew Meredith <andrew@anvil.org>
  */
 void
-search_gui_add_targetted_search(record_t *rec, filter_t *unused_filter)
+search_gui_add_targetted_search(gpointer data, gpointer unused_udata)
 {
-    search_t *new_search;
+	const record_t *rec = data;
+    search_t *new_search = NULL;
     rule_t *rule;
 
-	(void) unused_filter;
+	(void) unused_udata;
     g_assert(rec != NULL);
-    g_assert(rec->name != NULL);
+    g_assert(rec->utf8_name != NULL);
 
     /* create new search item with search string set to filename */
-    search_gui_new_search(rec->name, 0, &new_search);
+    search_gui_new_search(rec->utf8_name, 0, &new_search);
     g_assert(new_search != NULL);
 
     if (rec->sha1) {
-        rule = filter_new_sha1_rule(rec->sha1, rec->name,
-            filter_get_download_target(), RULE_FLAG_ACTIVE);
+        rule = filter_new_sha1_rule(rec->sha1, rec->utf8_name,
+            		filter_get_download_target(), RULE_FLAG_ACTIVE);
     } else {
-        rule = filter_new_text_rule(rec->name, RULE_TEXT_EXACT, TRUE,
-            filter_get_download_target(), RULE_FLAG_ACTIVE);
+        rule = filter_new_text_rule(rec->utf8_name, RULE_TEXT_EXACT, TRUE,
+            		filter_get_download_target(), RULE_FLAG_ACTIVE);
     }
     g_assert(rule != NULL);
 
@@ -1260,8 +1445,10 @@ search_gui_restart_search(search_t *sch)
 	sch->skipped = sch->ignored = sch->auto_downloaded = sch->duplicates = 0;
 
 	search_gui_update_items(sch);
+	guc_search_set_create_time(sch->search_handle, tm_time());
 	guc_search_update_items(sch->search_handle, sch->items);
 	guc_search_reissue(sch->search_handle);
+	search_gui_update_expiry(sch);
 }
 
 struct query {
@@ -1284,7 +1471,7 @@ search_gui_parse_text_query(const gchar *s, struct query *query)
 
 	*query = zero_query;
 	dst = query->text = g_strdup(s);
-	
+
 	for (p = s; *p != '\0'; p = q) {
 		gboolean neg;
 		gint n;
@@ -1301,7 +1488,7 @@ search_gui_parse_text_query(const gchar *s, struct query *query)
 		n = q - p;
 		g_assert(n >= 0);
 		if (neg) {
-			if (*p != '\0') {
+			if (*p != '\0' && n > 0) {
 				gchar *w;
 
 				w = g_strndup(p, n + 1);
@@ -1319,7 +1506,7 @@ search_gui_parse_text_query(const gchar *s, struct query *query)
 				g_message("pos: \"%s\"", dst);
 			dst += n;
 		}
-	
+
 		if (*q != '\0')
 			q = skip_ascii_blanks(++q);
 	}
@@ -1331,7 +1518,7 @@ search_gui_parse_text_query(const gchar *s, struct query *query)
  * Parses a query string as entered by the user.
  *
  * @param	querystr must point to the query string.
- * @param	rule
+ * @param	**rules
  * @param	*error will point to an descriptive error message on failure.
  * @return	NULL if the query was not valid. Otherwise, the decoded query
  *			is returned.
@@ -1342,7 +1529,6 @@ search_gui_parse_query(const gchar *querystr, GList **rules,
 {
 	static const gchar urnsha1[] = "urn:sha1:";
 	static gchar query[512];
-	size_t len;
 	struct query qs;
 
 	g_assert(querystr != NULL);
@@ -1352,6 +1538,11 @@ search_gui_parse_query(const gchar *querystr, GList **rules,
 	if (rules)
 		*rules = NULL;
 
+	if (!utf8_is_valid_string(querystr)) {
+		*error = _("The query string is not UTF-8 encoded");
+		return NULL;
+	}
+	
 	/*
 	 * If the text is a magnet link we extract the SHA1 urn
 	 * and put it back into the search field string so that the
@@ -1359,8 +1550,7 @@ search_gui_parse_query(const gchar *querystr, GList **rules,
 	 *		--DBelius   11/11/2002
 	 */
 
-	len = g_strlcpy(query, querystr, sizeof query);
-	if (len >= sizeof query) {
+	if (utf8_strlcpy(query, querystr, sizeof query) >= sizeof query) {
 		*error = _("The entered query is too long");
 		return NULL;
 	}
@@ -1432,9 +1622,9 @@ search_gui_parse_query(const gchar *querystr, GList **rules,
 
 		target = filter_get_drop_target();
 		g_assert(target != NULL);
-		for (l = qs.neg; l != NULL; l = g_list_next(l)) {		
+		for (l = qs.neg; l != NULL; l = g_list_next(l)) {
 			gchar *w;
-		
+
 			w = l->data;
 			g_assert(w != NULL);
 			if (rules) {
@@ -1452,7 +1642,7 @@ search_gui_parse_query(const gchar *querystr, GList **rules,
 		g_list_free(qs.neg);
 
 	G_FREE_NULL(qs.text);
-	
+
 	return query;
 }
 
@@ -1482,4 +1672,271 @@ search_gui_filter_new(search_t *sch, GList *rules)
 	}
 }
 
+/**
+ * Adds some indendation to XML-like text. The input text is assumed to be
+ * "flat" and well-formed. If these assumptions fail, the output might look
+ * worse than the input.
+ *
+ * @param s the string to format.
+ * @return a newly allocated string.
+ */
+gchar *
+search_xml_indent(const gchar *s)
+{
+	GString *gs;
+	const gchar *p, *q;
+	guint i, depth = 0;
+	gchar *res;
+
+	gs = g_string_new("");
+
+	q = s;
+	for (;;) {
+
+		q = skip_ascii_spaces(q);
+
+		/* Find the start of the tag */
+		p = strchr(q, '<');
+		if (!p)
+			p = strchr(q, '\0');
+
+		/* Append the text between the previous and the current tag, if any */
+		if (p != q)
+			gs = g_string_append_len(gs, q, p - q);
+		if ('\0' == *p)
+			break;
+
+		/* Find the end of the tag */
+		q = strchr(p, '>');
+		if (!q)
+			q = strchr(p, '\0');
+
+		if (p[1] != '/') {
+			/* Something like <start> */
+
+			for (i = 0; i < depth; i++)
+				gs = g_string_append_c(gs, '\t');
+			gs = g_string_append_len(gs, p, (q - p) + 1);
+
+			/* Check for tags like <tag/> */
+			if ('/' != *(q - 1)) {
+				depth++;
+			}
+		} else {
+			/* Something like </end> */
+
+			if (depth > 0) {
+				depth--;
+			}
+
+			for (i = 0; i < depth; i++)
+				gs = g_string_append_c(gs, '\t');
+			gs = g_string_append_len(gs, p, (q - p) + 1);
+		}
+		gs = g_string_append(gs, "\n");
+
+		if ('>' == *q)
+			q++;
+	}
+
+	res = gs->str;
+	g_string_free(gs, FALSE);		/* glib1.x returns nothing here */
+
+	return res;
+}
+
+/**
+ * Adds a search string to the search history combo. Makes
+ * sure we do not get more than 10 entries in the history.
+ * Also makes sure we don't get duplicate history entries.
+ * If a string is already in history and it's added again,
+ * it's moved to the beginning of the history list.
+ */
+static void
+search_gui_history_add(const gchar *s)
+{
+    GList *new_hist = NULL, *cur_hist = list_search_history;
+    guint n = 0;
+
+    g_return_if_fail(s);
+
+    while (cur_hist != NULL) {
+        if (n < 9 && 0 != g_ascii_strcasecmp(s, cur_hist->data)) {
+            /* copy up to the first 9 items */
+            new_hist = g_list_prepend(new_hist, cur_hist->data);
+            n++;
+        } else {
+            /* and free the rest */
+            G_FREE_NULL(cur_hist->data);
+        }
+        cur_hist = g_list_next(cur_hist);
+    }
+	new_hist = g_list_reverse(new_hist);
+
+    /* put the new item on top */
+    new_hist = g_list_prepend(new_hist, g_strdup(s));
+
+    /* set new history */
+    gtk_combo_set_popdown_strings(
+        GTK_COMBO(lookup_widget(main_window, "combo_search")),
+        new_hist);
+
+    /* free old list structure */
+    g_list_free(list_search_history);
+
+    list_search_history = new_hist;
+}
+
+/**
+ * Create a new search from a query entered by the user.
+ */
+void
+search_gui_new_search_entered(void)
+{
+	GtkWidget *widget;
+	const gchar *ep;
+	gchar *text;
+	
+    widget = lookup_widget(main_window, "entry_search");
+   	text = STRTRACK(gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1));
+    g_strstrip(text);
+	
+	if (NULL != (ep = is_strprefix(text, "browse:"))) {
+		host_addr_t addr;
+		const gchar *s;
+
+		s = ep;
+		if (string_to_host_or_addr(s, &ep, &addr)) {
+			if (':' == *ep) {
+				guint16 port;
+				gint error;
+
+				/* Erase the colon and skip over it */
+				text[ep - text] = '\0';
+				ep++;
+
+				port = parse_uint16(ep, NULL, 10, &error);
+				if (!error) {
+					search_gui_new_browse_host(is_host_addr(addr) ? NULL : s,
+						addr, port, NULL, FALSE, NULL);
+				}
+			}
+		}
+	} else if ('\0' != text[0]) {
+        filter_t *default_filter;
+        search_t *search;
+        gboolean res;
+
+        /*
+         * It's important gui_search_history_add is called before
+         * new_search, otherwise the search entry will not be
+         * cleared.
+         *      --BLUE, 04/05/2002
+         */
+        search_gui_history_add(text);
+
+        /*
+         * We have to capture the selection here already, because
+         * new_search will trigger a rebuild of the menu as a
+         * side effect.
+         */
+        default_filter = option_menu_get_selected_data(GTK_OPTION_MENU(
+					lookup_widget(main_window, "optionmenu_search_filter")));
+
+		res = search_gui_new_search(text, 0, &search);
+
+        /*
+         * If we should set a default filter, we do that.
+         */
+        if (res && (default_filter != NULL)) {
+            rule_t *rule;
+		   
+			rule = filter_new_jump_rule(default_filter, RULE_FLAG_ACTIVE);
+
+            /*
+             * Since we don't want to distrub the shadows and
+             * do a "force commit" without the user having pressed
+             * the "ok" button in the dialog, we add the rule
+             * manually.
+             */
+            search->filter->ruleset =
+					g_list_append(search->filter->ruleset, rule);
+            rule->target->refcount++;
+        }
+
+        if (!res)
+        	gdk_beep();
+    }
+
+	gtk_widget_grab_focus(widget);
+	G_FREE_NULL(text);
+}
+
+/**
+ * Create a new "browse host" type search.
+ *
+ * @param hostname	the DNS name of the host, or NULL if none known
+ * @param addr		the IP address of the host to browse
+ * @param port		the port to contact
+ * @param guid		the GUID of the remote host
+ * @param push		whether a PUSH request is neeed to reach remote host
+ * @param proxies	vector holding known push-proxies
+ *
+ * @return whether the browse host request could be launched.
+ */
+gboolean
+search_gui_new_browse_host(
+	const gchar *hostname, host_addr_t addr, guint16 port,
+	const gchar *guid, gboolean push, const gnet_host_vec_t *proxies)
+{
+	const gchar *hostport;
+	search_t *search;
+
+	/*
+	 * Browse Host (client-side) works thusly:
+	 *
+	 * We are going to issue a download to request "/" on the remote host.
+	 * Once the HTTP connection is established, the remote servent will
+	 * send back possibly compressed query hits listing all the shared files.
+	 * Those hits will be displayed in the search results.
+	 *
+	 * The core side is responsible for managing the relationship between
+	 * the HTTP packets and the search.  From a GUI standpoint, all we
+	 * want to do is display the results.
+	 *
+	 * However, the "browse host" search is NOT persisted among the searches,
+	 * so its lifetime is implicitely this session only.
+	 */
+
+	hostport = hostname ?
+		hostname_port_to_string(hostname, port) :
+		host_addr_port_to_string(addr, port);
+
+	if (
+		!search_gui_new_search_full(hostport, tm_time(), 0, 0,
+			 search_sort_default_column, search_sort_default_order,
+			 SEARCH_F_BROWSE | SEARCH_F_ENABLED, &search)
+	)
+		goto failed;
+
+	if (
+		!guc_search_browse(search->search_handle, hostname, addr, port,
+			guid, push, proxies)
+	) {
+		search_gui_close_search(search);
+		goto failed;
+	}
+
+	statusbar_gui_message(15,
+		_("Added search showing browsing results for %s"), hostport);
+
+	return TRUE;
+
+failed:
+	statusbar_gui_message(10,
+		_("Could not launch browse host for %s"), hostport);
+
+	return FALSE;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
