@@ -1,10 +1,8 @@
 /*
- * $Id: search_xml.c,v 1.22 2005/05/04 20:30:45 cbiere Exp $
+ * $Id: search_xml.c,v 1.39 2005/11/27 04:37:45 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
- * Persistance for searches and filters in XML format.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Persistance for searches and filters in XML format.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "gui.h"
 
-RCSID("$Id: search_xml.c,v 1.22 2005/05/04 20:30:45 cbiere Exp $");
+RCSID("$Id: search_xml.c,v 1.39 2005/11/27 04:37:45 cbiere Exp $");
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -41,7 +49,9 @@ RCSID("$Id: search_xml.c,v 1.22 2005/05/04 20:30:45 cbiere Exp $");
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/getdate.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -117,6 +127,8 @@ static const gchar TAG_SEARCH_ENABLED[]         = "Enabled";
 static const gchar TAG_SEARCH_SPEED[]           = "Speed";
 static const gchar TAG_SEARCH_PASSIVE[]         = "Passive";
 static const gchar TAG_SEARCH_REISSUE_TIMEOUT[] = "ReissueTimeout";
+static const gchar TAG_SEARCH_CREATE_TIME[] 	= "CreateTime";
+static const gchar TAG_SEARCH_LIFETIME[] 		= "LifeTime";
 static const gchar TAG_SEARCH_SORT_COL[]        = "SortCol";
 static const gchar TAG_SEARCH_SORT_ORDER[]      = "SortOrder";
 static const gchar TAG_RULE_TEXT_CASE[]         = "Case";
@@ -159,7 +171,7 @@ static node_parser_t parser_map[] = {
     { NULL, NULL }
 };
 
-/* Get rid of the obnoxious (xmlChar *) */
+/** Get rid of the obnoxious (xmlChar *) */
 static inline gchar *
 xml_get_string(xmlNode *node, const gchar *id)
 {
@@ -167,7 +179,7 @@ xml_get_string(xmlNode *node, const gchar *id)
 }
 
 static inline const xmlChar *
-gchar_to_xmlChar(const gchar *p)
+string_to_xmlChar(const gchar *p)
 {
 	return (const xmlChar *) p;
 }
@@ -175,7 +187,7 @@ gchar_to_xmlChar(const gchar *p)
 static inline xmlNodePtr
 xml_new_empty_child(xmlNodePtr parent, const gchar *name)
 {
-	return xmlNewChild(parent, NULL, gchar_to_xmlChar(name), NULL);
+	return xmlNewChild(parent, NULL, string_to_xmlChar(name), NULL);
 }
 
 
@@ -205,7 +217,7 @@ parse_number(const gchar *buf, gint *error)
 		g_message("buf=\"%s\"", buf);
 		return 0;
 	}
-	
+
 	return ret;
 }
 
@@ -230,7 +242,7 @@ parse_target(const gchar *buf, gint *error)
 	g_assert(error != NULL);
 
 	if ('0' == buf[0] && 'x' == buf[1]) {
-		/* 
+		/*
 		 * In previous versions, targets were printed using "%p". This format
 		 * is implementation-specific and thus causes a non-portable
 		 * configuration. We expect a hexadecimal value that is optionally
@@ -283,13 +295,13 @@ target_new_id(gboolean do_reset)
 		/* 4 billion filters/searches should be enough for everyone */
 		g_assert(0 != id_counter);
 	}
-	
+
 	ret = id_counter;
 	return GUINT_TO_POINTER(ret);
 }
 
 /**
- * Resets the target ID counter and frees target_map if it was created. 
+ * Resets the target ID counter and frees target_map if it was created.
  */
 static void
 target_map_reset(void)
@@ -322,9 +334,9 @@ target_to_string(filter_t *target)
 		value = target_new_id(FALSE);
 		g_hash_table_insert(target_map, target, value);
 	}
-	
+
     gm_snprintf(buf, sizeof buf, "0x%x", GPOINTER_TO_UINT(value));
-	
+
 	return buf;
 }
 
@@ -339,9 +351,9 @@ target_to_string(filter_t *target)
 static inline xmlAttrPtr
 xml_prop_set(xmlNodePtr node, const gchar *name, const char *value)
 {
-    return xmlSetProp(node, gchar_to_xmlChar(name), gchar_to_xmlChar(value));
+    return xmlSetProp(node, string_to_xmlChar(name), string_to_xmlChar(value));
 }
-	
+
 /**
  * A wrapper to set use xmlSetProp() through a printf-like interface. The
  * length of the created string is limited to 4096 byte and truncation occurs
@@ -372,24 +384,24 @@ void
 search_store_xml(void)
 {
 	const GList *l;
-	time_t now = time((time_t *) NULL);
+	time_t now = tm_time();
     xmlDocPtr doc;
     xmlNodePtr root;
 	gchar *filename_new;
 
 
 	/* Free target_map and reset the target ID counter */
-	target_map_reset();	
+	target_map_reset();
 
     /*
      * Create new xml document with version 1.0
      */
-    doc = xmlNewDoc(gchar_to_xmlChar("1.0"));
+    doc = xmlNewDoc(string_to_xmlChar("1.0"));
 
     /*
      * Create a new root node "gtkGnutella searches"
      */
-    root = xmlNewDocNode(doc, NULL, gchar_to_xmlChar("Searches"), NULL);
+    root = xmlNewDocNode(doc, NULL, string_to_xmlChar("Searches"), NULL);
     xmlDocSetRootElement(doc, root);
 	/* Discard the newline of the ctime string */
     xml_prop_printf(root, "Time", "%24.24s", ctime(&now));
@@ -404,14 +416,14 @@ search_store_xml(void)
      * Iterate over the searches and add them to the tree
      */
     for (l = search_gui_get_searches(); l; l = g_list_next(l))
-        search_to_xml(root, (search_t *) l->data);
+        search_to_xml(root, l->data);
 
     /*
      * Iterate over the rulesets and add them to the tree.
      * Only those that are not bound to a search.
      */
     for (l = filters; l; l = g_list_next(l))
-        filter_to_xml(root, (filter_t *) l->data);
+        filter_to_xml(root, l->data);
 
     /*
      * Try to save the file
@@ -451,7 +463,7 @@ search_store_xml(void)
 	xmlFreeDoc(doc);
 
 	/* Free target_map and reset the target ID counter */
-	target_map_reset();	
+	target_map_reset();
 }
 
 /**
@@ -557,7 +569,7 @@ search_retrieve_xml(void)
 
     for (f = filters; f != NULL; f = f_next) {
 		gboolean damaged = FALSE;
-        filter_t *filter = (filter_t *) f->data;
+        filter_t *filter = f->data;
         GList *r;
         gint n = 0;
 
@@ -571,7 +583,7 @@ search_retrieve_xml(void)
         if (!filter_is_builtin(filter)) {
 
             for (r = filter->ruleset; r != NULL; r = g_list_next(r)) {
-                rule_t *rule = (rule_t *) r->data;
+                rule_t *rule = r->data;
                 gpointer new_target;
 
                 g_assert(rule->target != NULL);
@@ -580,7 +592,7 @@ search_retrieve_xml(void)
                     g_warning("Failed to resolve rule %d in \"%s\": "
 						"missing key %p",
                         n, filter->name,
-						cast_to_gconstpointer(filter_rule_to_gchar(rule)));
+						cast_to_gconstpointer(filter_rule_to_string(rule)));
 
 					/* Remove the corrupted filter, we can't handle it */
 					damaged = TRUE;
@@ -627,7 +639,7 @@ search_retrieve_xml(void)
             g_message("verifying bindings...");
 
         for (s = search_gui_get_searches(); s != NULL; s = g_list_next(s)) {
-            search_t *search = (search_t *) s->data;
+            search_t *search = s->data;
 
             if (search->filter->search == search) {
                 if (gui_debug >= 6)
@@ -687,13 +699,15 @@ static void
 search_to_xml(xmlNodePtr parent, search_t *s)
 {
     xmlNodePtr newxml;
-    GList *l;
-	gchar *query;
+    GList *iter;
 
     g_assert(s != NULL);
     g_assert(s->query != NULL);
     g_assert(parent != NULL);
 
+	if (s->browse)
+		return;			/* Don't persist "browse host" searches. */
+
     if (gui_debug >= 6) {
         g_message(
 			"saving search: %s (%p enabled=%d)\n"
@@ -704,26 +718,22 @@ search_to_xml(xmlNodePtr parent, search_t *s)
 			cast_to_gconstpointer(s));
     }
 
-	if (NULL == (query = locale_to_utf8_full(s->query))) {
-		g_warning("search_to_xml: Cannot convert search string to UTF-8. "
-			"Search won't be saved. (query=\"%s\")", s->query);
-		return;
-	}
-
     newxml = xml_new_empty_child(parent, NODE_SEARCH);
-    xml_prop_set(newxml, TAG_SEARCH_QUERY, query);
-	if (query != s->query)
-		G_FREE_NULL(query);
+    xml_prop_set(newxml, TAG_SEARCH_QUERY, s->query);
 
 	xml_prop_printf(newxml, TAG_SEARCH_ENABLED, "%u", s->enabled);
     xml_prop_printf(newxml, TAG_SEARCH_PASSIVE, "%u", TO_BOOL(s->passive));
-    xml_prop_printf(newxml, TAG_SEARCH_REISSUE_TIMEOUT,
-		"%u", guc_search_get_reissue_timeout(s->search_handle));
-    xml_prop_printf(newxml, TAG_SEARCH_SORT_COL, "%i", s->sort_col);
-    xml_prop_printf(newxml, TAG_SEARCH_SORT_ORDER, "%i", s->sort_order);
+    xml_prop_printf(newxml, TAG_SEARCH_REISSUE_TIMEOUT, "%u",
+		guc_search_get_reissue_timeout(s->search_handle));
+    xml_prop_printf(newxml, TAG_SEARCH_CREATE_TIME, "%s",
+		timestamp_to_string(guc_search_get_create_time(s->search_handle)));
+    xml_prop_printf(newxml, TAG_SEARCH_LIFETIME, "%u",
+		guc_search_get_lifetime(s->search_handle));
+    xml_prop_printf(newxml, TAG_SEARCH_SORT_COL, "%d", s->sort_col);
+    xml_prop_printf(newxml, TAG_SEARCH_SORT_ORDER, "%d", s->sort_order);
 
-    for (l = s->filter->ruleset; l != NULL; l = g_list_next(l))
-        rule_to_xml(newxml, (rule_t *) l->data);
+    for (iter = s->filter->ruleset; iter != NULL; iter = g_list_next(iter))
+        rule_to_xml(newxml, iter->data);
 }
 
 
@@ -732,7 +742,6 @@ filter_to_xml(xmlNodePtr parent, filter_t *f)
 {
     xmlNodePtr newxml;
     GList *l;
-	gchar *name;
 
     g_assert(f != NULL);
     g_assert(f->name != NULL);
@@ -755,18 +764,9 @@ filter_to_xml(xmlNodePtr parent, filter_t *f)
 			cast_to_gconstpointer(f->search));
     }
 
-	if (NULL == (name = locale_to_utf8_full(f->name))) {
-		g_warning("filter_to_xml: Cannot convert search string to UTF-8. "
-			"Filter won't be saved. (name=\"%s\")", f->name);
-		return;
-	}
-
+	g_assert(utf8_is_valid_string(f->name));
     newxml = xml_new_empty_child(parent, NODE_FILTER);
-    xml_prop_set(newxml, TAG_FILTER_NAME, name);
-	if (name != f->name)
-		G_FREE_NULL(name);
-
-	
+    xml_prop_set(newxml, TAG_FILTER_NAME, f->name);
     xml_prop_printf(newxml, TAG_FILTER_ACTIVE,
 		"%u", TO_BOOL(filter_is_active(f)));
 
@@ -792,7 +792,7 @@ filter_to_xml(xmlNodePtr parent, filter_t *f)
      * application.
      */
     for (l = f->ruleset; l != NULL; l = g_list_next(l))
-        rule_to_xml(newxml, (rule_t *) l->data);
+        rule_to_xml(newxml, l->data);
 }
 
 static void
@@ -810,39 +810,28 @@ rule_to_xml(xmlNodePtr parent, rule_t *r)
 
     switch (r->type) {
     case RULE_TEXT:
-		{
-			gchar *match;
-			
-           	if (NULL == (match = locale_to_utf8_full(r->u.text.match))) {
-				g_warning("rule_to_xml: Cannot convert string to UTF-8. "
-					"Omitting rule (\"%s\")", r->u.text.match);
-				return;
-			}
-
-			newxml = xml_new_empty_child(parent, NODE_RULE_TEXT);
-        	xml_prop_set(newxml, TAG_RULE_TEXT_CASE,
-				r->u.text.case_sensitive ? "1" : "0");
-        	xml_prop_set(newxml, TAG_RULE_TEXT_MATCH, match);
-			if (match != r->u.text.match)
-				G_FREE_NULL(match);
-
-        	xml_prop_printf(newxml, TAG_RULE_TEXT_TYPE, "%u", r->u.text.type);
-		}
+		g_assert(utf8_is_valid_string(r->u.text.match));
+		newxml = xml_new_empty_child(parent, NODE_RULE_TEXT);
+       	xml_prop_set(newxml, TAG_RULE_TEXT_CASE,
+			r->u.text.case_sensitive ? "1" : "0");
+       	xml_prop_set(newxml, TAG_RULE_TEXT_MATCH, r->u.text.match);
+       	xml_prop_printf(newxml, TAG_RULE_TEXT_TYPE, "%u", r->u.text.type);
         break;
     case RULE_IP:
         newxml = xml_new_empty_child(parent, NODE_RULE_IP);
-        xml_prop_set(newxml, TAG_RULE_IP_ADDR, ip_to_gchar(r->u.ip.addr));
-        xml_prop_set(newxml, TAG_RULE_IP_MASK, ip_to_gchar(r->u.ip.mask));
+        xml_prop_set(newxml, TAG_RULE_IP_ADDR,
+			host_addr_to_string(r->u.ip.addr));
+        xml_prop_printf(newxml, TAG_RULE_IP_MASK, "%u", r->u.ip.mask);
         break;
     case RULE_SIZE:
 		{
-			gchar buf[32];
-			
+			gchar buf[UINT64_DEC_BUFLEN];
+
+			uint64_to_string_buf(r->u.size.lower, buf, sizeof buf);
         	newxml = xml_new_empty_child(parent, NODE_RULE_SIZE);
-        	xml_prop_printf(newxml, TAG_RULE_SIZE_LOWER,
-				"%s", uint64_to_string_buf(buf, sizeof buf, r->u.size.lower));
-        	xml_prop_printf(newxml, TAG_RULE_SIZE_UPPER,
-				"%s", uint64_to_string_buf(buf, sizeof buf, r->u.size.upper));
+        	xml_prop_printf(newxml, TAG_RULE_SIZE_LOWER, "%s", buf);
+			uint64_to_string_buf(r->u.size.upper, buf, sizeof buf);
+        	xml_prop_printf(newxml, TAG_RULE_SIZE_UPPER, "%s", buf);
 		}
         break;
     case RULE_JUMP:
@@ -853,24 +842,14 @@ rule_to_xml(xmlNodePtr parent, rule_t *r)
          */
         break;
     case RULE_SHA1:
-		{
-			gchar *name;
-			
-        	newxml = xml_new_empty_child(parent, NODE_RULE_SHA1);
+       	newxml = xml_new_empty_child(parent, NODE_RULE_SHA1);
 
-        	if (r->u.sha1.hash != NULL)
-            	xml_prop_set(newxml, TAG_RULE_SHA1_HASH,
-					sha1_base32(r->u.sha1.hash));
+       	if (r->u.sha1.hash != NULL)
+           	xml_prop_set(newxml, TAG_RULE_SHA1_HASH,
+				sha1_base32(r->u.sha1.hash));
 
-        	if (NULL == (name = locale_to_utf8_full(r->u.sha1.filename))) {
-				g_warning("rule_to_xml: Cannot convert filename to UTF-8. "
-					"Omitting filename (\"%s\")", r->u.sha1.filename);
-			} else {
-				xml_prop_set(newxml, TAG_RULE_SHA1_FILENAME, name);
-				if (name != r->u.sha1.filename)
-					G_FREE_NULL(name);
-			}
-		}
+		g_assert(utf8_is_valid_string(r->u.sha1.filename));
+		xml_prop_set(newxml, TAG_RULE_SHA1_FILENAME, r->u.sha1.filename);
         /*
          * r->u.sha1.hash is NULL, we just omit the hash.
          */
@@ -1012,7 +991,8 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
     xmlNodePtr node;
     search_t * search;
     guint flags = 0;
-	gboolean override;
+	guint lifetime;
+	time_t create_time;
 
 	(void) unused_udata;
     g_assert(xmlnode != NULL);
@@ -1021,7 +1001,6 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
 					NODE_SEARCH));
 
     gnet_prop_get_guint32_val(PROP_SEARCH_REISSUE_TIMEOUT, &reissue_timeout);
-	gnet_prop_get_boolean_val(PROP_ALLOW_AUTO_REQUERIES, &override);
 
 	buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_QUERY));
     if (!buf) {
@@ -1033,11 +1012,11 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
     buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_ENABLED));
     if (buf) {
         if (atoi(buf) == 1) {
-			flags |= SEARCH_ENABLED;
+			flags |= SEARCH_F_ENABLED;
 		}
         G_FREE_NULL(buf);
     } else
-		flags |= SEARCH_ENABLED;	 /* Compatibility: searches always began */
+		flags |= SEARCH_F_ENABLED;	 /* Compatibility: searches always began */
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_SPEED));
     if (buf) {
@@ -1054,7 +1033,7 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
     buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_PASSIVE));
     if (buf) {
         if (atol(buf) == 1)
-			flags |= SEARCH_PASSIVE;
+			flags |= SEARCH_F_PASSIVE;
         G_FREE_NULL(buf);
     }
 
@@ -1070,23 +1049,44 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
         G_FREE_NULL(buf);
     }
 
-	/**
-	 * LimeWire considers *any* form of requerying unacceptable.
-	 * Deactivate it for now.
-	 *		-- cbiere, 2005-03-22
-	 */
+	create_time = (time_t) -1;
+	buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_CREATE_TIME));
+    if (buf) {
+		create_time = date2time(buf, tm_time());
+		if (create_time == (time_t) -1)
+			g_warning("xml_to_search: Unparseable \"%s\" attribute.",
+				TAG_SEARCH_CREATE_TIME);
+        G_FREE_NULL(buf);
+    }
+		/* consider legacy searches as created right now */
+	if (create_time == (time_t) -1)
+		create_time = tm_time();
+
+	lifetime = (guint) -1;
+	buf = STRTRACK(xml_get_string(xmlnode, TAG_SEARCH_LIFETIME));
+    if (buf) {
+		gint error;
+		lifetime = parse_uint16(buf, NULL, 10, &error);
+		if (error)
+			g_warning("xml_to_search: Unparseable \"%s\" attribute.",
+				TAG_SEARCH_LIFETIME);
+        G_FREE_NULL(buf);
+	}
+	/* legacy searches get a 2 week expiration time */
+	lifetime = MIN(14 * 24, lifetime);
+
+	/* A zero lifetime means the search expired with the previous session */
+	if (0 == lifetime && 0 == (flags & SEARCH_F_PASSIVE))
+		flags &= ~SEARCH_F_ENABLED;
 
-	if (!(flags & SEARCH_PASSIVE) && !override)
-		flags &= ~SEARCH_ENABLED;
-	
     if (gui_debug >= 4) {
         g_message("adding new %s %s search: %s",
-			(flags & SEARCH_ENABLED) ? "enabled" : "disabled",
-			(flags & SEARCH_PASSIVE) ? "passive" : "active",
+			(flags & SEARCH_F_ENABLED) ? "enabled" : "disabled",
+			(flags & SEARCH_F_PASSIVE) ? "passive" : "active",
 			query);
 	}
 
-	search_gui_new_search_full(query, reissue_timeout,
+	search_gui_new_search_full(query, create_time, lifetime, reissue_timeout,
 		sort_col, sort_order, flags, &search);
 
     G_FREE_NULL(query);
@@ -1248,7 +1248,7 @@ xml_to_text_rule(xmlNodePtr xmlnode, gpointer data)
 static void
 xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
 {
-    guint32 addr;
+    host_addr_t addr;
     guint32 mask;
     gchar *buf;
     rule_t *rule;
@@ -1267,7 +1267,7 @@ xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
         g_warning("xml_to_ip_rule: rule without ip address");
 		return;
 	}
-	addr = gchar_to_ip(buf);	/* XXX: Needs validity check! */
+	addr = string_to_host_addr(buf, NULL); /* XXX: Needs validity check! */
     G_FREE_NULL(buf);
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_IP_MASK));
@@ -1275,7 +1275,15 @@ xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
         g_warning("xml_to_ip_rule: rule without netmask");
 		return;
 	}
-	mask = gchar_to_ip(buf);	/* XXX: Needs validity check! */
+	mask = string_to_ip(buf);
+	if (mask == 0) {
+		gint error;
+		mask = parse_uint16(buf, NULL, 10, &error);
+		/* XXX: Needs validity check! */
+	} else {
+		/* For backwards-compatibility */
+		mask = netmask_to_cidr(mask);
+	}
     G_FREE_NULL(buf);
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_TARGET));
